/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet");

/*
 * Imports 
 */
dojo.require("dojo.i18n");
dojo.require("dojo.string");  
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("jazz.app.context");
dojo.require("jazz.util.i18n");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.proxy");
dojo.require("net.jazz.ajax.dataServerRoot");

dojo.require("com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");

dojo.requireLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");

(function() {

/*
 * Aliases 
 */
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var I18nUtil = jazz.util.i18n;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ReportsPreferenceProvider = com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider;
var HoverPopup = jazz.ui.HoverPopup;

/*
 * Constants 
 */
var PREF_DTO_NOT_STARTED = 0;
var PREF_DTO_IN_PROGRESS = 1;
var PREF_DTO_SUCCESS = 2;
var PREF_DTO_FAILURE = 3;

var PREF_TIMEOUT = 10000; // 10 sec

var GROUP_PARAMETER = "com.ibm.team.reports.dto:ReportParameterGroupDTO";
var SINGLE_PARAMETER = "com.ibm.team.reports.dto:ReportParameterDTO";

var ONLY_MICRO = 1 << 5;

/*
 * Class 
 */
dojo.declare("com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet",
	Viewlet, {

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "ui/internal/templates/TrendReportViewlet.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 

	/*
 	 * @param Object args
 	 * 
 	 * @return TrendReportViewlet
 	 */
	constructor: function(args) {
		
		// Messages //--------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");

		// Icons // ----------------------------------------------------------->

		this._chevronDownSrc = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/icons/chevron_dn.gif");
		this._chevronUpSrc = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/icons/chevron_up.gif");
		
	},
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {

		// Static Variables //------------------------------------------------->

		this._initializeStaticCommons();

		// Instance Variables //----------------------------------------------->

		var site = this.getSite();
		if (!site.isRemote || site.isRemote()) {
			this._local = false;
		} else {
			this._local = true;
		}
		
		this._messageNoResults = this.getPreference(this._static._PREFERENCE_ID_NO_RESULTS_MESSAGE);
		
		this._savedQueryUUID = this.getPreference(this._static._PREFERENCE_ID_QUERY_UUID);
		this._transientQueryUUID = null;

		this._preferenceProvider = new ReportsPreferenceProvider({
			context: this
		});
		
		this._parameterHoverPopups = [];
		this._parametersAreaOpen = false;

		// Tracks project areas a report resource has been deployed to
		this._reportDeploymentCache = {};
		
		// Drill-in related variables
		this._titleHyperlinkURL = null; 
		this._serverRoot = null;
		this._serverURL = null;
		this._canViewDashboard = this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA);

		// Parameters area variables
		this._loadingTextNode = null;

		// Report preference variables
		this._prefDTOLoadingStatus = PREF_DTO_NOT_STARTED;
		this._prefDTOResult = null;
		this._prefTimeoutID = null;
		this._prefLabelCache = {};
		
		this._microRegex = null;
		
		// Report REST-related variable
		// TODO: Make a viewlet preference 
		this._showArchived = false;
		
		// Setup //------------------------------------------------------------>

		// FIXME: Asynchronous REST service call is not properly chained with callbacks. 
		this._getShowArchived();		
		
		this._initParametersArea();
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function(){
		
		if (this._isValidUUID(this._transientQueryUUID)) {
			com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet.deleteTransientReportQueryUUID(this._transientQueryUUID, this.id);
			delete this._transientQueryUUID;
		}
		
		this._deleteParameterHoverPopups();
		
		this._preferenceProvider.destroy();
		
		this.inherited(arguments);
		this._isDestroyed = true;
	},

	/*
 	 * @param String settingId
 	 * @param Object workingCopy
 	 * 
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(settingId, workingCopy) {
		if (settingId === "scopeItem") {
			// Changed: Project Area
			// Reset: Report, Query
			return [this._static._PREFERENCE_ID_REPORT, this._static._PREFERENCE_ID_QUERY_UUID];
		} else if (settingId === this._static._PREFERENCE_ID_REPORT) {
			// Changed: Report
			// Reset: Query
			return [this._static._PREFERENCE_ID_QUERY_UUID];
		}
		return [];
	},
	
	/*
 	 * @param String preferenceId
 	 * @param Object currentSettings
 	 * 
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceOptions: function(preferenceId, currentSettings) {
		// Preference: report
		if (preferenceId !== this._static._PREFERENCE_ID_REPORT) {
			// Wrong preference
			this._logError(new Error("Unexpected preference ID encountered"));
			return this._getPreferenceFailureList();
		}
		
		var projectAreaUUID = currentSettings.scopeItemId;
		if (!projectAreaUUID) {
			// User needs to select a project area first
			// Return failure
			return this._getPreferenceProjectAreaList();
		}
		
		if (this._prefDTOLoadingStatus === PREF_DTO_IN_PROGRESS) {
			// In Progress
			// Continue polling
			return null;
		} else if (this._prefDTOLoadingStatus === PREF_DTO_SUCCESS) {
			// Success
			this._resetPreferencePoll();
			// Check for results
			if (!this._prefDTOResult) {
				// No results
				return this._getPreferenceNoResultsList();
			}
			// Return results
			return this._prefDTOResult;
		} else if (this._prefDTOLoadingStatus === PREF_DTO_NOT_STARTED) {
			// Not started
			// -> In Progress
			this._prefDTOLoadingStatus = PREF_DTO_IN_PROGRESS;
			this._prefDTOResult = null;
			// Invoke REST service
			this._getReportDescriptors(projectAreaUUID);
			// Set a timeout to stop polling if the DTO is never received
			this._prefTimeoutID = setTimeout(dojo.hitch(this, function() {
				this._logError(new Error("Preference loading timed-out"));
				this._prefDTOLoadingStatus = PREF_DTO_FAILURE;
			}), PREF_TIMEOUT);		
			// Return null to start an interval-based polling of this method until the DTO is received.
			return null;
		}
		// Failure
		// PREF_DTO_FAILURE
		this._resetPreferencePoll();
		// Return failure
		return this._getPreferenceFailureList();
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_resetPreferencePoll: function() {
		this._prefDTOLoadingStatus = PREF_DTO_NOT_STARTED;
		clearTimeout(this._prefTimeoutID);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceFailureList: function() {
		var label = dojo.string.substitute(this.messages.msgError, [this.messages.msgPreferenceFailure]);
		var value = "";
		
		return this._getPreferenceList(label, value);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceProjectAreaList: function() {
		var label = this.messages.msgSelectProjectAreaFirst;
		var value = "";
		
		return this._getPreferenceList(label, value);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceNoResultsList: function() {
		var label = this.messages.msgNoReportsDeployed;
		var value = "";
		
		return this._getPreferenceList(label, value);
	},		
	
	/*
 	 * @param String label
 	 * @param String value
 	 * 
 	 * @return Object
 	 */		
	_getPreferenceList: function(label, value) {

		var result = {
			options: [{
				label: label,
				value: value
			}]
		};
		
		return result;
	},		
	
	/*
 	 * @param String projectAreaUUID
 	 * 
 	 * @return void
 	 */
	_getReportDescriptors: function(projectAreaUUID) {

		var serviceObject = {
			success: dojo.hitch(this, function(dto) {
				if (this._isDestroyed) {
					return;
				}
				this._processReportDescriptors(dto);
				this._prefDTOLoadingStatus = PREF_DTO_SUCCESS;
			}),
			failure: dojo.hitch(this, function(errorObject) {
				if (this._isDestroyed) {
					return;
				}
				this._logError(errorObject);
				this._prefDTOLoadingStatus = PREF_DTO_FAILURE;
			})
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTGetReportDescriptors(handler, projectAreaUUID);
	},		
	
	/*
 	 * @param Array dto
 	 * 
 	 * @return void
 	 */
	_processReportDescriptors: function(dto) {

		this._prefLabelCache = {};
		
		if (!(dto instanceof Array)) {
			// No results
			return;
		} else if (dto.length === 0) {
			// No results
			return;
		}
		
		var reportID = this.getPreference(this._static._PREFERENCE_ID_REPORT);
		
		var result = {
			options: [],
			computedDefault: reportID
		};		
		
		for (var index = 0; index < dto.length; index++) {
			var descriptor = dto[index]; 
			if (!descriptor) {
				this._logError(new Error("Undefined report descriptor encountered."));
				continue;
			}
			var label = descriptor.name;
			if (!label) {
				this._logError(new Error("Undefined report descriptor name encountered."));
				continue;
			}
			var value = descriptor.reportId;			
			if (!value) {
				this._logError(new Error("Undefined report descriptor ID encountered."));
				continue;
			}
			
			if (djConfig.locale.match(/^en/)) {
				// English (only)
				// Normalize label
				// Remove "Micro" prefix
				label = label.replace(this._getMicroRegex(), "$1");
			}
			
			result.options.push({
				label: label,
				value: value
			});
			
			// Cache labels by values & project area item IDs
			this._prefLabelCache[value] = label;
		}
		
		if (result.options.length === 0) {
			// No results
			return;
		}
		
		// Store result
		this._prefDTOResult = result;
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return RegExp
 	 */
	_getMicroRegex: function() {
		if (!this._microRegex) {
			var pattern = '^' + this.messages.msgMicro + '\\s+(.*)';
			this._microRegex = new RegExp(pattern, "i");
		}
		return this._microRegex;
	},		
	
	/*
 	 * @param Element node
 	 * @param boolean show
 	 * 
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			node.style.display = "block";	
		} else {
			node.style.display = "none";	
		}
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initParametersArea: function() {
		this.connect(this._chevronDownControl, "onclick", this._handleEventChevDownLinkClicked);
		this.connect(this._chevronUpControl, "onclick", this._handleEventChevUpLinkClicked);
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventChevDownLinkClicked: function(event) {
		dojo.stopEvent(event);
		
		this._addParameters();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventChevUpLinkClicked: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		
		this._updateVisibility(this._chevronDownControl, true);
		this._updateVisibility(this._chevronUpControl, false);
		this._updateVisibility(this._parameterTable, false);
		
		this._parametersAreaOpen = false;
		
		return true;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	update: function() {
		// Called to update UI (e.g. Zoom)
		this.getSite().refresh();
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getTitleHyperlinkURL: function() {
		
		// Check license
		if (!this._canViewDashboard) {
			// No Reports Web UI vertical
			return null;
		} else if (this._titleHyperlinkURL) {
			// Note: Changes to the project area, report ID or report query render this link stale
			return this._titleHyperlinkURL;
		}
		
		var queryUUID = this._getReportQueryUUID();
		var reportID = this.getPreference(this._static._PREFERENCE_ID_REPORT);

		if (this._local) {
			// Local viewlet
			if (reportID && this._isValidUUID(queryUUID)) {
				// Report query selected
				// Show report query results
				// e.g. 
				// #action=com.ibm.team.reports.viewReport
				// &queryUUID=_jCf14DWSEd6DNqVVoYmf3w
				// &reportUUID="workitems.OpenWorkItemsByType"
				// &projectAreaUUID=_VYHd8EVTEd6_vsgEcFTCrA
				this._titleHyperlinkURL = this._static._URI_VIEWREPORT_ACTION + 
											this._static._URI_COMP_QUERY + 
											queryUUID +
											this._static._URI_COMP_REPORT +
											this._getParentReportUUID(reportID) + 
											this._static._URI_COMP_PROJECT +
											this._getProjectAreaUUID();
			} else {
				// No report query selected
				// Show Reports welcome page
				this._titleHyperlinkURL = this._static._URI_VIEWPAGE_ACTION;
			}
		} else {
			// Remote viewlet
			if (reportID && this._isValidUUID(queryUUID)) {
				// Report query selected
				// Show report query results
				// e.g. 
				// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
				// #action=com.ibm.team.reports.viewReport
				// &queryUUID=_jCf14DWSEd6DNqVVoYmf3w
				// &reportUUID="workitems.OpenWorkItemsByType"
				// &projectAreaUUID=_VYHd8EVTEd6_vsgEcFTCrA
				this._titleHyperlinkURL = this._getServerURL() +
											this._static._URI_VIEWREPORT_ACTION + 
											this._static._URI_COMP_QUERY + 
											queryUUID +
											this._static._URI_COMP_REPORT +
											this._getParentReportUUID(reportID) + 
											this._static._URI_COMP_PROJECT +
											this._getProjectAreaUUID();
			} else {
				// No report query selected
				// Electing not to build a URL to the Reports welcome page if
				// the project area is specified because it is too much of an
				// edge case.
				// No drill-in
				this._titleHyperlinkURL = null;
			}
		}
		
		return this._titleHyperlinkURL;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return boolean
 	 * @overrides super
 	 */		
	hasZoomableContent: function() {
		if (this.getSite().isLoading()) {
			return false;
		}
		return true; 
	},

	/*
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	settingsChanged: function(oldSettings, newSettings) {
		// Viewlet settings area: "Save" button pressed

		// Project Area
		
		var newProjectArea = newSettings.scopeItemId;
		var oldProjectArea = oldSettings.scopeItemId;
		
		var scopeChanged;
		if (oldProjectArea === newProjectArea) {
			scopeChanged = false;
		} else {
			scopeChanged = true;
			this._serverURL = null;
			this._titleHyperlinkURL = null;
		}		
		
		// Report

		var newReportID = newSettings.preferences[this._static._PREFERENCE_ID_REPORT];
		var oldReportID = oldSettings.preferences[this._static._PREFERENCE_ID_REPORT];

		var reportChanged;
		if (newReportID === oldReportID) {
			reportChanged = false;
		} else {
			reportChanged = true;
			this._titleHyperlinkURL = null;
		}		
		
		// Query

		var newQueryUUID = newSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];
		var oldQueryUUID = oldSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];
		
		var queryChanged;
		if (newQueryUUID === oldQueryUUID) {
			queryChanged = false;
		} else {
			queryChanged = true;
			this._titleHyperlinkURL = null;
		}
		
		// Validate selections
		if (!newProjectArea || !newReportID || !newQueryUUID) {
			// Delete current transient query (if defined)
			// Note: Saved query will leak (if defined). See utilty method for details.
			this._setSavedReportQueryUUID(null);
			// Reset the title
			var title = this._getReportTitle(newReportID);
			this.getSite().setTitle(title, true);
			// Even if the viewlet is now unconfigured, let the refresh routine handle it accordingly
			this.getSite().refresh();
			return;
		}
		
		// Check to see if the report is deployed to the project area
		var deferred = this._isReportDeployed(newReportID, newProjectArea);
		
		deferred.addCallback(dojo.hitch(this, function(deployed) {
			if (deployed) {
				// Report deployed
				if ((newProjectArea && newReportID && newQueryUUID) &&
					(scopeChanged || reportChanged || queryChanged)) {
					// Defined: Project Area, Report, Query
					// Changed: Any
					// Save the report query in permanent storage
					this._resaveQuery(newQueryUUID, newReportID, dojo.hitch(this, function(queryUUID) {
						this._setSavedReportQueryUUID(queryUUID);
						this._updatePrimaryTitle(queryUUID, true);
						this.getSite().refresh();			
					}));
				}			
			} else {
				// Report not deployed
				this._displayActionReportNotDeployed(newReportID);	
			}
			return deployed;
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._displayErrorMessage(
					this.messages.msgWhitelistError, 
					this.messages.msgWhitelistError_explanation, 
					this.messages.msgWhitelistError_useraction, 
					errorObject);
			return errorObject;
		}));		
		
	},

	/*
 	 * @param Object canceledSettings
 	 * @param Object restoredSettings
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	settingsCanceled: function(canceledSettings, restoredSettings) {
		// Viewlet settings area:  "Cancel" button pressed
		
		var newQueryUUID = canceledSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];
		var oldQueryUUID = restoredSettings.preferences[this._static._PREFERENCE_ID_QUERY_UUID];

		if (newQueryUUID !== oldQueryUUID) {
			// Delete the canceled query
			this._deleteReportQueryUUID(newQueryUUID, true);
		}
	},

	/*
 	 * @param String reportID
 	 * 
 	 * @return String
 	 */	
	_getReportTitle: function(reportID) {

		if (reportID && this._prefLabelCache.hasOwnProperty(reportID)) {
			var title = this._prefLabelCache[reportID];
			if (title) {
				// Title found
				return title;
			}
		}
		// Title not found
		return this.getSite().getTitle();
	},	
	
	/*
 	 * @param String queryUUID
 	 * @param String reportID
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_resaveQuery: function(queryUUID, reportID, callback) {

		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			this._displayErrorMessage(
					this.messages.msgQuerySaveFailed,
					this.messages.msgQuerySaveFailed_explanation,
					this.messages.msgQuerySaveFailed_useraction,
					new Error("Project area is not defined"));
			return;
		}			

		var self = this;
		var serviceObject = {
			success: function(queryUUID2){
				if (self._isValidUUID(queryUUID2) === false) {
					self._displayErrorMessage(
							self.messages.msgQuerySaveFailed, 
							self.messages.msgQuerySaveFailed_explanation, 
							self.messages.msgQuerySaveFailed_useraction, 
							new Error(self.messages.msgInvalidQueryUUID));
				} else if (queryUUID !== queryUUID2) {
					self._displayErrorMessage(
							self.messages.msgQuerySaveFailed, 
							self.messages.msgQuerySaveFailed_explanation, 
							self.messages.msgQuerySaveFailed_useraction, 
							new Error(self.messages.msgQueryMismatchError));
				} else {
					callback(queryUUID2);
				}
			},
			failure: function(errorObject){
				self._displayErrorMessage(
						self.messages.msgQuerySaveFailed, 
						self.messages.msgQuerySaveFailed_explanation, 
						self.messages.msgQuerySaveFailed_useraction, 
						errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTPostSaveQueryDescriptor(
			handler, 
			reportID,
			projectAreaUUID,
			queryUUID);
	},		

	/*
 	 * @param String queryUUID
 	 * @param Boolean transientQuery
 	 * 
 	 * @return void
 	 */	
	_deleteReportQueryUUID: function(queryUUID, transientQuery) {

		var self = this;
		var serviceObject = {
			success: function() {
				if (transientQuery) {
					self._logInfo(new Error("Transient query deleted: " + queryUUID));
				} else {
					self._logInfo(new Error("Saved query deleted: " + queryUUID));
				}
			},
			failure: function(errorObject) {
				if (transientQuery) {
					self._logError(new Error("Failed to delete transient query: " + queryUUID));
				} else {
					self._logError(new Error("Failed to delete saved query:" + queryUUID));
				}
				self._logError(errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTPostDeleteQueryDescriptor(
			handler, 
			queryUUID,
			transientQuery);
	},	

	/*
 	 * @param String queryUUID
 	 * @param Boolean persist
 	 * 
 	 * @return void
 	 */	
	_updatePrimaryTitle: function(queryUUID, persist) {

		var queryLabel = this._preferenceProvider.getCachedQueryLabel(queryUUID);
		if (queryLabel) {
			this.getSite().setTitle(queryLabel, persist);
		}	
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticCommons: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._MAX_VISIBLE_PARAMETERS = 3;

		this._static._PREFERENCE_ID_REPORT = "report";	
		this._static._PREFERENCE_ID_QUERY_UUID = "reportQueryUUID";	
		this._static._PREFERENCE_ID_REPORT_PARENT = "reportParent";	
		this._static._PREFERENCE_ID_NO_RESULTS_MESSAGE = "noResultsMessage";	

		this._static._REPORT_REST_SERVICE_URI = "com.ibm.team.reports.common.internal.service.IReportRestService";
		
		this._static._PARAMETER_TYPE_TEAM = "TEAM";
		this._static._PARAMETER_TYPE_CATEGORY = "CATEGORY";
		this._static._PARAMETER_TYPE_DURATION = "DURATION";

		// TODO: Remove and devise a better report mapping story
		// 32 Micro Reports
		// Note: Do not change variable, otherwise may break QM viewlet.
		this._static._microReportIDMap = {
			"workitems.MicroAdvancedReleaseBurndown": "workitems.AdvancedReleaseBurndown",
			"workitems.MicroBlockingWorkItems": "workitems.BlockingWorkItems",
			"build.MicroBuildDuration": "build.BuildHealth",			
			"build.MicroBuildHealth": "build.BuildHealth",
			"workitems.MicroBurndown": "workitems.Burndown",			
			"workitems.MicroBurnup": "workitems.Burnup",
			"scm.MicroChangeSetActivity": "scm.ProjectActivity",
			"workitems.MicroClosedWorkItemsByPriority": "workitems.WorkItemsByPriority",
			"workitems.MicroClosedWorkItemsDaily": "workitems.ClosedWorkItemsDaily",
			"build.MicroCompilationWarnings": "build.CodeHealth",
			
			"workitems.MicroCumulativeValueFlow": "workitems.CumulativeValueFlow",
			"workitems.MicroDeferredWorkItems": "workitems.DeferredWorkItems",
			"workitems.MicroEstimatedVsActualWork": "workitems.EstimatedVsActualWork",
			"scm.MicroFileChangeActivity": "scm.ProjectActivity",
			"build.MicroFrequentTestFailures": "build.FrequentTestFailures",
			"build.MicroFrequentTestRegressions": "build.FrequentTestRegressions",
			"workitems.MicroNewWorkItemsBySeverity": "workitems.NewWorkItemsBySeverity",
			"workitems.MicroOpenVsClosedStoryPointsByIteration": "workitems.OpenVsClosedStoryPointsByIteration",
			"workitems.MicroOpenVsClosedWorkItems": "workitems.OpenVsClosedWorkItems",
			"workitems.MicroOpenWorkItems": "workitems.OpenWorkItems",
			
			"workitems.MicroOpenWorkItemsByPriority": "workitems.WorkItemsByPriority",
			"workitems.MicroOpenWorkItemsByType": "workitems.OpenWorkItemsByType",
			"workitems.MicroReleaseBurndown": "workitems.ReleaseBurndown",
			"workitems.MicroStoryPoints": "workitems.StoryPoints",
			"workitems.MicroStoryPointsByIteration": "workitems.StoryPointsByIteration",
			"workitems.MicroStoryPointsProgress": "workitems.StoryPointsProgress",
			"workitems.MicroStoryPointsRemaining": "workitems.StoryPointsRemaining",
			"scm.MicroStreamSizeCounts": "scm.StreamSizeCounts",
			"workitems.MicroTeamVelocity": "workitems.TeamVelocity",
			"build.MicroTestFailures": "build.CodeHealth",
			
			// "build.MicroTests": "No associated normal report",
			"workitems.MicroWorkItemComparison": "workitems.WorkItemComparison"
		};		
		
		// URL
		this._static._URI_VIEWPAGE_ACTION = "#action=jazz.viewPage&id=com.ibm.team.reports";
		this._static._URI_VIEWREPORT_ACTION = "#action=com.ibm.team.reports.viewReport";
		this._static._URI_COMP_QUERY = "&queryUUID=";
		this._static._URI_COMP_REPORT = "&reportUUID=";
		this._static._URI_COMP_PROJECT = "&projectAreaUUID=";

		// Special parameter values
		this._static._SPECIAL_CURRENT_ITERATION = "{Current Iteration}";
		this._static._SPECIAL_MY_TEAM_AREAS = "{My Team Areas}";
		this._static._SPECIAL_NO_SELECTION = "{No Selection}";
		
		// Dashboard drill-in URL components
		this._static._DASHBOARD_DRILL_ACTION_VIEW_TEAM = "#action=com.ibm.team.dashboard.viewDashboard&team=";
		this._static._DASHBOARD_DRILL_ACTION_VIEW_ALL = "#action=com.ibm.team.dashboard.viewAllDashboards";
		
		// Memento data
		this._static._MEMENTO_PARAMETERS = "parameters";
		
		// Content area parameter section
		this._static._UNKNOWN_PARAMETER = "(?)";
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		
		var reportID = this.getPreference(this._static._PREFERENCE_ID_REPORT);
		var projectAreaUUID = this._getProjectAreaUUID();
		
		if (!reportID) {
			// No Report selected
			this._showUnconfiguredReportMessage();
			return;
		} else if (!projectAreaUUID ||
			!this._getProjectAreaName()) {
			// No project area
			// Remote viewlet
			this._showUnconfiguredReportMessage();
			return;
		}
		
		// Check to see if the report is deployed to the project area
		var deferred = this._isReportDeployed(reportID, projectAreaUUID);
		
		deferred.addCallback(dojo.hitch(this, function(deployed) {
			if (deployed) {
				// Report deployed
				this._doRefresh();
			} else {
				// Report not deployed
				this._displayMsgReportNotDeployed(reportID);	
				this.getSite().doneRefresh();
			}
			return deployed;
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			this._displayErrorMessage(
					this.messages.msgWhitelistError, 
					this.messages.msgWhitelistError_explanation, 
					this.messages.msgWhitelistError_useraction, 
					errorObject);
			this.getSite().doneRefresh();
			return errorObject;
		}));

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_doRefresh: function() {
		
		// TEST
		//this._debugDefaultValues();

		var queryUUID = this._getReportQueryUUID();
		
		if (this._isValidUUID(queryUUID)) {
			this._getReport(queryUUID);
		} else if (this._hasPrimedData()) {
			this._getPrimedValuesReport();
		} else {
			this._getDefaultValuesReport();
		}

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_debugDefaultValues: function() {
		
		// e.g.
		//
		//<preference id="report" value="workitems.MicroBurndown"/>
		//<memento>
		//    <property name="parameters" type="xsd:string">
		//        <memento>
		//            <property name="name" type="xsd:string">ProjectAreaName</property>
		//            <property name="value" type="xsd:string">'{Current Project Area}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">TeamAreaName</property>
		//            <property name="value" type="xsd:string">'{My Team Areas}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">ProjectAreaName1</property>
		//            <property name="value" type="xsd:string">'{Current Project Area}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">Interval</property>
		//            <property name="value" type="xsd:string">'{Current Iteration}'</property>
		//        </memento>
		//        <memento>
		//            <property name="name" type="xsd:string">Zoom</property>
		//            <property name="value" type="xsd:string">false</property>
		//        </memento>
		//    </property>
		//</memento>		
		
		var self = this;
		var reportID = this.getPreference(this._static._PREFERENCE_ID_REPORT);
		
		this._getDefaultParameterValues(function(data) {
			console.log("= START ================================>");
			
			console.log('<preference id="report" value="' + reportID + '"/>');
			console.log('<memento>');
			
			console.log("\t" + '<property name="parameters" type="xsd:string">');
			
			for (var index = 0; index < data.length; index++) {
				var name = data[index].name;
				var value = self._normalizeValue(data[index].defaultValue);
				
				console.log("\t\t" + '<memento>');
				console.log("\t\t\t" + '<property name="name" type="xsd:string">' + name + '</property>');
				console.log("\t\t\t" + '<property name="value" type="xsd:string">' + value + '</property>');
				console.log("\t\t" + '</memento>');
			}

			console.log("\t" + '</property>');
			console.log('</memento>');
			console.log("= END ==================================>");
		});
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_showUnconfiguredReportMessage: function() {
		if (this._canEditDashboard()) {
			this._displayMsgSelectPreferences();
		} else {
			this._displayMessage(this.messages.msgUnconfiguredReport);
		}			
		this.getSite().doneRefresh();
	},	
	
	/*
 	 * @param String reportID
 	 * @param String projectAreaUUID
 	 * 
 	 * @return Deferred
 	 */		
	_isReportDeployed: function(reportID, projectAreaUUID) {
		
		var deferred = new dojo.Deferred();
		
		if (this._reportDeploymentCache.hasOwnProperty(reportID)) {
			// Result in cache
			var deployed = this._reportDeploymentCache[reportID].hasOwnProperty(projectAreaUUID);
			deferred.callback(deployed);
			return deferred;
		}
		// Result not in cache
		this._getGetContainingProjectAreas(reportID, projectAreaUUID, deferred);
		
		return deferred;
	},

	/*
 	 * @param String reportID
 	 * @param String projectAreaUUID
 	 * @param Deferred deferred
 	 * 
 	 * @return void
 	 */	
	_getGetContainingProjectAreas: function(reportID, projectAreaUUID, deferred) {
		
		var serviceObject = {
			success: dojo.hitch(this, function(list) {
				var deployed = this._processContainingProjectAreas(list, reportID, projectAreaUUID);
				deferred.callback(deployed);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				this._logError(errorObject);
				deferred.errback(errorObject);
			})
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");		
		
		this._callRESTGetContainingProjectAreaItemIds(handler, reportID);
	},	
	
	/*
 	 * @param Array list
 	 * @param String reportID
 	 * @param String projectAreaUUID
 	 * 
 	 * @return Boolean
 	 */	
	_processContainingProjectAreas: function(list, reportID, projectAreaUUID) {
		
		this._reportDeploymentCache[reportID] = {};
		
		if (!(list instanceof Array) ||
			(list.length === 0)) {
			// Report not contained in any project areas
			return false;
		}
		
		// Populate cache
		for (var index = 0; index < list.length; index++) {
			var projectAreaEntry = list[index]; 
			this._reportDeploymentCache[reportID][projectAreaEntry] = true;
		}
		
		var deployed = this._reportDeploymentCache[reportID].hasOwnProperty(projectAreaUUID);
		return deployed;
	},	
	
	/*
 	 * @param String preferenceId
 	 * 
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		return this._preferenceProvider;		
	},

	/*
	 * @param void
	 * 
	 * @return Boolean
	 */
	_canEditDashboard: function() {
		return this.getSite().isDashboardEditing();
	},		
	
	/*
	 * @param void
	 * 
	 * @return Boolean
	 */
	_hasPrimedData: function() {
		var data = this.getViewletState().get(this._static._MEMENTO_PARAMETERS);
		if (data) {
			return true;
		}
		return false;
	},	
	
	/*
	 * @param void
	 * 
	 * @return void
	 */
	_getPrimedValuesReport: function() {
		
		var data = this.getViewletState().get(this._static._MEMENTO_PARAMETERS);
		// TEST
		//console.debug(data);
			
		this._savePrimedValuesQuery(
			data,
			dojo.hitch(this, this._processTransientQuery));
	},		
	
	/*
 	 * @param Array data 
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_savePrimedValuesQuery: function(data, callback) {

		// TEST
		//console.debug(data);
		
		var normalizedData = this._normalizePrimedValuesData(data);
		
		var self = this;
		var serviceObject = {
			success: function(queryUUID) {
				callback(queryUUID);
			},
			failure: function(errorObject) {
				self._logError(errorObject);
				callback(null);
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		
		// Use preference provider's REST method
		this._preferenceProvider._callRESTPostSaveQueryDescriptor(
			handler, 
			this.getPreference(this._static._PREFERENCE_ID_REPORT), // Is defined
			this._getProjectAreaUUID(),								// Is defined
			this._getReportQueryUUID(),								// Is defined
			normalizedData.names, 
			normalizedData.values, 
			this.getSite().getTitle(),
			true); // keepInTempStorage		

	},
	
	/*
	 * @param Array data
	 * 
	 * @return Object
	 */
	_normalizePrimedValuesData: function(data) {
		
		var normalizedData = {
			names: [],
			values: []
		};
		
		for (var index = 0; index < data.length; index++) {
			var parameter = data[index]; 
			normalizedData.names.push(parameter.name);
			normalizedData.values.push(parameter.value);
		}
		
		return normalizedData;
	},	
	
	/*
	 * @param void
	 * 
	 * @return String
	 */
	_getReportQueryUUID: function() {
		if (this._savedQueryUUID) {
			return this._savedQueryUUID;
		} else if (this._transientQueryUUID) {
			return this._transientQueryUUID;
		}
		return "";
	},	
	
	/*
	 * @param String queryUUID
	 * 
	 * @return void
	 */
	_setSavedReportQueryUUID: function(queryUUID) {

		// BUG:
		// (1) Auto-save off
		// (2) Change query preference
		// (3) Save preferences
		// (4) Do not save Dashboard
		// (5) Reload page
		//     => New query not saved
		//     => Old query deleted
		//     => Error: Query descriptor not found  
		
		if (this._transientQueryUUID) {
			// Delete the transient query
			this._deleteReportQueryUUID(this._transientQueryUUID, true);
		} 
		//else if (this._savedQueryUUID) {
			// Delete the old saved query
			// BUG: Saved queries are leaking
			// Cannot delete the saved query because this viewlet may have been
			// duplicated from another viewlet still using that query.
			// See Work Item 85220 for details.
			//this._deleteReportQueryUUID(this._savedQueryUUID, false);
		//}

		// Set the new saved query
		// (Discard the transient or old saved query reference if applicable)
		this._savedQueryUUID = queryUUID;
		this._transientQueryUUID = null;
		
		// Remove primed data
		if (this._hasPrimedData()) {
			this.getViewletState().set(this._static._MEMENTO_PARAMETERS, null);
			this._logInfo(new Error("Parameters memento data deleted."));
		}
	},		
	
	/*
	 * @param String queryUUID
	 * 
	 * @return void
	 */
	_setTransientReportQueryUUID: function(queryUUID) {
		
		if (this._transientQueryUUID) {
			this._logError(new Error("Transient query UUID already exists"));
		} else if (this._savedQueryUUID) {
			this._logError(new Error("Saved query UUID already exists"));
		}
		
		this._transientQueryUUID = queryUUID;
	},		
	
	/*
	 * @param void
	 * 
	 * @return void
	 */
	_getDefaultValuesReport: function() {
		this._getDefaultValuesQuery(dojo.hitch(this, this._processTransientQuery));	
	},		

	/*
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_getDefaultValuesQuery: function(callback) {
		var self = this;
		this._getDefaultParameterValues(function(data) {
			self._saveDefaultValuesQuery(data, callback);
		});
	},	
	
	/*
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_getDefaultParameterValues: function(callback) {
		
		var self = this;
		var serviceObject = {
			success: function(baseParamsArray) {
				var data = self._processDefaultBaseParams(baseParamsArray);
				callback(data);
			},
			failure: function(errorObject) {
				self._logError(errorObject);
				callback(null);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		// Use preference provider's REST method
		this._preferenceProvider._callRESTGetParameters(
			handler, 
			this.getPreference(this._static._PREFERENCE_ID_REPORT), // Is defined
			this._getProjectAreaUUID());							// Is defined
	},	
	
	/*
 	 * @param Array baseParamsArray
 	 * 
 	 * @return Array
 	 */	
	_processDefaultBaseParams: function(baseParamsArray) {
		// TEST
		//console.debug(baseParamsArray);
		
		var data = [];
		
		if ((baseParamsArray instanceof Array) === false) {
			return data;
		}
		for (var index = 0; index < baseParamsArray.length; index++) {
			var baseParameter = baseParamsArray[index];
			
			if (!baseParameter) {
				this._logWarning(new Error("Base parameter not defined"));
			} else if (baseParameter._eQualifiedClassName === GROUP_PARAMETER) {
				this._processDefaultGroupBaseParam(baseParameter, data);
			} else if (baseParameter._eQualifiedClassName === SINGLE_PARAMETER) {
				this._processDefaultSingleBaseParam(baseParameter, data);
			} else {
				this._logWarning(new Error("Unrecognized qualified class name: " + baseParameter._eQualifiedClassName));
			}
		}
		
		return data;
	},	
	
	/*
 	 * @param Object baseParameter
 	 * @param Array data
 	 * 
 	 * @return void
 	 */	
	_processDefaultSingleBaseParam: function(baseParameter, data) {
		
		if (!baseParameter.name) {
			this._logWarning(new Error("Base parameter name not defined"));
			return;
		}
		
		// TEST
		//console.debug(baseParameter);
		
		// Add parameter
		data.push({
			name: baseParameter.name,
			required: baseParameter.required,
			defaultValue: baseParameter.defaultValue
		});
	},	
	
	/*
 	 * @param Object baseParameter
 	 * @param Array data
 	 * 
 	 * @return void
 	 */	
	_processDefaultGroupBaseParam: function(baseParameter, data) {
		
		if ((baseParameter.parameters instanceof Array) === false) {
			this._logWarning(new Error("Group has no parameters"));
			return;
		}
		
		for (var index = 0; index < baseParameter.parameters.length; index++) {
			var parameter = baseParameter.parameters[index];
			this._processDefaultSingleBaseParam(parameter, data);
		}
	},	
	
	/*
 	 * @param Array data 
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_saveDefaultValuesQuery: function(data, callback) {

		// TEST
		//console.debug(data);

		var normalizedData;
		try {
			normalizedData = this._normalizeDefaultValuesData(data);
		} catch (errorObject) {
			this._logInfo(errorObject);
			callback(null);
			return;
		}
		
		var self = this;
		var serviceObject = {
			success: function(queryUUID) {
				callback(queryUUID);
			},
			failure: function(errorObject) {
				self._logError(errorObject);
				callback(null);
			}
		};
		
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		
		// Use preference provider's REST method
		this._preferenceProvider._callRESTPostSaveQueryDescriptor(
			handler, 
			this.getPreference(this._static._PREFERENCE_ID_REPORT), // Is defined
			this._getProjectAreaUUID(),								// Is defined
			this._getReportQueryUUID(),								// Is defined		
			normalizedData.names, 
			normalizedData.values, 
			this.getSite().getTitle(),
			true); // keepInTempStorage		
		
	},	
	
	/*
	 * @param Array data
	 * 
	 * @return Object
	 * @throws Error
	 */
	_normalizeDefaultValuesData: function(data) {
		
		var normalizedData = {
			names: [],
			values: []
		};
		
		for (var index = 0; index < data.length; index++) {
			var parameter = data[index]; 
		
			if (parameter.required &&
				!parameter.defaultValue) {
				throw new Error("Required parameter has no default value: " + parameter.name);
			} 
			
			normalizedData.names.push(parameter.name);
			normalizedData.values.push(this._normalizeValue(parameter.defaultValue));
		}
		
		return normalizedData;
	},			
	
	/*
 	 * @param String value
 	 * 
 	 * @return String
 	 */	
	_normalizeValue: function(value) {
		
		if (!value) {
			// All
			return "";
		} else if (value === "true") {
			// Boolean
			return value;
		} else if (value === "false") {
			// Boolean
			return value;
		} else if (value.charAt(0) === "'") {
			// Quoted
			return value;
		}
		// TODO: MP: Worry about special variables? Strip {}
		
		return "'" + value + "'";
	},		
	
	/*
	 * @param String queryUUID
	 * 
	 * @return void
	 */
	_processTransientQuery: function(queryUUID) {
		
		if (this._isValidUUID(queryUUID)) {
			this._setTransientReportQueryUUID(queryUUID);
			this._getReport(queryUUID);
		} else if (this._canEditDashboard()) {
			this._displayMsgSelectPreferences();
			this.getSite().doneRefresh();
		} else {
			this._displayMessage(this.messages.msgUnconfiguredReport);
			this.getSite().doneRefresh();
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addParameters: function() {
		
		// TODO: MP: Add updateParameters method?
		// TODO: MP: Check if parameters / scope changed before re-adding parameters?
		// TODO: MP: POST 2.0: Add animation
		// TODO: MP: POST 2.0: Should add loading message
		
		// Note: Safe to get meta data since the report descriptor has already
		// been saved at this point
		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			this._logError(new Error("Project area is not defined"));
			return;
		}		
		
		var queryUUID = this._getReportQueryUUID();
		if (this._isValidUUID(queryUUID) === false) {
			this._logError(new Error(this.messages.msgInvalidQueryUUID));
			return;
		}
		
		// Show the loading message
		if(!this._parametersAreaOpen){// show loading only when Parameters area is closed
			this._loadingTextNode = document.createTextNode(this.messages.msgLoading);
			this._parametersArea.appendChild(this._loadingTextNode);
			dojo.addClass(this._parametersArea, "loading");
		}
		
		this._getParametersMetaData(
			projectAreaUUID, 
			queryUUID, 
			dojo.hitch(this, this._addParameterRows));
			
	},

	/*
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_getParametersMetaData: function(projectAreaUUID, queryUUID, callback) {
		
		var self = this;
		var serviceObject = {
			success: function(queryDescriptor) {
				self._getNamesMetaData(queryDescriptor, projectAreaUUID, callback);
			},
			failure: function(errorObject) {
				self._logError(errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		
		// Use preference provider's REST method
		this._preferenceProvider._callRESTGetQueryDescriptor(handler, queryUUID);		
	},	
	
	/*
 	 * @param Object queryDescriptor
 	 * @param String projectAreaUUID
 	 * @param Function callback
 	 * 
 	 * @return void
 	 */	
	_getNamesMetaData: function(queryDescriptor, projectAreaUUID, callback) {
		// TEST
		//console.debug(queryDescriptor);		
		
		if (!queryDescriptor) {
			this._logError(new Error("Query descriptor not defined"));
			return;
		} else if (!queryDescriptor.reportUUID) {
			this._logError(new Error("Report UUID not defined"));
			return;
		} else if (!projectAreaUUID) {
			this._logError(new Error("Project area is not defined"));
			return;
		} else if (queryDescriptor.engine.hasCustomParams) {
			//no parameter metadata; just return empty array
			callback(null);
		} else {
			var self = this;
			var serviceObject = {
					success: function(baseParamsArray) {
						var data = self._processBaseParams(queryDescriptor, baseParamsArray);
						callback(data);
					},
					failure: function(errorObject) {
						self._logError(errorObject);
					}
			};
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

			// Use preference provider's REST method
			this._preferenceProvider._callRESTGetParameters(handler, queryDescriptor.reportUUID, projectAreaUUID);
		}
	},	
	
	/*
 	 * @param Object queryDescriptor
 	 * @param Array baseParamsArray
 	 * 
 	 * @return Object
 	 */	
	_processBaseParams: function(queryDescriptor, baseParamsArray) {
		// TEST
		//console.debug(baseParamsArray);
		
		var data = {};
		
		if ((baseParamsArray instanceof Array) === false) {
			return data;
		}
		for (var index = 0; index < baseParamsArray.length; index++) {
			var baseParameter = baseParamsArray[index];
			
			if (!baseParameter) {
				this._logWarning(new Error("Base parameter not defined"));
			} else if (baseParameter._eQualifiedClassName === GROUP_PARAMETER) {
				this._processGroupBaseParam(queryDescriptor, baseParameter, data);
			} else if (baseParameter._eQualifiedClassName === SINGLE_PARAMETER) {
				this._processSingleBaseParam(queryDescriptor, baseParameter, data);
			} else {
				this._logWarning(new Error("Unrecognized qualified class name: " + baseParameter._eQualifiedClassName));
			}
		}
		
		return data;
	},	
	
	/*
 	 * @param Object queryDescriptor
 	 * @param Object baseParameter
 	 * @param Object data
 	 * 
 	 * @return void
 	 */	
	_processSingleBaseParam: function(queryDescriptor, baseParameter, data) {
		
		if (!baseParameter.name) {
			this._logWarning(new Error("Base parameter name not defined"));
			return;
		} else if (baseParameter.concealed) {
			return;
		} else if ((queryDescriptor.parameterNames instanceof Array) === false) {
			this._logWarning(new Error("Parameter names not defined"));
			return;
		} else if ((queryDescriptor.parameterValues instanceof Array) === false) {
			this._logWarning(new Error("Parameter values not defined"));
			return;
		} else if (queryDescriptor.parameterNames.length !== queryDescriptor.parameterValues.length) {
			this._logWarning(new Error("Parameter name count does not equal parameter value count"));
			return;
		}
		
		var index = 0;
		
		// Get values
		var rawValues = null;
		for (index = 0; index < queryDescriptor.parameterNames.length; index++) {
			// e.g.
			// TeamAreaName
			// WhichBuilds
			var parameterName = queryDescriptor.parameterNames[index];
			if (parameterName === baseParameter.name) {
				// e.g.
				// "'/Jazz Development/Reports','/Jazz Development/Repository','/Jazz Development/Work Item'"
				// "'1 Last 10 Builds'"
				// ""
				// "false"
				rawValues = queryDescriptor.parameterValues[index];
				break;
			}
		}
		
		if (typeof(rawValues) !== "string") {
			this._logWarning(new Error(
				"Report resource parameter '" + 
				baseParameter.name + 
				"' not found in the report query. The report resource (new) is out of sync with the report (old)."));
			return;
		}
		
		var values;
		if (rawValues.length === 0) {
			// Empty string "" (All)
			values = [""];
		} else {
			// Split values (if more than one value)
			values = rawValues.split(",");
			// Remove single quotes (if any)
			var pattern = /'(.*)'/; 
			for (index = 0; index < values.length; index++) {
				var matches = values[index].match(pattern);
				if ((matches instanceof Array) &&
					(matches.length > 1)) {
					values[index] = matches[1];
				}
			}
		}
		
		var displayName;
		if (baseParameter.displayName) {
			displayName = baseParameter.displayName;
		} else {
			this._logInfo(new Error("No display name: " + baseParameter.name));
			displayName = baseParameter.name;
		}
		
		// Add parameter
		data[baseParameter.name] = {
			displayName: displayName,
			values: values
		};
		
	},	

	/*
 	 * @param Object queryDescriptor
 	 * @param Object baseParameter
 	 * @param Array data
 	 * 
 	 * @return void
 	 */	
	_processGroupBaseParam: function(queryDescriptor, baseParameter, data) {
		
		if ((baseParameter.parameters instanceof Array) === false) {
			this._logWarning(new Error("Group has no parameters"));
			return;
		}
		
		for (var index = 0; index < baseParameter.parameters.length; index++) {
			var parameter = baseParameter.parameters[index];
			this._processSingleBaseParam(queryDescriptor, parameter, data);
		}
	},	
	
	/*
 	 * @param Object data
 	 * 
 	 * @return void
 	 */	
	_addParameterRows: function(data) {

		this._deleteParameterHoverPopups();
		
		// Hide the loading message
		if (this._loadingTextNode) {
			this._parametersArea.removeChild(this._loadingTextNode);
			this._loadingTextNode = null;
		}
		dojo.removeClass(this._parametersArea, "loading");
		
		this._updateVisibility(this._chevronDownControl, false);	
		this._updateVisibility(this._chevronUpControl, true);
		this._updateVisibility(this._parameterTable, true);
		
		this._parametersAreaOpen = true;
		
		var newTableBody = document.createElement("tbody");
		dojo.addClass(newTableBody, "parameter-rows");
		newTableBody.id = this._getParameterRowsID();

		// Project Area
		this._addProjectAreaRow(newTableBody);
		
		// TEST
		// console.debug(data);
		
		for (var name in data) {
			var parameter = data[name];
			
			// Row
			var row = document.createElement("tr");		

			// Column 1
			var col1 = document.createElement("td");
			dojo.addClass(col1, "label");
			col1.innerHTML = parameter.displayName + ":";

			// Column 2
			var col2 = document.createElement("td");
			dojo.addClass(col2, "value");
			
			//fetch parameter labels from the server
			var self = this;
			
			//wrap this function inside a "closure-providing" function to correctly scope the "td" 
				//and "name" variables inside the loop
			var closureFunction = function(name, td) {
				return function(labels) {
					var valueNode = self._getFormattedParameterValues(name, labels);
					td.appendChild(valueNode);
				};
			};
			var obj = {
				success: closureFunction(name, col2),
				
				failure: function(errorObject) {
					self._logError(errorObject);
				}
			};
			
			var srh = new ServiceResponseHandler(obj, "success", "failure");
			this._callRESTGetParameterLabels(srh, parameter.values);			

			// Add columns to row
			row.appendChild(col1);
			row.appendChild(col2);

			// Add row to table
			newTableBody.appendChild(row);
		}

		var oldTableBody = dojo.byId(this._getParameterRowsID());		
		
		try {
			if (oldTableBody) {
				this._parameterTable.replaceChild(newTableBody, oldTableBody);
			} else {
				this._parameterTable.appendChild(newTableBody);
			}
		} catch (errorObject) {
			this._logError(errorObject);
			this._removeAllChildren(this._parameterTable);
			this._parameterTable.appendChild(newTableBody);
		}
	},

	/*
 	 * @param Element tableBody
 	 * 
 	 * @return void
 	 */	
	_addProjectAreaRow: function(tableBody) {

		// Row
		var row = document.createElement("tr");		

		// Column 1
		var col1 = document.createElement("td");
		dojo.addClass(col1, "label");
		col1.innerHTML = this.messages.msgProjectArea + ":";
		// Column 2
		var col2 = document.createElement("td");
		dojo.addClass(col2, "value");
		col2.innerHTML = this._getProjectAreaName();

		// Add columns to row
		row.appendChild(col1);
		row.appendChild(col2);
		
		// Add row to table
		tableBody.appendChild(row);
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getServerRoot: function() {
		
		if (typeof(this._serverRoot) === "string") {
			return this._serverRoot;
		}
		
		// e.g. Data server root:
		// Local:   undefined
		// Remote:  https://127.0.0.1:9444/jazz
		var root = net.jazz.ajax.dataServerRoot();
		if (!root) {
			// e.g. Context root:
			// Local:   /jazz
			// Remote:  https://127.0.0.1:9444/jazz 
			root = dojo.getObject("net.jazz.ajax._contextRoot");
		}
		if (!root) {
			root = "";
		}
		this._serverRoot = root;
		
		return this._serverRoot;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */
	_getServerURL: function() {
		// e.g.
		// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
		
		if (this._serverURL) {
			return this._serverURL;
		}
		
		var url = this._getServerRoot() + 
					"/web/projects/" +
					encodeURIComponent(this._getProjectAreaName());
		
		if (djConfig.isDebug) {
			url = url + "?debug=true";
		}

		this._serverURL = url;
		
		return this._serverURL;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 * 				String name
 	 * 				String itemId
 	 */	
	_getProjectArea: function() {
		// Report via saved query
		var scope = this.getScopeItem();
		if (scope) {
			return scope;
		}
		// Report via primed values
		// Report via default values
		if (this._local) {
			// Local viewlet
			return jazz.app.context.get();
		}
		// Remote viewlet
		return null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getProjectAreaName: function() {
		var projectArea = this._getProjectArea();
		if (projectArea &&
			projectArea.name) {
			return projectArea.name;
		} 
		return null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getProjectAreaUUID: function() {
		var projectArea = this._getProjectArea();
		if (projectArea &&
			projectArea.itemId) {
			return projectArea.itemId;
		} 
		return null;
	},

	/*
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getFormattedParameterValues: function(name, values) {
		
		if (values.length <= this._static._MAX_VISIBLE_PARAMETERS) {
			return this._getCustomFormatParameterValues(name, values);
		}

		var subsetValues = values.slice(0, this._static._MAX_VISIBLE_PARAMETERS);
		var formattedValuesElement = this._getCustomFormatParameterValues(name, subsetValues);
		this._addParameterEllipses(formattedValuesElement, name, values);
		
		return formattedValuesElement;
	},

	/*
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return Element
 	 */	
	_getCustomFormatParameterValues: function(name, values) {
		var content = document.createElement("div");
		var specialType = this._getParameterType(name);
		
		if (values.length === 1) {
			var rawValue = values[0];

			if (!rawValue) {
				if (specialType === this._static._PARAMETER_TYPE_TEAM) {
					// All
					var allText = document.createTextNode(this.messages.msgAll);
					var allLink = document.createElement("a");
					// e.g.
					// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
					// #action=com.ibm.team.dashboard.viewAllDashboards
					allLink.href = this._getServerURL() + this._static._DASHBOARD_DRILL_ACTION_VIEW_ALL;
					allLink.appendChild(allText);
					content.appendChild(allLink);			
					return content;
				} else {			
					// All
					var allText = document.createTextNode(this.messages.msgAll);
					content.appendChild(allText);			
					return content;
				}			
			}
		}		
		
		// List		
		for (var index = 0; index < values.length; index++) {
			
			var rawValue = values[index];
			var node;
			
			if (!rawValue) {
				// Error
				node = document.createTextNode(this._static._UNKNOWN_PARAMETER);
			} else if (rawValue === this._static._SPECIAL_MY_TEAM_AREAS) {
				// My Team Areas
				node = document.createTextNode(this.messages.msgMyTeamAreas);
			} else if (rawValue === this._static._SPECIAL_CURRENT_ITERATION) {
				node = document.createTextNode(this.messages.msgCurrentIteration);
			} else if (rawValue === this._static._SPECIAL_NO_SELECTION) {
				node = document.createTextNode(this.messages.msgNoSelection);
			} else if (specialType === this._static._PARAMETER_TYPE_TEAM){
				// Display last segment only
				// e.g. 
				// Before: /Jazz Development/Dashboard
				// After:  Dashboard
				var segments = rawValue.split(/\//);
				var displayValue = segments[segments.length - 1];
				var displayText = document.createTextNode(displayValue);
				
				node = document.createElement("a");
				node.title = rawValue;
				node.href = this._getTeamDashboardURL(rawValue);
				node.appendChild(displayText);
			} else if (specialType === this._static._PARAMETER_TYPE_CATEGORY){
				// Display last segment only
				// e.g. 
				// Before: Work Items/Web UI
				// After:  Web UI
				node = document.createElement("span");
				var segments = rawValue.split(/\//);
				displayValue = segments[segments.length - 1];
				node.title = rawValue;
				
				var displayText = document.createTextNode(displayValue);
				node.appendChild(displayText);
				content.appendChild(node);				
			} else if (specialType === this._static._PARAMETER_TYPE_DURATION) {
				// Remove initial digit
				// e.g. Before: 7 Last 50 Builds
				// e.g. After:  Last 50 Builds
				var matches = rawValue.match(/^[0-9]+ (.+)/);
				if ((matches instanceof Array) &&
					(matches.length > 1) &&
					matches[1]) {
					// [0]: Entire match
					// [1]: First match group
					displayValue = matches[1];
				} else {
					displayValue = rawValue;
				}
				
				node = document.createTextNode(displayValue);
			}  else if (rawValue === 'false') {
				// Boolean
				node = document.createTextNode(this.messages.msgNo);
			}  else if (rawValue === 'true') {
				// Boolean
				node = document.createTextNode(this.messages.msgYes);
			} else {
				node = document.createTextNode(rawValue); 
			}
			
			content.appendChild(node);

			// Comma separated
			if (index !== (values.length - 1)) {
				var comma = document.createTextNode(", ");
				content.appendChild(comma);
			}
		}	
		
		return content;
	},

	/*
 	 * @param String rawTeamArea
 	 * 
 	 * @return String
  	 */	
	_getTeamDashboardURL: function(rawTeamArea) {
		
		// Replace the first forward slash (if any)
		// e.g.
		// Before: /RTC Development/Repository/Gateway
		// After:  RTC Development/Repository/Gateway
		var teamArea = rawTeamArea.replace(/^\//, "");
		
		// Encode component parts
		// e.g.
		// Before: RTC Development/Repository/Gateway
		// After:  RTC%20Development/Repository/Gateway
		var delimeter = "/";
		var segments = teamArea.split(delimeter);
		for (var index = 0; index < segments.length; index++) {
			segments[index] = encodeURIComponent(segments[index]);
		}		
		var teamDashboard = segments.join(delimeter);
		
		// e.g.
		// https://localhost:9443/jazz/web/projects/Rational%20Team%20Concert?debug=true
		// #action=com.ibm.team.dashboard.viewDashboard&team=RTC%20Development/Repository/Gateway
		var url = this._getServerURL() + 
					this._static._DASHBOARD_DRILL_ACTION_VIEW_TEAM + teamDashboard; 

		return url;
	},
	
	/*
	 * @param String parameterName
	 */
	_getParameterType: function(parameterName) {
		//this is not very robust, since it depends on certain hard-coded parameter strings.
		//is there a better way?
		if (parameterName.indexOf("TeamArea") != -1) {
			// Team Area
			return this._static._PARAMETER_TYPE_TEAM;
		} else if (parameterName.indexOf("Category") != -1) {
			// Category
			return this._static._PARAMETER_TYPE_CATEGORY;
		} else if (parameterName.indexOf("WhichBuilds") != -1) {
			// Duration
			return this._static._PARAMETER_TYPE_DURATION;
		}
	},

	/*
 	 * @param Element content
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return void
 	 */	
	_addParameterEllipses: function(content, name, values) {

		var span = document.createElement("span");
		var comma = document.createTextNode(", ");
		var ellipses = document.createTextNode("...");
		
		span.appendChild(ellipses);
		content.appendChild(comma);
		content.appendChild(span);
		
		this._parameterHoverPopups.push(
					new HoverPopup({
						around: span,
						html: this._getParameterHoverContent(name, values),
						orientation: "R",
						openDelay: 300}));
	},

	/*
 	 * @param String name
 	 * @param Array values
 	 * 
 	 * @return String
 	 */	
	_getParameterHoverContent: function(name, values) {

		var content = "";
		
		var parameterType = this._getParameterType(name);

		if (parameterType === this._static._PARAMETER_TYPE_TEAM) {
			// Team Area
			content = this._getHierarchicalHoverContent(values, true);
		} else if (parameterType === this._static._PARAMETER_TYPE_CATEGORY) {
			// Category
			content = this._getHierarchicalHoverContent(values, false);
		} else {
			// Other
			for (var i = 0; i < values.length; i++) {
				var str;
				if (values[i] === this._static._SPECIAL_CURRENT_ITERATION) {
					str = this.messages.msgCurrentIteration;
				} else {
					str = values[i];
				}
				
				content = content + str;
				
				if (i < values.length - 1) {
					content = content + "<br>";
				}			
			}
		}

		var html = "<span style=\"font-size:8pt;\">" + content + "</span>";
			
		return html;
	},

	/*
 	 * @param Array values
 	 * @param Boolean showLinks
 	 * 
 	 * @return String
 	 */	
	_getHierarchicalHoverContent: function(values, showLinks) {

		// Split each value into path segments
		var sortedValues = this._sortRawTeamValues(values);
		var valueMatrix = [];
		var index;
		var maxColumnCount = 0;
		for (index = 0; index < sortedValues.length; index++) {
			valueMatrix[index] = sortedValues[index].split(/\//);
			if (valueMatrix[index].length > maxColumnCount) {
				maxColumnCount = valueMatrix[index].length;
			}
		}

		// Normalize values into a hierarchy by replacing matching values
		// with whitespace
		var targetValue = ""; 
		var tabCount = 0;
		var whitespace = "&nbsp;&nbsp;&nbsp;";
		for (var columnY = 0; columnY < maxColumnCount; columnY++) {
			for (var rowX = 0; rowX < sortedValues.length; rowX++) {

				if (columnY >= valueMatrix[rowX].length) {
					continue;
				}

				var currentValue = valueMatrix[rowX][columnY];

				if ((currentValue === targetValue) &&
					this._isSameParentValue(valueMatrix, rowX, columnY)) {
					// Replace value with whitespace padding to represent value's
					// proper level within the hierarchy
					valueMatrix[rowX][columnY] = this._getRepeatedString(whitespace, tabCount);
				} else {
					// Set as value to match
					targetValue = currentValue;
					// If this is not the last segment of the path, display it
					// as an unselected value on its own row in the hierarchy
					if (columnY !== (valueMatrix[rowX].length - 1)) {
						valueMatrix[rowX][columnY] = "<span style=\"color:gray;\">" + 
													valueMatrix[rowX][columnY] +
													"</span><br>" + 
													this._getRepeatedString(whitespace, (tabCount + 1));
						tabCount--;
					}
				}
			}
			tabCount++;
		}

		// Collapse value matrix into assembled content
		var content = "";

		if (showLinks) {
			for (index = 0; index < valueMatrix.length; index++) {
				var lastIndex = valueMatrix[index].length - 1;
				var indent;
				if (lastIndex > 0) {
					// Note: Does not include the last entry
					indent = valueMatrix[index].slice(0, lastIndex).join("");
				} else {
					indent = "";
				}
				
				var text;
				if (valueMatrix[index].length === 1 
						&& valueMatrix[index][0] === this._static._SPECIAL_MY_TEAM_AREAS) {
					// My Team Areas
					text = this.messages.msgMyTeamAreas;
				} else {
					// Turn the last entry into a link
					text = "<a href=\"" + 
								this._getTeamDashboardURL(sortedValues[index]) + 
								"\">" +
								valueMatrix[index][lastIndex] +
								"</a>";

				}				
						
				content = content + indent + text + "<br>";
			}
		} else {
			for (index = 0; index < valueMatrix.length; index++) {
				content = content + valueMatrix[index].join("") + "<br>";
			}
		}		

		return content;
	},

	/*
 	 * @param Array valueMatrix
 	 * @param Number rowX
 	 * @param Number columnY
 	 *  
 	 * @return Boolean
 	 */	
	_isSameParentValue: function(valueMatrix, rowX, columnY) {
		// Pre: valueMatrix is sorted
		// Pre: rowX is a positive integer
		// Pre: rowY is a positive integer
		if (rowX === 0) {
			return true;
		} else if (columnY === 0) {
			return true;
		} 
		// e.g. rowX = 1; columnY = 1 
		// Not same parent
		//    0       1
		// 0: Admin / Web UI
		// 1: Build / Web UI
		var currentParent = valueMatrix[rowX][columnY - 1];
		var previousParent = valueMatrix[rowX - 1][columnY - 1];
		if (currentParent === previousParent) {
			return true;
		}
		return false;
	},

	/*
 	 * @param Array values
 	 * 
 	 * @return Array
 	 */	
	_sortRawTeamValues: function(values) {
		var sortedValues = [];
		// Remove leading '/' (if any)
		for (var index = 0; index < values.length; index++) {
			sortedValues.push(values[index].replace(/^\//, ""));
		}
		// Sort array
		sortedValues.sort();
		
		return sortedValues;
	},

	/*
 	 * @param String text
 	 * @param Number count
 	 * 
 	 * @return String
 	 */	
	_getRepeatedString: function(text, count) {
		var result = text;
		for (var index = 1; index < count; index++) {
			result = result + text;
		}
		return result;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_deleteParameterHoverPopups: function() {
		
		for (var index = 0; index < this._parameterHoverPopups.length; index++) {
			var widget = this._parameterHoverPopups[index];
			if (widget) {
				widget.destroy();
			}
		}
		this._parameterHoverPopups = [];
	},

	/*
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */
	_getReport: function(queryUUID) {

		var projectAreaUUID = this._getProjectAreaUUID();
		if (!projectAreaUUID) {
			this._displayErrorMessage(
					this.messages.msgReportLoadingFailed,
					this.messages.msgReportLoadingFailed_explanation,
					this.messages.msgReportLoadingFailed_useraction,
					new Error("Project area is not defined"));
			this.getSite().doneRefresh();
			return;
		}

		// Update parameters
		if (this._parametersAreaOpen) {
			this._addParameters();
		}
		
		// Show the loading message		
		this._updateLoadingMessage(this.messages.msgLoading);
		this._showLoadingArea();
		
		// Remember the zoom state at the time of this request
		var previousZoomState = this.isZoomed();
		
		var self = this;
		var serviceObject = {
			success: function(html) {
				self._processReportHTML(html, previousZoomState);
				self.getSite().doneRefresh();
			},
			failure: function(errorObject) {
				self._displayErrorMessage(
						self.messages.msgReportLoadingFailed, 
						self.messages.msgReportLoadingFailed_explanation, 
						self.messages.msgReportLoadingFailed_useraction, 
						errorObject);
				self.getSite().doneRefresh();
			}
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTPostRenderQuery(
			handler,
			projectAreaUUID,
			queryUUID,
			['Zoom'],
			[previousZoomState]);

	},	

	/*
 	 * @param void
 	 * 
 	 * @return dojo.Deferred
 	 */
	_getShowArchived: function() {
		var deferred = new dojo.Deferred();		
		var serviceObject = {
			success: dojo.hitch(this, function(result) {
				this._showArchived = result;
				deferred.callback(result);
			}),
			failure: dojo.hitch(this, function(errorObject) {
				this._logError(errorObject);
				deferred.errback(errorObject);
			})
		};

		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTGetShowArchived(handler);
		
		return deferred;
	},		
	
	/*
 	 * @param String html
 	 * 
 	 * @return void
 	 */	
	_addReportToViewlet: function(html) {
		// Update loading message
		this._updateLoadingMessage("");
		// Get the attach point
		var node = this._displayArea;
		// Add the report HTML to the node
		this._addHTMLToNode(html, node);
	},	

	/*
 	 * @param String html
 	 * @param Element node
 	 * 
 	 * @return void
 	 */	
	_addHTMLToNode: function(html, node) {
		// Create a wrapping div for the parameters and report nodes
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());	
		
		// Show the parameters area
		this._updateVisibility(this._parametersArea, true);

		// Create the report node
		var reportNode = document.createElement("div");
		// Set the content to be the html
		reportNode.innerHTML = html;
		// Add the report node
		container.appendChild(reportNode);
		
		// Add the wrapping div to the node
		this._addReportNode(node, container);
		// Show the display area
		this._showDisplayArea();		
	},		

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getReportDomID: function() {
		return "report:" + this.id;	
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getParameterRowsID: function() {
		return "params:" + this.id;	
	},	

	/*
 	 * @param Element targetParent
 	 * @param Element newReportNode
 	 * 
 	 * @return void
 	 */		
	_addReportNode: function(targetParent, newReportNode) {
		// Get the old report node (if any)
		var oldReportNode = dojo.byId(this._getReportDomID());		
		
		try {
			if (oldReportNode) {
				targetParent.replaceChild(newReportNode, oldReportNode);
			} else {
				targetParent.appendChild(newReportNode);
			}
		} catch (errorObject) {
			this._logError(errorObject);
			this._removeAllChildren(targetParent);
			targetParent.appendChild(newReportNode);
		}
	},	

	/*
 	 * @param Node parent
 	 * 
 	 * @return void
 	 */	
	_removeAllChildren: function(parent) {
		while (parent.hasChildNodes()) {
			parent.removeChild(parent.firstChild);
		}
	},

	/*
 	 * @param String reportID
 	 * 
 	 * @return String
 	 */	
	_getParentReportUUID: function(reportID) {
		// Get the associated normal report UUID
		
		// From hard-coded mapping
		if (this._static._microReportIDMap.hasOwnProperty(reportID)) {
			return this._static._microReportIDMap[reportID];
		}
		
		// From extension		
		var parentReportUUID = this.getPreference(this._static._PREFERENCE_ID_REPORT_PARENT);
		if (parentReportUUID) {
			return parentReportUUID;
		}

		// Use micro report ID
		return reportID;
	},	

	/*
 	 * @param String html
 	 * @param String hashkey
 	 * @param boolean previousZoomState
 	 * 
 	 * @return boolean
 	 */	
	_processReportHTML: function(html, previousZoomState) {
		// Check if a report was found
		if (previousZoomState != this.isZoomed()) {
			// NO-OP
			// If the zoom state made at request time is different than 
			// the zoom state on request completion (now), do not add the 
			// report (but still cache it)
			this._logInfo(new Error("Report results discarded. Zoom state changed between request initiation and completion."));
		} else if (this._containsReport(html)) {
			// Report found
			if (!this._canViewDashboard) {
				// TODO: MP: Disable drill-in links via API
				// No Reports Web UI vertical
				// Disable chart drill-in
				// Note: Only applicable to Build Duration report (at the moment)
				html = this._removeHrefs(html);
			}
			var preProcessedHTML = this.preProcessReportHTML(html);
			// Add the report to the display area
			this._addReportToViewlet(preProcessedHTML);
		} else {
			// Report NOT found
			this._addReportToViewlet(this._getHTMLNoResultsMessage());
		}
	},	

	/*
 	 * Pre-process the report content before it is embedded in the viewet
 	 * content area.
 	 * 
 	 * @param String html: The report content as an HTML string.
 	 * 
 	 * @return String 
 	 */	
	preProcessReportHTML: function(html) {
		// Subclasses to override 
		return html;
	},

	/*
 	 * Remove all target attributes from the report content.
 	 * Anchor targets break links contained within an iframe (e.g. Remote viewlet content)
 	 * 
 	 * BEFORE
 	 * <a target="_self" href="...">...</a>
 	 * AFTER
 	 * <a href="...">...</a>
 	 * 
 	 * @param String html: The report content as an HTML string. 
 	 * 
 	 * @return String 
 	 */	
	removeAnchorTargets: function(html) {
		var result = html.replace(/target=\".*?\"/g, "");
		return result;		
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getHTMLNoResultsMessage: function() {
		var html = "<div class=\"no-results\">" + this._messageNoResults + "</div>";
		return html;
	},

	/*
 	 * @param String html
 	 * 
 	 * @return String
 	 */	
	_removeHrefs: function(html) {
		// Before
		// <area title="4.838" target="_self" href="#action=com.ibm.team.reports..." coords="91,139,91,206,100,206,100,139" shape="poly"/>	
		// After
		// <area title="4.838" target="_self" coords="91,139,91,206,100,206,100,139" shape="poly"/>		

		var result = html.replace(/href=\".*?\"/g, "");

		return result;
	},	

	/*
 	 * @param String html
 	 * 
 	 * @return boolean
 	 */	
	_containsReport: function(html) {
		var matcher = new String(html);
		if (matcher.match(/\.png/)) {
			// Image results
			return true;
		} else if (this._containsReportTable(matcher)) {
			// Table results
			return true;
		} else if (matcher.match(/iframe/)) { //for RRDI reports
			return true;
		}
		return false;
	},		

	/*
 	 * @param String htmlString
 	 * 
 	 * @return boolean
 	 */	
	_containsReportTable: function(htmlString) {
		// Look for tables in the result
		var matches = htmlString.match(/<table/g);
		if ((matches instanceof Array) &&
					(matches.length > 1)) {
			// Hacky solution
			// If there is more than one table, there are results
			return true;
		}
		return false;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getImageServerRoot: function() {
		var url;
		if (this._local) {
			// Local viewlet
			url = this._getServerRoot();
		} else {
			// Remote viewlet
			// Create a proxy url in order to embed images from another 
			// domain in the current domain.
			// Absolute links on their own work; however, security related
			// issues (e.g. firewall, invalid certificates, etc.) prevent the 
			// images from resolving in test environments.
			url = jazz.app.proxy.encodeProxiedURI(this._getServerRoot());
		}
		url = url + "/";
		
		return url;
	},	

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */	
	_getDrillInServerRoot: function() {
		if (this._local) {
			// Local viewlet
			return "";
		}
		// Remote viewlet
		return this._getServerURL();
	},	

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTPostSaveQueryDescriptor: function(handler, reportUUID, projectAreaUUID, queryUUID) {

		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID,
			queryUUID: queryUUID,
			name: "",  					// Preserve existing query name
			keepInTempStorage: false,
			sharedIfNullFolder: true,
			ignoreParameters: true		// Preserve existing parameter names and values
		};
		this._callRESTReportsService(handler, "postSaveQueryDescriptor", arguments);
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportID
 	 * 
 	 * @return void
 	 */	
	_callRESTGetContainingProjectAreaItemIds: function(handler, reportID) {
		var args = {
			reportId: reportID
		};
		this._callRESTReportsService(handler, "getContainingProjectAreaItemIds", args);
	},	
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String queryUUID
 	 * @param Boolean onlyIfInTempStorage
 	 * 
 	 * @return void
 	 */	
	_callRESTPostDeleteQueryDescriptor: function(handler, queryUUID, onlyIfInTempStorage) {
		var arguments = {
			queryUUID: queryUUID,
			onlyIfInTempStorage: onlyIfInTempStorage
		};
		this._callRESTReportsService(handler, "postDeleteQueryDescriptor", arguments);
	},		
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * 
 	 * @return void
 	 */	
	_callRESTPostRenderQuery: function(handler, projectAreaUUID, queryUUID, parameterName, parameterValue) {

		var arguments = {
			projectAreaUUID: projectAreaUUID,
			queryUUID: queryUUID,
			parameterName: parameterName,
			parameterValue: parameterValue,
			showArchived: this._showArchived,
			urlPrefix: this._getImageServerRoot(),
			serverURL: this._getDrillInServerRoot()
		};
		
		this._callRESTReportsService(handler, "postRenderQuery", arguments);
	},	

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String projectAreaUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTGetReportDescriptors: function(handler, projectAreaUUID) {

		var args = {
			projectAreaUUID: projectAreaUUID,
			sortByName: true,
			flags: ONLY_MICRO
		};
		
		this._callRESTReportsService(handler, "getReportDescriptors", args);
	},		
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * 
 	 * @return void
 	 */	
	_callRESTGetShowArchived: function(handler) {
		this._callRESTReportsService(handler, "getShowArchived");
	},		
	
	/*
	 * @param values the array of values (Strings)
	 */
	_callRESTGetParameterLabels: function(handler, values) {
		var args = {
			values:values
		};
		this._callRESTReportsService(handler, "getParameterValueLabels", args);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String method
 	 * @param Object arguments
 	 * 
 	 * @return void
 	 */	
	_callRESTReportsService: function(handler, method, arguments) {
		// Create the service request
		var request = new ServiceRequest(
					this._static._REPORT_REST_SERVICE_URI,
					method,
					arguments);
		this.getServiceTracker().register(request, handler);
		// Invoke the service
		TeamServerClient.invokeService(request, handler);
	},

	// Utility Methods -------------------------------------------------------->

	/*
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventPrefLinkClicked: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Load the settings area
		this.getSite().toggleSettings();
	},		

	/*
 	 * @param String message
 	 * 
 	 * @return void
 	 */	
	_displayMessage: function(message) {
		// Get attach point
		var node = this._displayArea;
		
		// Create a wrapping div 
		var container = document.createElement("div");
		container.setAttribute("id", this._getReportDomID());
		// Create a text node for the message
		var textNode = document.createTextNode(message);
		// Assemble contents		
		container.appendChild(textNode);
		// Add the wrapping div to the node
		this._addReportNode(node, container);		
		
		// Show the display area
		this._showDisplayArea();
	},

	/*
 	 * @param String text
 	 * 
 	 * @return void
 	 */	
	_updateLoadingMessage: function(text) {
		// Get the attach point
		var node = this._loadingArea;
		// Get the text node
		var textNode = node.firstChild;
		
		if (!textNode &&
			(text.length === 0)) {
			// Nothing to do
			return;
		} else if (text.length === 0) {
			// Remove the old text node
			node.removeChild(textNode);
		} else if (!textNode) {
			// Create a new text node
			textNode = document.createTextNode(text);
			node.appendChild(textNode);
		} else {
			// Modify the existing text node
			textNode.replaceData(0, textNode.length, text); 
		}
	},		

	/*
 	 * @param String message
 	 * @param String explanation
 	 * @param String useraction
 	 * 
 	 * @return String
 	 */	
	_getOperationalErrorText: function(message, explanation, useraction) {
		var text = message + " " +
			explanation + " " +
			useraction;
		return text;
	},		
	
	/*
 	 * @param String message
 	 * @param String explanation
 	 * @param String useraction
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_alertErrorMessage: function(message, explanation, useraction, errorObject) {
		
		var text = this._getOperationalErrorText(message, explanation, useraction);
		
		if (errorObject) {
			this._logError(errorObject);
			if (errorObject.status === 403) {
				var errText = this._getOperationalErrorText(
						this.messages.msgReadPermissionError, 
						this.messages.msgReadPermissionError_explanation, 
						this.messages.msgReadPermissionError_useraction);
				this._logError(new Error(errText));
			}
		}		
		
		alert(text);		
	},	
	
	/*
 	 * @param String message
 	 * @param String explanation
 	 * @param String useraction
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */	
	_displayErrorMessage: function(message, explanation, useraction, errorObject) {
		
		var text = this._getOperationalErrorText(message, explanation, useraction);
		
		if (errorObject) {
			this._logError(errorObject);
			if (errorObject.status === 403) {
				var errText = this._getOperationalErrorText(
						this.messages.msgReadPermissionError, 
						this.messages.msgReadPermissionError_explanation, 
						this.messages.msgReadPermissionError_useraction);
				this._logError(new Error(errText));
			}
		}
		
		this._displayMessage(text);		
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_displayMsgSelectPreferences: function() {
		
		// Hide the parameters area
		this._updateVisibility(this._parametersArea, false);
		
		var container = document.createElement("div");
		container.id = this._getReportDomID();
		
		// Create actionable status message
		var links = I18nUtil.addMessage(container, this.messages.msgActionConfigureReport);
		var link = links[0];
		link.href = "javascript:void(0);";
		this.connect(link, "onclick", this._handleEventPrefLinkClicked);		
		
		this._addReportNode(this._displayArea, container);
		this._showDisplayArea();
	},	

	/*
 	 * @param String reportID
 	 * 
 	 * @return void
 	 */	
	_displayMsgReportNotDeployed: function(reportID) {
		if (this._canEditDashboard()) {
			this._displayActionReportNotDeployed(reportID);
		} else {
			var message = dojo.string.substitute(this.messages.msgReportNotDeployed, 
					[this._getReportTitle(reportID), reportID, this._getProjectAreaName()]);
			var text = this._getOperationalErrorText(
					message, 
					this.messages.msgReportNotDeployed_explanation, 
					this.messages.msgReportNotDeployed_useraction);			
			this._displayMessage(text);
		}			
	},
	
	/*
 	 * @param String reportID
 	 * 
 	 * @return void
 	 */	
	_displayActionReportNotDeployed: function(reportID) {

		// Hide the parameters area
		this._updateVisibility(this._parametersArea, false);		
		
		var container = document.createElement("div");
		container.id = this._getReportDomID();
		var message = dojo.string.substitute(this.messages.msgActionReportNotDeployed, 
				[this._getReportTitle(reportID), reportID, this._getProjectAreaName()]);
		
		// Create actionable status message
		var links = I18nUtil.addMessage(container, message);
		var link = links[0];
		link.href = "javascript:void(0);";
		this.connect(link, "onclick", this._handleEventPrefLinkClicked);		
		
		this._addReportNode(this._displayArea, container);
		this._showDisplayArea();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_showDisplayArea: function() {
		// Show the loading area
		var loadingNode = this._loadingArea;
		this._updateVisibility(loadingNode, false);
		// Hide the loading image
		this.getSite().setLoading(false);
		// Hide the display area
		var displayNode = this._displayArea;
		this._updateVisibility(displayNode, true);
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_showLoadingArea: function() {
		// Show the loading area
		var loadingNode = this._loadingArea;
		this._updateVisibility(loadingNode, true);
		// Show the loading image
		this.getSite().setLoading(true);
		// Hide the display area
		var displayNode = this._displayArea;
		this._updateVisibility(displayNode, false);
	},		
	
	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logInfo: function(errorObject) {
		// Log info only in debug mode
		if (djConfig.isDebug) {
			console.log(this._getErrorMessage("INFO", errorObject));
		}
	},	

	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logError: function(errorObject) {
		console.error(this._getErrorMessage("ERROR", errorObject));
	},	
	
	/*
 	 * @param Error errorObject
 	 * 
 	 * @return void
 	 */
	_logWarning: function(errorObject) {
		console.warn(this._getErrorMessage("WARNING", errorObject));
	},		
	
	/*
 	 * @param String prefix
 	 * @param Error errorObject
 	 * 
 	 * @return String
 	 */
	_getErrorMessage: function(prefix, errorObject) {
		
		var classID = "";
		var lineNumber = "";
		var message = "";

		// Class name
		if (this.id) {
			classID = this.id + ": ";
		} 

		if (errorObject) {
			// Line number
			if (errorObject.lineNumber) {
				lineNumber = "(line " + errorObject.lineNumber + "): ";
			} 
			// Error message
			if (errorObject.message) {
				message = errorObject.message;
			} else {
				message = "Message unavailable";
			}
			if (errorObject.status === 403) {
				message = message + ": " + this.messages.msgReadPermissionError;
			}			
		} else {
			// Error message
			message = "Undefined exception";			
		}
		
		var errorMessage = "[" + prefix + "]: " + 
							classID +
							lineNumber +
							message;
							
		return errorMessage;
	},		
	
	/*
 	 * @param String value
 	 * 
 	 * @return Boolean
 	 */	
	_isValidUUID: function(value) {
		// See com.ibm.team.repository.common.UUID.validateUUID(char[])
		// e.g. _H9EB5esEEdycivs5PH0c-g
		// A valid UUID must:
		// * Have a fixed length of 23 characters
		// * Start with an underscore character
		// * Contain characters only in the range of the MIME64 encoding
		//   (Not checked for performance reasons)
        if (value &&
			(value.length === 23) &&
			(value.charAt(0) === "_")) {
            return true;
        }
		return false;
	},	
	
	/*
 	 * @param String text
 	 * 
 	 * @return String
 	 */		
	 _hashString: function(text) {
		var hash = text.length;
		for (var index = 0; index < text.length; index++) {
			hash = ((hash << 5) ^ (hash >> 27)) ^ text.charCodeAt(index);
		}
		var hashKeyString = "K" + new Number(hash).toString();
		
		return hashKeyString;
	}
	
});

/*
 * Static Methods
 */
dojo.mixin(com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet, {

	/*
 	 * @param String transientQueryUUID
 	 * @param String classID
 	 * 
 	 * @return void
 	 */	
	deleteTransientReportQueryUUID: function(transientQueryUUID, classID) {
		
		var serviceObject = {
			success: function() {
				if (djConfig.isDebug) {
					console.log("[INFO]: " + classID + ": Transient report query deleted");
				}
			},
			failure: function(errorObject) {
				console.error("[ERROR]: " + classID + ": Failed to delete the transient report query");
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var arguments = {
			queryUUID: transientQueryUUID,
			onlyIfInTempStorage: true
		};
		var request = new ServiceRequest(
					"com.ibm.team.reports.common.internal.service.IReportRestService",
					"postDeleteQueryDescriptor",
					arguments);

		TeamServerClient.invokeService(request, handler);		
	}
	
});

})();

