/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureTable"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit.form.NumberSpinner"); //$NON-NLS-1$
	dojo.require("dijit.form.ValidationTextBox"); //$NON-NLS-1$
	dojo.require("dijit.MenuItem"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.MenuButton"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	var NUM_ACTIONS = 3;
	var MAX_WEEKS_DURATION = 10000;
	var MAX_ITEMS_TO_CREATE = 250;
	/**
	 * Table whose rows are a hierarchy of iteration structure, (aka pattern)
	 * groups, as used in the dialog CreateIterationsDialog.
	 * 
	 * Columns are editable controls for the groups' name, quantity, and
	 * duration. Buttons in an Actions column allow groups to be created,
	 * deleted, and advanced properties to be edited via an in-table sub-editor.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureTable", [dijit._Widget, jazz.ui.StyledTable], { //$NON-NLS-1$
		
		/**
		 * Besides the arguments to jazz.ui.StyledTable, the following are
		 * required:
		 * 
		 * - model			CreateIterationsDialogIterationStructureModel
		 * - rootIsEditable	Whether the root node's name can be modified. If
		 * 					false, plain text is displayed instead of a control
		 * 					to edit the name. Does not affect properties besides
		 * 					the name, which may never be modified for the root.
		 * - changesPendingCallback	Function that will be called when a
		 * 							change has been made. Clients may want to
		 * 							invoke validate() to see whether the
		 * 							control is in a valid state.
		 * - timelineStore	Store that contains scopes defined in the editor,
		 * 					used to populate the UI to select iteration types.
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			args.columns = [{
				label: this.messages.CreateIterationsDialogIterationStructureTable_1
			}, {
				label: this.messages.CreateIterationsDialogIterationStructureTable_2,
				className: "quantityColumnHeading" //$NON-NLS-1$
			}, {
				label: this.messages.CreateIterationsDialogIterationStructureTable_3,
				className: "durationColumnHeading" //$NON-NLS-1$
			}, {
				label: this.messages.CreateIterationsDialogIterationStructureTable_4,
				className: "actionsColumnHeading", //$NON-NLS-1$
				width: (NUM_ACTIONS * 40) + "px" //$NON-NLS-1$
			}];
			
			args.showHover = true;
			
			this._rowsByIdentity = {};
			
			this._invalidNameRows = {};
			this._invalidQuantityRows = {};
			this._invalidDurationRows = {};
			this._invalidIterationTypeRows = {};
			
			this._allInvalidMaps = [this._invalidNameRows, this._invalidQuantityRows, this._invalidDurationRows, this._invalidIterationTypeRows];
			
			this._changesPendingCallback = args.changesPendingCallback;
			
			this._inlineEditorUUID = 0;
			
			this._onInitFunctions = [];
		},
		
		postCreate: function() {
			this.inherited(arguments);
			
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-CreateIterationsDialogIterationStructureTable toolbar-table"); //$NON-NLS-1$
		},
		
		startup: function() {
			this._configureColumnWidths();
			
			this.model.getRoot(
				dojo.hitch(this, function(item) {
					this._createRow(item, 0);
				}),
				function(error) {
					//This should only be the result of a programming error;
					//log to the console w/ static message
					console.error(this, "error loading model root: ", error); //$NON-NLS-1$
				}
			);
			
			this.connect(this.model, "onChildrenChange", "_onItemChildrenChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.model.store, "aboutToDelete", "_onItemDelete"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._previousWidth = dojo.marginBox(this.tbody).w;
			
			setTimeout(dojo.hitch(this, function() {
				//initialize field _initialNameColumnWidth which is used in the
				//calculation of the widths of the 'Name' column textboxes
				var widthDelta = this._previousWidth - dojo.marginBox(this.domNode).w;
				this._initialNameColumnWidth = this._maximumNameColumnWidth - widthDelta;
			}));
		},
		
		/**
		 * Should be called when the container is ready to have rows that
		 * were created during startup be selected, (which fires
		 * selectionChanged events).
		 */
		init: function() {
			dojo.forEach(this._onInitFunctions, function(func) {
				func();
			});
			this._initialized = true;
		},
		
		/**
		 * Should be called when this widget or its containing element or
		 * window has been resized. Adjustments are made to elements such
		 * as the inputs for iteration names.
		 */
		resized: function() {
			var newWidth = dojo.marginBox(this.domNode).w;
			this._maximumNameColumnWidth -= (this._previousWidth - newWidth);
			this._adjustAllNameInputWidths();
			this._previousWidth = newWidth;
		},
		
		/**
		 * Returns whether the data that's been entered has left the iteration
		 * structure in a valid state.
		 */
		validate: function() {
			return dojo.every(this._allInvalidMaps, function(map) {
				for (var itemId in map) {
					if (map.hasOwnProperty(itemId)) {
						return false;
					}
				}
				return true;
			});
		},
		
		/**
		 * Consumers may dojo.connect to this function to be notified when
		 * the selected row has changed.
		 * 
		 * item		The item corresponding to the row that was selected, or
		 * 			<code>null</code> if no item is selected.
		 */
		selectionChanged: function(item) {
		},
		
		/**
		 * Set the width of the duration and quantity columns to minimum values,
		 * so that the Name column can be as wide as there is space available.
		 */
		_configureColumnWidths: function() {
			dojo.query("th.quantityColumnHeading div", this.domNode).forEach(dojo.hitch(this, function(node) { //$NON-NLS-1$
				//Set the width of the Quantity column to be a little more
				//than the space required for the word "Quantity". A
				//min-width is declared in the CSS as a backup, in case this
				//width is less than the space we want for the spinner in the
				//rows
				var maxStringDisplayWidthInEms=UIHelper.getMaxStringDisplayWidthInEms([this.messages.CreateIterationsDialogIterationStructureTable_2], node);
				dojo.style(node.parentNode, "width", (.5 + maxStringDisplayWidthInEms) + "em"); //$NON-NLS-1$ //$NON-NLS-2$
			}));
			
			dojo.query("th.durationColumnHeading div", this.domNode).forEach(dojo.hitch(this, function(node) { //$NON-NLS-1$
				//Set the width to the maximum of... 
				var width = Math.max(
					//the space required for "weeks" plus 5em to to account for
					//the duration spinner control plus some padding
					5 + UIHelper.getMaxStringDisplayWidthInEms([this.messages.CreateIterationsDialogIterationStructureTable_14], node),
					
					//... the width of the header text
					.5 + UIHelper.getMaxStringDisplayWidthInEms([this.messages.CreateIterationsDialogIterationStructureTable_3], node)
				);
				dojo.style(node.parentNode, "width", width + "em"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(node.parentNode, "minWidth", width + "em"); //$NON-NLS-1$ //$NON-NLS-2$
			}));
		},
		
		_createRow: function(item, level, parentItem, node) {
			var tr;
			var afterNode= node;
			if (afterNode && afterNode._editorRow) {
				afterNode = afterNode._editorRow;
			}
			if (afterNode === undefined || !afterNode.nextSibling) {
				tr = this._createElement("tr", {'class': "iterationStructureTableRow"}, this.tbody); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			} else {
				tr = this._createElement("tr", {'class': "iterationStructureTableRow"}, afterNode.nextSibling, "before"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			}
			
			tr._level = level;
			tr._item = item;
			tr._parentItem = parentItem;
			tr._connects = [];
			tr._childRows = [];
			tr._itemIdentity = this.model.store.getIdentity(item);
			
			tr._nameTd = this._createElement("td", {'class': "nameColumn"}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			if (this._maximumNameColumnWidth === undefined) {
				this._maximumNameColumnWidth = dojo.marginBox(tr._nameTd).w;
			}
			
			//Create node for expand/collapse
			tr._expanded = true;
				
	    	var expandoLink = this._createElement("a", { //$NON-NLS-1$
				"href": "#" //$NON-NLS-1$ //$NON-NLS-2$
			}, tr._nameTd);
			dojo.addClass(expandoLink, "expandoLink"); //$NON-NLS-1$
			
			tr._nameTd._expandoImg = this._createElement("img", {//$NON-NLS-1$
				'class': "expandoImage", //$NON-NLS-1$ //$NON-NLS-2$
				src: tr._expanded
					? dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse.gif") //$NON-NLS-1$ //$NON-NLS-2$
					: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/expand.gif") //$NON-NLS-1$ //$NON-NLS-2$
			}, expandoLink);
			
			tr._expandNode = expandoLink;
			
			tr._connects = tr._connects.concat(this._selectRowOnFocus(tr, expandoLink));
			
			tr._icon = this._createElement("a", {href: "#"}, tr._nameTd); //$NON-NLS-1$ //$NON-NLS-2$
			this._createElement("img", { //$NON-NLS-1$
				'class': "typeIcon", //$NON-NLS-1$ //$NON-NLS-2$
				src: true === this.model.store.getValue(item, "isTimeline") //$NON-NLS-1$
					? dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/develop_line_obj.gif") //$NON-NLS-1$ //$NON-NLS-2$
					: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_obj.gif") //$NON-NLS-1$ //$NON-NLS-2$
			}, tr._icon);
			
			tr._connects = tr._connects.concat([
				dojo.connect(tr._icon, "onclick", function(event) { //$NON-NLS-1$
	    			dojo.stopEvent(event);
	    		})
	    	]);
    		tr._connects = tr._connects.concat(this._selectRowOnFocus(tr, tr._icon));
			
			this._createNameControl(tr, tr._nameTd, tr._item, tr._itemIdentity, parentItem);
			
			var getParentRow = function(node) {
				var theNode = node;
				while (theNode && !dojo.hasClass(theNode, "iterationStructureTableRow")) { //$NON-NLS-1$
					theNode = theNode.parentNode;
				}
				return theNode;
			};
			
			tr._connects = tr._connects.concat([
				dojo.connect(tr._expandNode, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
					dojo.stopEvent(event);
					this._toggleExpanded(getParentRow(event.target));
				}))
			]);
			
			dojo.toggleClass(tr._expandNode, "invisible", !this.model.mayHaveChildren(item)); //$NON-NLS-1$
			
			dojo.style(tr._nameTd, this.isLeftToRight() ? "paddingLeft" : "paddingRight", (((level) * 18)) + "px"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			tr._quantityTd = this._createElement("td", {'class': "quantityColumn"}, tr); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			if (level !== 0) {
				this._createQuantitySpinner(tr, tr._quantityTd, item, tr._itemIdentity);
	        }
			
			tr._durationTd = this._createElement("td", {}, tr); //$NON-NLS-1$
			if (true === this.rootIsEditable || level !== 0) {
				this._createDurationSpinner(tr, tr._durationTd, item, tr._itemIdentity);
	        }
			
			tr._actionsTd = this._createElement("td", {}, tr); //$NON-NLS-1$
			this._createActionsColumn(tr._actionsTd, tr._item, tr._parentItem, tr);
			
			this._rowsByIdentity[tr._itemIdentity] = tr;
			
			if (tr._nameTd._textBox) {
				//We want to select all text in the new row's name, which will
				//indirectly focus the row and cause a node in the Preview tree
				//to be highlighted and scrolled into view, (which we
				//absolutely want to happen). However, we don't necessarily
				//want the events to occur immediately; if the dialog is
				//just opening, there is an ugly thrashing while the auto-
				//scrolling fights with the centering of the dialog. If
				//function init() hasn't been called, delay the focus of the
				//row until it is.
				var adjustWidthsAndHighlightText = dojo.hitch(this, function() {
					this._adjustAllNameInputWidths();
					
					//Highlight all text in the Name of newly created rows
					dijit.selectInputText(tr._nameTd._textBox.textbox);
				});
				if (this._initialized) {
					setTimeout(adjustWidthsAndHighlightText);	
				} else {
					this._onInitFunctions.push(adjustWidthsAndHighlightText);
				}
			}
			
			//Give focus to the timeline/iteration icon when the row is clicked.
			//This is to support the 'selectionChanged' mechanism to notify
			//listeners when rows are selected
			tr._connects.push(
				dojo.connect(tr, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
					if (dojo.hasClass(event.target, "selectRowOnClick")) { //$NON-NLS-1$
						dojo.stopEvent(event);
						tr._icon.focus();
					}
				}))
			);
			
            if(!this._validateTotalNewNumberOfElements()){
            	this._invalidQuantityRows[tr._itemIdentity] = tr._quantityTd._quantitySpinner;
            }
			return tr;
		},
		
		_createNameControl: function(row, nameTd, item, itemIdentity, parentItem) {
			var iterationsName = this.model.getLabel(item);
			if (parentItem === undefined && false === this.rootIsEditable) {
				nameTd.appendChild(document.createTextNode(iterationsName));
			} else {
				nameTd._textBox = this._createTextBox(row, {
					value: iterationsName,
					required: true
				}, dojo.hitch(this, function(value) {
					if (nameTd._textBox.isValid()) {
						delete this._invalidNameRows[itemIdentity];
	            		
	            		this.model.store.setValue(item, "name", value); //$NON-NLS-1$
		            } else {
		            	this._invalidNameRows[itemIdentity] = true;
						
						//We don't want to use the empty string, since clients
						//may require names to specified for uniqueness checks.
						//So while the user hasn't provided a name, use the
						//original (default) name.
						this.model.store.setValue(item, "name", this.model.store.getValue(item, "originalName")); //$NON-NLS-1$ //$NON-NLS-2$
		            }
	            	if (row._nameTd._textBox.get('disabled') !== true) { //$NON-NLS-1$
						//If we're using a custom naming convention, the
						//callbacks have already been invoked
						this.model.namingConventionChanged(item);
						this._changesPendingCallback();
					}
				}));
				
				nameTd.appendChild(nameTd._textBox.domNode);
			}
		},
		
		_refreshQuantityValues: function() {
			var map = this._invalidQuantityRows;
			for ( var itemId in map) {
				if (map.hasOwnProperty(itemId)) {
					var spinner = map[itemId];
					dojo.hitch(spinner, function() {
						var val = spinner.get("value");//$NON-NLS-1$
						spinner.set("value", "");//$NON-NLS-1$ //$NON-NLS-2$
						spinner.set("value", val);//$NON-NLS-1$
						return;
					})();
				}
			}
		},
		
		_createQuantitySpinner: function(row, quantityTd, item, itemIdentity) {
			quantityTd._quantitySpinner = this._createNumberSpinner(
				row, {
	                value: this.model.store.getValue(item, "quantity"), //$NON-NLS-1$
	                smallDelta: 1,
	                constraints: {min: 1, places: 0},
	                required: true,
	                intermediateChanges: true
	            }, dojo.hitch(this, function(value) {
	            	if (quantityTd._quantitySpinner.isValid() && this._validateTotalNewNumberOfElements()) {
	            		delete this._invalidQuantityRows[itemIdentity];
	            		
	            		this.model.store.setValue(item, "quantity", value); //$NON-NLS-1$
	            		this._refreshQuantityValues();
	            	}else {
	            		this._invalidQuantityRows[itemIdentity] = quantityTd._quantitySpinner;
	            	}
	            	
	            	this._changesPendingCallback();
	            })
	        );
			
			//Customize adjust() so that '1' is returned when the Duration
            //field is empty. This is a bit of an edge case since the value is
            //defaulted to '1', but without this change clearing the value and
            //clicking to decrement the value changes it to a very large
            //number, and the browser crashes when that number of nodes are
            //created in the Preview tree
            quantityTd._quantitySpinner._origAdjust = dojo.hitch(quantityTd._quantitySpinner, quantityTd._quantitySpinner.adjust);
            quantityTd._quantitySpinner.adjust = function(/*Object*/ val, /*Number*/ delta) {
            	if (isNaN(val)) {
            		return 1;
            	}
            	return quantityTd._quantitySpinner._origAdjust(val, delta);
            };
            
            dojo.addClass(quantityTd._quantitySpinner.domNode, "quantitySpinner"); //$NON-NLS-1$
            quantityTd.appendChild(quantityTd._quantitySpinner.domNode);
            
            //center control within its parent
            setTimeout(dojo.hitch(this, function(node) {
            	var parentBox = dojo.marginBox(node.parentNode);
            	var nodeBox = dojo.marginBox(node);
            	//I'm not why, but multiplying by .5 does not add enough to the
            	//margin
            	dojo.style(node, "marginLeft", ((parentBox.w - nodeBox.w) * .75) + "px"); //$NON-NLS-1$ //$NON-NLS-2$
            }, quantityTd._quantitySpinner.domNode));
		},
		
		_validateTotalNewNumberOfElements: function() {
			var count = this._countTotalNewNumberOfElements(this.model.root);
			valid = (count <= MAX_ITEMS_TO_CREATE);
			if (valid) {
				this.reportError(); // Clear the error message
				this._enableAllAddIterationsButtons(true);
			} else {
				this.reportError(dojo.string.substitute(this.messages.CreateIterationsDialogCreateIterationsDialogPreviewTree_2, [count, MAX_ITEMS_TO_CREATE]));
				this._enableAllAddIterationsButtons(false);
			}
			return valid;
		},

		_enableAllAddIterationsButtons: function(enable) {
			for ( var i in this._rowsByIdentity) {
				var row = this._rowsByIdentity[i];
				if (row._actionsTd && row._actionsTd._addIterationsButton) {
					row._actionsTd._addIterationsButton.set("enabled", enable);//$NON-NLS-1$
				}
			}
		},

		_countTotalNewNumberOfElements: function(item) {
			var currentChildren = this.model.store.getValues(item, "children") || []; //$NON-NLS-1$
			var childrenCount = 0;
			for ( var i = 0; i < currentChildren.length; i++) {
				childrenCount += this._countTotalNewNumberOfElements(currentChildren[i]);
			}

			var row = this._rowsByIdentity[this.model.getIdentity(item)];
			var itemQuantity;
			if (row._quantityTd && row._quantityTd._quantitySpinner) {
				itemQuantity = row._quantityTd._quantitySpinner.get("value"); //$NON-NLS-1$
			} else {
				itemQuantity = 1;
			}
			if (itemQuantity) {
				childrenCount = childrenCount * itemQuantity;
				if (row._durationTd && row._durationTd._durationSpinner) {
					//Count the item if it is new (duration can be set)
					childrenCount += itemQuantity;
				}
			}
			return childrenCount;
		},

		_createDurationSpinner: function(row, durationTd, item, itemIdentity) {
			durationTd._durationSpinner = this._createNumberSpinner(
				row, {
	                smallDelta: 1,
	                constraints: {min: 0, max: MAX_WEEKS_DURATION, places: 0},
	                intermediateChanges: true
	            }, dojo.hitch(this, function(value) {
	            	var value = durationTd._durationSpinner.get("value"); //$NON-NLS-1$
	            	 
	            	if (durationTd._durationSpinner.isValid()) {
	            		delete this._invalidDurationRows[itemIdentity];
	            		
	            		this.model.store.setValue(item, "duration", value); //$NON-NLS-1$
	            	} else {
	            		this._invalidDurationRows[itemIdentity] = true;
	            	}
	            	
	            	this._changesPendingCallback();
	           	})
	        );
            
            //Customize adjust() so that '1' is returned when the Duration
            //field is empty, (its default state). The default adjust() would
            //return 0 when incrementing and 90000... when decrementing.
            //We still want to support entering 0 on clicking on decrement, but we want 1 to be the
            //default clicking to increment. And we want the
            //field to be blank initially (-- we're picky)
            durationTd._durationSpinner._origAdjust = dojo.hitch(durationTd._durationSpinner, durationTd._durationSpinner.adjust);
            durationTd._durationSpinner.adjust = function(/*Object*/ val, /*Number*/ delta) {
            	if (isNaN(val)) {
            		return (delta>0) ? 1:0;
            	}
            	return durationTd._durationSpinner._origAdjust(val, delta);
            };
            
            dojo.addClass(durationTd._durationSpinner.domNode, "durationSpinner"); //$NON-NLS-1$
            durationTd.appendChild(durationTd._durationSpinner.domNode);
            
            var weeksSpan = this._createElement("span", {}, durationTd); //$NON-NLS-1$
            weeksSpan.appendChild(document.createTextNode(this.messages.CreateIterationsDialogIterationStructureTable_14));
		},
		
		_createActionsColumn: function(actionsTd, item, parentItem, row) {
			actionsTd._toolbar = new com.ibm.team.process.web.ui.internal.TRToolbar({
				_rowSelected: dojo.hitch(this, "_rowSelected") //$NON-NLS-1$
			});
			
    		var addWidget = dojo.hitch(this, function(widget) {
    			actionsTd._toolbar.addChild(widget);
    			return widget;
    		});
    		
    		var getParentRow = function(element) {
				var row = element;
				while (row && row.tagName.toLowerCase() !== "tr") { //$NON-NLS-1$
					row = row.parentNode;
				}
				return row;
			};
			
			var self = this;
			
			if (parentItem === undefined) {
				//Root nodes have a button to toggle edit properties if they're
				//editable, and a to add child iterations
				
				if (true === this.rootIsEditable) {
					actionsTd._editPropertiesButton = new jazz.ui.toolbar.TwoStateButton({
						imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/edit.gif"), //$NON-NLS-1$ //$NON-NLS-2$
						title: this.messages.CreateIterationsDialogIterationStructureTable_10,
						onClick: function() {
							var row = getParentRow(this.domNode);
							self._toggleEditProperties(row, this.pressed, dojo.hitch(self, self._createEditorContentsForTimeline));
						}
					});
					addWidget(actionsTd._editPropertiesButton);
				}
				
				actionsTd._addIterationsButton = new jazz.ui.toolbar.Button({
					imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/add.gif"), //$NON-NLS-1$ //$NON-NLS-2$
					title: this.messages.CreateIterationsDialogIterationStructureTable_6,
					onClick: function(event) {
						dojo.stopEvent(event);
						self._addChildIterations(item, null);
					}
				});
				addWidget(actionsTd._addIterationsButton);
			} else {
				//Non-root nodes have actions to create iterations, edit
				//properties, and delete
				
				actionsTd._addIterationsButton = this._createMenuButton(row, {
					title: this.messages.CreateIterationsDialogIterationStructureTable_5,
					menuProvider: dojo.hitch(this, this._addIterationsMenuProvider, item, parentItem),				
					imgSrc :  "com.ibm.team.process.web/ui/internal/admin/projects/images/add.gif", //$NON-NLS-1$				
					align: this.isLeftToRight() ? "right" : "left", //$NON-NLS-1$ //$NON-NLS-2$
					destroyOnClose: true
				});
					 
				
				addWidget(actionsTd._addIterationsButton);
				
				actionsTd._editPropertiesButton = new jazz.ui.toolbar.TwoStateButton({
					imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/edit.gif"), //$NON-NLS-1$ //$NON-NLS-2$
					title: this.messages.CreateIterationsDialogIterationStructureTable_10,
					onClick: function() {
						var row = getParentRow(this.domNode);
						self._toggleEditProperties(row, this.pressed, dojo.hitch(self, self._createEditorContentsForIteration));
					}
				});
				addWidget(actionsTd._editPropertiesButton);
				
				actionsTd._deleteButton = new jazz.ui.toolbar.Button({
					imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/delete_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
					title: this.messages.CreateIterationsDialogIterationStructureTable_9,
					onClick: function(event) {
						dojo.stopEvent(event);
						self._deleteIterations(item);
					}
				});
				addWidget(actionsTd._deleteButton);
			}
			actionsTd.appendChild(actionsTd._toolbar.domNode);
			
			return actionsTd;
		},
		
		_addIterationsMenuProvider: function(item, parentItem) {
			var menu = new jazz.ui.Menu();
			dojo.addClass(menu.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorEditor-addConfigurationDropDown"); //$NON-NLS-1$
			
			//Add child iterations
			menu.addChild(new dijit.MenuItem({
				label: this.messages.CreateIterationsDialogIterationStructureTable_6,
				onClick: dojo.hitch(this, this._addChildIterations, item, null)
			}));
			
			//Add peer iterations
								
			menu.addChild(new dijit.MenuItem({
				label: dojo.string.substitute(this.messages.CreateIterationsDialogIterationStructureTable_7, [this.model.store.getValue(item, "name")]), //$NON-NLS-1$
				onClick: dojo.hitch(this, this._addChildIterations, parentItem, item, true)
			}));
			
			menu.addChild(new dijit.MenuItem({
				label: dojo.string.substitute(this.messages.CreateIterationsDialogIterationStructureTable_8, [this.model.store.getValue(item, "name")]), //$NON-NLS-1$
				onClick: dojo.hitch(this, this._addChildIterations, parentItem, item, false)
			}));
			
			menu.startup();
			
			return menu;
		},
		
		_addChildIterations: function(parentItem, existingItem, before) {
			var defaultName = this.messages.IterationDialog_NewIterationDefaultName;
			var newItem = this.model.store.newItem({
				name: defaultName,
				originalName: defaultName,
				quantity: 1,
				parentId: this.model.store.getIdentity(parentItem),
				hasDeliverable: true,
				iterationTypeItemId: ProcessConstants.ITERATION_TYPE_NONE
			});
			
			var currentChildren = this.model.store.getValues(parentItem, "children") || []; //$NON-NLS-1$
			if (existingItem === null) {
				currentChildren.push(newItem);
			} else {
				for(var i = 0; i < currentChildren.length; i++) {
					if (currentChildren[i] === existingItem) {
						currentChildren.splice(before ? i : i+1, 0, newItem);
						break;
					}
				} 
			}
			this.model.store.setValues(parentItem, "children", currentChildren); //$NON-NLS-1$
			
			this._changesPendingCallback();
		},
		
		_deleteIterations: function(item) {
			//delete descendant items and then item itself
			this.model.getChildren(item, dojo.hitch(this, function(children) {
				dojo.forEach(children, dojo.hitch(this, this._deleteIterations));
			}));
			this.model.store.deleteItem(item);
			
			this._validateTotalNewNumberOfElements();
			this._refreshQuantityValues();
			this._changesPendingCallback();
		},
		
		_toggleExpanded: function(row) {
			row._expanded = !row._expanded;
			
    		if (row._nameTd._expandoImg) {
    			var imgSrc;
	    		if (row._expanded) {
	    			imgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	    		} else {
	    			imgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/expand.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	    		}
    			row._nameTd._expandoImg.src = imgSrc;
    		}
    		
    		if (row._childRows) {
    			var handleRow = dojo.hitch(this, function(nextRow) {
    				if (nextRow._childRows) {
	    				dojo.forEach(nextRow._childRows, function(childRow) {
		    				dojo.toggleClass(childRow, "hidden", !row._expanded); //$NON-NLS-1$
		    				
		    				if (!row._expanded || childRow._expanded) {
		    					handleRow(childRow);
		    				}
		    			});
		    		}
		    		
		    		if (!row._expanded) {
						this._toggleEditProperties(nextRow, false);
					}
    			});
    			handleRow(row);
    		}
		},
		
		_namingConventionChanged: function(row) {
			row._nameTd._textBox.set('disabled', true); //$NON-NLS-1$
			
			this.model.setNameGenerator(row._item, row._editorRow._customNamingConventionSection, row._editorRow._customNamingConventionSection.generateName);
			
			row._nameTd._textBox.set("value", row._editorRow._customNamingConventionSection.getNamingConventionSummary()); //$NON-NLS-1$
			
			this.changesPendingCallback();
		},
		
		_revertToDefaultNamingConvention: function(row) {
			dojo.toggleClass(row._editorRow._customizeNamingConventionButton.domNode, "hidden", false); //$NON-NLS-1$
			
			dojo.destroy(row._editorRow._customNamingConventionSection.domNode.parentNode);
			row._editorRow._customNamingConventionSection.destroyRecursive();
			delete row._editorRow._customNamingConventionSection;
			
			//The order of the following is critical:
			
			//Update the item's name. This way the default name generator will
			//return the correct value.
			this.model.store.setValue(row._item, "name", row._originalName); //$NON-NLS-1$
			
			//Clear the name generator so that the default name generator is
			//used. This will fire a namingConventionChanged event
			this.model.setNameGenerator(row._item, null);
			
			//Restore the value of the name field while it's disabled. This way
			//a second namingConventionChanged event isn't fired
			row._nameTd._textBox.set("value", row._originalName); //$NON-NLS-1$
			
			//Finally, enabled the name field
			row._nameTd._textBox.set('disabled', false); //$NON-NLS-1$
			
			//So that the row remains selected
			row._editorRow._customizeNamingConventionButton.focusNode.focus();
			
			this.changesPendingCallback();
		},
		
		/**
		 * Show or hide the row to edit operation properties.
		 * 
		 * @param	row		DOM node for the row
		 * @param	editing	Boolean that indicates whether to show or hide
		 * 					the editor row
		 * @param	createEditorContents	Function that is responsible for
		 * 									adding the controls to the editor
		 */
		_toggleEditProperties: function(row, editing, createEditorContents) {
			dojo.toggleClass(row, "propertiesExpanded", editing); //$NON-NLS-1$

			if (editing) {
				if (row._editorRow) {
					dojo.toggleClass(row._editorRow, "hidden", false); //$NON-NLS-1$
				} else {
					// create the table row to hold it
					row._editorRow = document.createElement("tr"); //$NON-NLS-1$
					dojo.addClass(row._editorRow, "editorRow"); //$NON-NLS-1$
	
					var td = document.createElement("td"); //$NON-NLS-1$
					td.setAttribute("colspan", row.children.length); //$NON-NLS-1$
					row._editorRow.appendChild(td);
					
					// create the editor
					var contentNode = this._createElement("div", {'class': "content"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					
					createEditorContents(row, contentNode);
					
					//Select the row's type icon when the background of
					//the properties section or a sub-element is clicked. This
					//is to support selectionChanged() to notify listeners when
					//rows are selected
					row._connects.push(
						dojo.connect(contentNode, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
							if (event.target === contentNode || dojo.hasClass(event.target, "selectRowOnClick")) { //$NON-NLS-1$
								dojo.stopEvent(event);
								row._icon.focus();
							}
						}))
					);
					
					td.appendChild(contentNode);
					
					// add the editor to the document
					if (row.nextSibling) {
						this.tbody.insertBefore(row._editorRow, row.nextSibling);
					} else {
						this.tbody.appendChild(row._editorRow);
					}
				}
			} else if (row._editorRow && this.tbody === row._editorRow.parentNode) {
				// toggle edit off
				dojo.toggleClass(row._editorRow, "hidden", true); //$NON-NLS-1$
				dojo.toggleClass(row, "propertiesExpanded", false); //$NON-NLS-1$
			}
			
			// make sure the button state is accurate
			if (row._actionsTd && row._actionsTd._editPropertiesButton) {
				row._actionsTd._editPropertiesButton.set("pressed", editing); //$NON-NLS-1$
			}
		},
		
		_createEditorContentsForTimeline: function(row, contentNode) {
			var isProjectTimelineCheckboxId = "CreateIterationsDialogIterationStructureTableIsProjectTimeline"; //$NON-NLS-1$
			row._editorRow.isProjectTimelineCheckbox = this._createCheckbox(
				row,
				isProjectTimelineCheckboxId,
				this.messages.TimelineDialog_NewProjectTimelineCheckBoxLabel,
				this.model.store.getValue(row._item, "projectTimeline"), //$NON-NLS-1$
				dojo.hitch(this, function(isChecked) {
					this.model.store.setValue(row._item, "projectTimeline", isChecked); //$NON-NLS-1$
					this._changesPendingCallback();
				})
			);
			var isProjectTimelineCheckboxDiv = this._createElement("div", {'class': "propertyControl"}, contentNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			isProjectTimelineCheckboxDiv.appendChild(row._editorRow.isProjectTimelineCheckbox);
		},
		
		_createEditorContentsForIteration: function(row, contentNode) {
			row._editorRow._customizeNamingConventionButton = this._createToolbarButton(row, {
				label: this.messages.CreateIterationsDialogIterationStructureTable_13,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/naming_convention_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(theRow) {
					dojo.toggleClass(row._editorRow._customizeNamingConventionButton.domNode, "hidden", true); //$NON-NLS-1$
					
					theRow._editorRow._customNamingConventionSection = new com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogCustomNamingConventionSection({
						iterationName: this.model.store.getValue(theRow._item, "name"), //$NON-NLS-1$
						originalIterationName: this.model.store.getValue(theRow._item, "originalName"), //$NON-NLS-1$
						namingConventionChanged: dojo.hitch(this, this._namingConventionChanged, theRow),
						revertToDefaultNamingConvention: dojo.hitch(this, this._revertToDefaultNamingConvention, theRow),
						createElement: dojo.hitch(this, function(tagName, properties, parentElement, position) {
							return this._createElement(tagName, properties, parentElement, position);
						}),
						createMenuButton: dojo.hitch(this, function(menuButtonArgs) {
							return this._createMenuButton(theRow, menuButtonArgs);
						}),
						createToolbarButton: dojo.hitch(this, function(buttonArgs) {
							return this._createToolbarButton(theRow, buttonArgs);
						}),
						createTextBox: dojo.hitch(this, function(textBoxArgs, valueChangedCallback) {
							return this._createTextBox(theRow, textBoxArgs, valueChangedCallback);
						}),
						createNumberSpinner: dojo.hitch(this, function(numberSpinnerArgs, valueChangedCallback) {
							return this._createNumberSpinner(theRow, numberSpinnerArgs, valueChangedCallback);
						}),
						createCheckbox: dojo.hitch(this, function(checkboxId, labelText, initialChecked, valueChangedCallback) {
							return this._createCheckbox(theRow, checkboxId, labelText, initialChecked, valueChangedCallback);
						})
					});
					theRow._editorRow._customNamingConventionDiv = this._createElement("div", { //$NON-NLS-1$
						'class': "propertyControl" //$NON-NLS-1$ //$NON-NLS-2$
					}, theRow._editorRow._customizeNamingConventionButton.domNode, "after"); //$NON-NLS-1$
					theRow._editorRow._customNamingConventionDiv.appendChild(theRow._editorRow._customNamingConventionSection.domNode);
					dojo.addClass(theRow._editorRow._customNamingConventionSection.domNode, "selectRowOnClick"); //$NON-NLS-1$
					theRow._editorRow._customNamingConventionSection.startup();
					
					row._originalName = this.model.store.getValue(row._item, "name"); //$NON-NLS-1$
					
					this._namingConventionChanged(theRow);
				}, row)
			});
			dojo.addClass(row._editorRow._customizeNamingConventionButton.domNode, "propertyControl customizeNamingConventionButton j-h2"); //$NON-NLS-1$
			contentNode.appendChild(row._editorRow._customizeNamingConventionButton.domNode);
			
			var iterationTypeDiv = this._createElement("div", { //$NON-NLS-1$
				'class': "propertyControl" //$NON-NLS-1$ //$NON-NLS-2$
			}, contentNode);
			var iterationTypeLabel = this._createElement("span", { //$NON-NLS-1$
				'class': "j-label iterationTypeLabel" //$NON-NLS-1$ //$NON-NLS-2$
			}, iterationTypeDiv);
			iterationTypeLabel.appendChild(document.createTextNode(this.messages.CreateIterationsDialogIterationStructureTable_11));
			
			row._editorRow._iterationTypesSelect = new com.ibm.team.process.web.ui.internal.admin.projects.IterationTypesSelect({
				timelineStore: this.timelineStore,
				required: false,
	    		intermediateChanges: true
			});
			row._editorRow._iterationTypesSelect.set("value", this.model.store.getValue(row._item, "iterationTypeItemId")); //$NON-NLS-1$ //$NON-NLS-2$
			row._connects.push(dojo.connect(row._editorRow._iterationTypesSelect, "validate", dojo.hitch(this, function(theRow) { //$NON-NLS-1$
				if (theRow._editorRow._iterationTypesSelect.isValid()) { 
					delete this._invalidIterationTypeRows[theRow._itemIdentity];
					
					var value = theRow._editorRow._iterationTypesSelect.get("value"); //$NON-NLS-1$
					var displayedValue = theRow._editorRow._iterationTypesSelect.get("displayedValue"); //$NON-NLS-1$
					if (value === "" && displayedValue === "") { //$NON-NLS-1$ //$NON-NLS-2$
						value = ProcessConstants.ITERATION_TYPE_NONE;
					}
					this.model.store.setValue(theRow._item, "iterationTypeItemId", value); //$NON-NLS-1$
				} else {
					this._invalidIterationTypeRows[theRow._itemIdentity] = true;
				}
				this._changesPendingCallback();
			}, row)));
			row._connects = row._connects.concat(this._selectRowOnFocus(row, row._editorRow._iterationTypesSelect.focusNode));
			iterationTypeDiv.appendChild(row._editorRow._iterationTypesSelect.domNode);
			
			var isReleaseCheckboxId = "CreateIterationsDialogIterationStructureTable" + this._inlineEditorUUID++; //$NON-NLS-1$
			
			row._editorRow.isReleaseCheckbox = this._createCheckbox(
				row,
				isReleaseCheckboxId,
				this.messages.CreateIterationsDialogIterationStructureTable_12,
				this.model.store.getValue(row._item, "hasDeliverable"), //$NON-NLS-1$
				dojo.hitch(this, function(isChecked) {
					this.model.store.setValue(row._item, "hasDeliverable", isChecked); //$NON-NLS-1$
					this._changesPendingCallback();
				})
			);
			var isReleaseCheckboxDiv = this._createElement("div", {'class': "propertyControl"}, contentNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			isReleaseCheckboxDiv.appendChild(row._editorRow.isReleaseCheckbox);
		},
		
		_adjustNameInputWidth: function(row) {
			if (row._nameTd && row._nameTd._textBox) {
				var icon = dojo.marginBox(row._icon);
				var width;
				var referenceWidth = this._initialNameColumnWidth === undefined
					? this._maximumNameColumnWidth
					: Math.max(this._maximumNameColumnWidth, this._initialNameColumnWidth);
				if (this.isLeftToRight()) {
					width = referenceWidth - icon.l - icon.w - 16;
				} else {
					var td = dojo.marginBox(row._nameTd);
					width = referenceWidth - (td.w - icon.l - icon.w) - 16;
				}
				
				dojo.style(row._nameTd._textBox.domNode, "width", width + "px"); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},
		
		_adjustAllNameInputWidths: function() {
			dojo.query("tr", this.domNode).forEach(dojo.hitch(this, this._adjustNameInputWidth)); //$NON-NLS-1$
		},
		
		/**
		 * Helper function that attaches listeners to a node's onfocus and
		 * onblur events that select the row when the node is focused.
		 * 
		 * This is useful to support keyboard navigation, where we want the
		 * row to be selected, which makes the controls in the Actions
		 * column visible, as soon as the user tabs to a node in the row.
		 * 
		 * In general, this function should be called for any DOM node in the
		 * row that accepts focus, eg. links and icons.
		 * 
		 * @param	row		The row that is to be selected when the node is
		 * 					focused
		 * @param	node	A DOM node that would typically be a child of the
		 * 					row 
		 */
		_selectRowOnFocus: function(row, node) {
			return [
				dojo.connect(node, "onfocus", dojo.hitch(this, function(e) { //$NON-NLS-1$
					dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
					this._rowSelected(row, true);
				})),
				
				dojo.connect(node, "onblur", dojo.hitch(this, function(e) { //$NON-NLS-1$
					dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
					this._rowSelected(row, false);
				}))
			];
		},
		
		_rowSelected: function(row, selected) {
			if (row._clearSelectionTimeout) {
				clearTimeout(row._clearSelectionTimeout);
			}
			if (this._clearSelectionTimeout) {
				clearTimeout(this._clearSelectionTimeout);
			}
			
			if (selected) {
				dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
				this._fireSelectionChanged(row);
			} else {
				//See references to _remainSelected for conditions where the
				//row should not be un-selected even though none of its elements
				//have focus
				if (row._remainSelected) {
					return;
				}
				
				//Removes styling on currently selected row
				row._clearSelectionTimeout = setTimeout(dojo.hitch(this, function() {
					dojo.toggleClass(row, "selected", false); //$NON-NLS-1$
				}), 0);
				
				//Causes selection to go to null
				this._clearSelectionTimeout = setTimeout(dojo.hitch(this, function() {
					this._fireSelectionChanged(null);
				}), 0);				
			}
		},
		
		_fireSelectionChanged: function(selectedRow) {
			if (this._selectedRow !== selectedRow) {
				this._selectedRow = selectedRow;
				this.selectionChanged(selectedRow === null ? null : selectedRow._item);
			}
		},
		
		_onItemChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList) {
			var parentRow = this._rowsByIdentity[this.model.getIdentity(parent)];
			var lastRow = null;
		
			this.model.getChildren(parent, dojo.hitch(this, function(childItems) {
				//Create rows for any new children
				dojo.forEach(childItems, dojo.hitch(this, function(childItem, index) {
					var row = this._rowsByIdentity[this.model.getIdentity(childItem)]; 
					if (row === undefined) {
						if (lastRow === null) {
							//Adding the first child to a parent
							
							lastRow = this._createRow(childItem, parentRow._level+1, parent, parentRow);
						} else {
							//Adding a node after a sibling
							
							if (parentRow._expanded === false) {
								this._toggleExpanded(parentRow);
							}
							
							var getLastChildNode = function(row) {
								if (row._childRows.length === 0) {
									return row;
								}
								return getLastChildNode(row._childRows[row._childRows.length-1]);
							};
							
							lastRow = this._createRow(childItem, lastRow._level, parent, getLastChildNode(lastRow));
						}
						parentRow._childRows.splice(index, 0, lastRow);
						lastRow._childRows = [];
					} else {
						lastRow = row;
					}
				}));
				
				//Show/hide the expander
				dojo.toggleClass(parentRow._expandNode, "invisible", childItems.length === 0); //$NON-NLS-1$
			}));
		},
		
		_onItemDelete: function(item) {
			var identity = this.model.getIdentity(item);
			var row = this._rowsByIdentity[identity];
			if (row) {
				this._cleanUpRow(row);
				dojo.destroy(row);
				if (row._editorRow) {
					dojo.destroy(row._editorRow);
				}
				delete this._rowsByIdentity[identity];
			}
		},
		
		/**
		 * The following functions wrap the creation of elements and widgets
		 * with code to customize for inclusion in an iteration structure
		 * row -- primarily to select the row on focus/click. They are passed
		 * to sub-widgets such as the section to customize the naming
		 * convention, to decouple this behavior from the sub-widgets.
		 */
		
		_createElement: function(tagName, properties, parentElement, position) {
			if (properties['class'] === undefined) { //$NON-NLS-1$
				properties['class'] = "selectRowOnClick"; //$NON-NLS-1$ //$NON-NLS-2$
			} else {
				properties['class'] += " selectRowOnClick"; //$NON-NLS-1$ //$NON-NLS-2$
			}
			return dojo.create(tagName, properties, parentElement, position);
		},
		
		_createMenuButton: function(row, menuButtonArgs) {
			var menuButton = new jazz.ui.toolbar.MenuButton(menuButtonArgs);
				
			//We want the row to remain selected after the "Add Iterations"
			//button has been clicked and before a menu item has been
			//selected
			var self = this;
			row._connects.push(dojo.connect(menuButton, "open", function() { //$NON-NLS-1$	
				row._remainSelected = true;
				
				self._rowSelected(row, true);
			}));
			row._connects.push(dojo.connect(menuButton, "close", function(event) { //$NON-NLS-1$	
				row._remainSelected = false;
				
				self._rowSelected(row, false);
			}));
			
			row._connects = row._connects.concat(this._selectRowOnFocus(row, menuButton.focusNode));
			
			return menuButton;
		},
		
		_createToolbarButton: function(row, buttonArgs) {
			var toolbarButton = new jazz.ui.toolbar.Button(buttonArgs);
			
			row._connects = row._connects.concat(this._selectRowOnFocus(row, toolbarButton.focusNode));
			
			return toolbarButton;
		},
		
		_createTextBox: function(row, textBoxArgs, valueChangedCallback) {
			var textBox = new dijit.form.ValidationTextBox(textBoxArgs);
			
			textBox.watch("displayedValue", function() { //$NON-NLS-1$
				valueChangedCallback(textBox.get("displayedValue")); //$NON-NLS-1$
			});
			
			row._connects = row._connects.concat(this._selectRowOnFocus(row, textBox.focusNode));
			
			return textBox;
		},
		
		_createNumberSpinner: function(row, spinnerArgs, valueChangedCallback) {
			var spinner = new dijit.form.NumberSpinner(spinnerArgs);
            
            row._connects.push(dojo.connect(spinner, "onChange", function() { //$NON-NLS-1$
            	valueChangedCallback(spinner.get("value")); //$NON-NLS-1$
            }));
            
            row._connects = row._connects.concat(this._selectRowOnFocus(row, spinner.focusNode));
            
            return spinner;
		},
		
		_createCheckbox: function(row, checkboxId, labelText, initialChecked, valueChangedCallback) {
			var label = dojo.create("label", {"for": checkboxId}); //$NON-NLS-1$ //$NON-NLS-2$
			var checkbox = dojo.create("input", { //$NON-NLS-1$
				"id": checkboxId, //$NON-NLS-1$
				type: "checkbox", //$NON-NLS-1$
				checked: initialChecked
			}, label);
			row._connects.push(dojo.connect(checkbox, "onclick", dojo.hitch(this, function(theRow) { //$NON-NLS-1$
				dojo.toggleClass(row, "selected", true); //$NON-NLS-1$
				this._rowSelected(row, true);
				
				valueChangedCallback(checkbox.checked);
			}, row)));
			row._connects = row._connects.concat(this._selectRowOnFocus(row, checkbox));
			label.appendChild(document.createTextNode("\t" + labelText)); //$NON-NLS-1$
			
			return label;
		},
		
		_cleanUpRow: function(row) {
			if (row._actionsTd && row._actionsTd._toolbar) {
				row._actionsTd._toolbar.destroyRecursive();
				delete row._actionsTd._toolbar;
			}
			
			if (row._nameTd && row._nameTd._textBox) {
				row._nameTd._textBox.destroyRecursive();
				delete row._nameTd._textBox;
			}
			
			if (row._quantityTd && row._quantityTd._quantitySpinner) {
				row._quantityTd._quantitySpinner.destroyRecursive();
				delete row._quantityTd._quantitySpinner;
			}
			
			if (row._durationTd && row._durationTd._durationSpinner) {
				row._durationTd._durationSpinner.destroyRecursive();
				delete row._durationTd._durationSpinner;
			}
			
			if (row._editorRow) {
				if (row._editorRow._iterationTypesSelect) {
					row._editorRow._iterationTypesSelect.destroyRecursive();
					delete row._editorRow._iterationTypesSelect;
				}
				
				if (row._editorRow._customizeNamingConventionButton) {
					row._editorRow._customizeNamingConventionButton.destroyRecursive();
					delete row._editorRow._customizeNamingConventionButton;
				}
				
				if (row._editorRow._customNamingConventionSection) {
					row._editorRow._customNamingConventionSection.destroyRecursive();
					delete row._editorRow._customNamingConventionSection;
				}
			}
			
			dojo.forEach(row._connects || [], dojo.disconnect);
			
			dojo.forEach(this._allInvalidMaps, function(map) {
				delete map[row._itemIdentity];
			});
		},
		
		uninitialize: function() {
			this.inherited(arguments);
			
			dojo.query("tr", this.domNode).forEach(dojo.hitch(this, this._cleanUpRow)); //$NON-NLS-1$
		}
	});
})();
