/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeSection"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var MEMENTO_SELECTED_SCOPE = "PermissionsScopeSection#selectedScope"; //$NON-NLS-1$
	var MEMENTO_FILTER_ARCHIVED = "PermissionsScopeSection#filterArchivedScopes"; //$NON-NLS-1$
	
	/**
	 * Section that's sometimes displayed in the Permissions editor, either
	 * when:
	 * 
	 * - editing permissions for a project team for which permissions have
	 *   been configured for an iteration type or timeline/iteration
	 * 
	 * - the user chooses to configure permissions for a time period
	 * 
	 * A tree of 'scopes' is displayed, where the appropriate sections in the
	 * Permissions editor react when an iteration type, a timeline/iteration,
	 * or 'Always' is selected.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeSection", [dijit._Widget], { //$NON-NLS-1$
		
		constructor: function(args) {
			this._processAreaEditor = args.editor;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._selectedRoleId = args.selectedRoleId;
			delete args.selectedRoleId;
		},
		
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsScopeSection"); //$NON-NLS-1$
			
			this._currentDevelopmentLineItem = this.processAreaStore.getCurrentDevelopmentLineItem();
			
			this._createSectionHeading();
			
			this._permissionsScopeTreeDiv = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(this._permissionsScopeTreeDiv, "scopeTree"); //$NON-NLS-1$
			
			//When permissions are changed or roles are deleted, update
			//filtering/highlighting if those things might have changed
			var updateFilteringIfNeccessary = dojo.hitch(this, function(roleId) {
				if ((roleId === this._selectedRoleId || null === this._selectedRoleId)) {
					this._updateFilterByConfiguredScopes();
				}
			});
			
			this.connect(this.workingCopy, "permissionChanged", dojo.hitch(this, function(store, item, roleId, contextItem) { //$NON-NLS-1$ 
				updateFilteringIfNeccessary(roleId);
			}));
			
			this.connect(this._processAreaEditor.getRolesStore(), "aboutToDelete", dojo.hitch(this, function(item) { //$NON-NLS-1$
				var roleId = this._processAreaEditor.getRolesStore().getValue(item, ProcessConstants.ProcessRoleDTO.ID);
				updateFilteringIfNeccessary(roleId);
			}));
			
			this.refresh();
			
			this.resized();
		},
		
		refresh: function(){
			this._clearState();
			
			this._configuredScopes = this.workingCopy.getConfiguredScopes(this._selectedRoleId);
			
			this._createScopeTree();
			dojo.empty(this._permissionsScopeTreeDiv);
			this._permissionsScopeTreeDiv.appendChild(this._scopeTree.domNode);
		},
		
		/**
		 * Called by Permissions editor when it is being cleared for a refresh
		 * or state. Updates a memento with any state that should be persisted
		 * after the editor is re-loaded, such as the selected time period.
		 */
		saveState: function(editorState) {
			if (!this._nothingSelected) {
				var selectedItems = this._scopeTree.get("selectedItems"); //$NON-NLS-1$
				if (selectedItems && selectedItems.length === 1) { 
					var selectedItem = selectedItems[0];
					if (this._scopeTree.model.store.hasAttribute(selectedItem, "contextId")) { //$NON-NLS-1$
						editorState[MEMENTO_SELECTED_SCOPE] = this._scopeTree.model.store.getValue(selectedItem, "contextId"); //$NON-NLS-1$
					}
				}
			}
			
			editorState[MEMENTO_FILTER_ARCHIVED] = this._filterArchivedButton.pressed;
		},
		
		startLoading: function() {
			this._scopeTree.setDisabled(true);
		},
		
		stopLoading: function() {
			this._scopeTree.setDisabled(false);
		},
		
		_clearState: function(){
			if (this._scopeTree) {
				this._scopeTree.destroyRecursive();
				delete this._scopeTree;
			}
		},
		
		/**
		 * Select the root node 'Always'. If the node is not visible -- it must
		 * be the case that the toggle to only show configured scopes is
		 * pressed -- the toggle to show configured scopes is first
		 * programatically un-pressed.
		 */
		selectAlways: function() {
			if (!this._isNodeForItemVisible(this._model.store.alwaysRootItem)) {
				this._filterConfiguredButton.set("pressed", false); //$NON-NLS-1$
				this._updateFilterByConfiguredScopes();
			}
			this._scopeTree.focusItem(this._model.store.alwaysRootItem);
		},
		
		/**
		 * Clear the selection in the tree, and notify listeners such that
		 * a message that a time period must be selected is displayed.
		 */
		_selectNothing: function() {
			this._nothingSelected = true;
			this._scopeTree.set("selectedItems", []); //$NON-NLS-1$
			this.timePeriodSelected(null, dojo.string.substitute("<div class=\"j-label\">${0}</div>", [this.messages.PermissionsScopeSection_3])); //$NON-NLS-1$
		},
		
		/**
		 * To be called when the container of this tab is activated, to give the
		 * section a chance to lazily respond to actions that were performed
		 * outside of the tab, (the widget doesn't act immediately upon some
		 * changes to stores because notifying listeners could cause loading
		 * that's only appropriate when the container of the widget is active).
		 */
		tabActivated: function() {
			//If the selected item was archived or deleted, reset selection
			if ((this._scopeTree.isArchivedItemSelected() && !this._scopeTree.model.showArchived) ||
				(this._scopeTree.lastFocused && !this.processAreaStore.isItem(this._scopeTree.lastFocused.item)) ||
				!this._scopeTree.selectedItem) {
				setTimeout(dojo.hitch(this, function() {
					this._selectNothing();
				}));
			}
			
			var selectedDevelopmentLineItem = this.processAreaStore.getCurrentDevelopmentLineItem();
			if (this._currentDevelopmentLineItem !== selectedDevelopmentLineItem) {
				this._currentDevelopmentLineItem = selectedDevelopmentLineItem;
				
				if (this._scopeTree.isIterationItemSelected()) {
					// reset selection if the last focused item is iteration
					// since the iteration structure will be changed at all
					setTimeout(dojo.hitch(this, function() {
						this._selectNothing();
					}));
				}
					
				this._scopeTree.update();
			}
		},
		
		/**
		 * To be called when the size of the container of this section changes,
		 * so that the contents of the widget can be layed out to make the best
		 * use of the space available.
		 */
		resized: function() {
			setTimeout(dojo.hitch(this, function() {
				//Make sure we haven't been destroyed, (this function may be
				//called when parent containers are destroyed)
				if (this.domNode) {
					var parentBox = dojo.marginBox(this.domNode);
					
					this._sectionHeading.layout();
					
					var headingBox = dojo.marginBox(this._sectionHeading.domNode);
					//8 pixels are subtracted from the space the tree would normally
					//have so that there is uniform space above and below the
					//splitter between sections in the Permissions editor
					var availableTreeHeight = parentBox.h - headingBox.h - dojo.style(this._permissionsScopeTreeDiv, "marginTop") - 8; //$NON-NLS-1$
					dojo.style(this._permissionsScopeTreeDiv, "height", availableTreeHeight + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}), 0);
		},
		
		/**
		 * Update the role for which permissions scopes are to be shown. This
		 * impacts which scopes are highlighted and which can be filtered.
		 * 
		 * Null can be passed to indicate that scopes across all roles should
		 * be shown.
		 */
		setSelectedRoleId: function(roleId) {
			if (this._selectedRoleId !== roleId) {
				this._selectedRoleId = roleId;
				this._updateFilterByConfiguredScopes();
				
				this._filterConfiguredButton.set("title", this._getFilterConfiguredTooltipTitle()); //$NON-NLS-1$
			}
		},
		
		_createSectionHeading: function() {
			this._sectionHeading = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading({
				headingText: this.messages.PermissionsScopeSection_1
			});
			this.domNode.appendChild(this._sectionHeading.domNode);
			
			this._toolbar = new jazz.ui.Toolbar();
			this._sectionHeading.addWidget(this._toolbar);
			
			this._filterArchivedButton = new jazz.ui.toolbar.TwoStateButton({
				title: this.messages.PermissionsScopeSection_2,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/milestone_archived_obj.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				pressed: this.editorState && this.editorState[MEMENTO_FILTER_ARCHIVED] === true,
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this._scopeTree.model.showArchived = this._filterArchivedButton.pressed;
					this._scopeTree.update();
					if (this._scopeTree.isArchivedItemSelected() && !this._scopeTree.model.showArchived) {
						this._selectNothing();
					}	
				})
			});
			this._toolbar.addChild(this._filterArchivedButton);
			
			this._filterConfiguredButton = new jazz.ui.toolbar.TwoStateButton({
				title: this._getFilterConfiguredTooltipTitle(),
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/filter-timeline_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this._updateFilterByConfiguredScopes();
				})
			});
			this._toolbar.addChild(this._filterConfiguredButton);
			
			this._scopePageHoverHelp = new jazz.app.ua.PageHoverHelp({
				contentNode: this._createScopeLookupHelpNode()
			});
			this._sectionHeading.placeHelpNode(this._scopePageHoverHelp.domNode);
		},
		
		_getFilterConfiguredTooltipTitle: function() {
			var title = this.messages.PermissionsScopeSection_4;
			if (null !== this._selectedRoleId) {
				var title;
				var query = {};
				query[ProcessConstants.ProcessRoleDTO.ID] = this._selectedRoleId;
				this._processAreaEditor.getRolesStore().fetch({
					query: query,
					onItem: dojo.hitch(this, function(item) {
						title = dojo.string.substitute(this.messages.PermissionsScopeSection_5, [this._processAreaEditor.getRolesStore().getRoleLabel(item)]);
					})
				});
			}
			return title;
		},
		
		/**
		 * Updates the model with information about which scopes to filter to,
		 * if any, which may cause the tree to be refreshed.
		 * 
		 * After the model is updated, if we are filtering to show a sub-set
		 * of scopes, the tree nodes are expanded to reveal those scopes.
		 * 
		 * Finally, the previously selected node is selected if it is still
		 * visible, and 'nothing' is selected if it is not.
		 */
		_updateFilterByConfiguredScopes: function() {
			var pathsArray = [];
			this._configuredScopes = this.workingCopy.getConfiguredScopes(this._selectedRoleId);
			if (this._filterConfiguredButton.pressed) {
				this._scopeTree.model.filteredContextIds = this._addParentItems(this._configuredScopes, pathsArray); 
			} else {
				delete this._scopeTree.model.filteredContextIds;
			}
						
			this._scopeTree.update();
			
			if (this._filterConfiguredButton.pressed) {
				//Expand nodes in tree to reveal configured scopes
				this._scopeTree.set("paths", pathsArray).then(dojo.hitch(this, function() { //$NON-NLS-1$
					this._scopeTree.set("paths", []); //$NON-NLS-1$
				}));
			}
			
			var nodes = this._scopeTree.getNodesByItem(this._scopeTree.lastFocused.item);
			var isInTree = dojo.hitch(this, function(node) {
				node = node.parentElement;
				while (node !== undefined && node !== null) {
					if (node === this._scopeTree.domNode) {
						return true;
					}
					node = node.parentElement; 
				}
				return false;
			});
			if (!this._isNodeForItemVisible(this._scopeTree.lastFocused.item)) {
				this._selectNothing();
			} else if (!this._nothingSelected) {
				this._scopeTree.set("selectedItems", [this._scopeTree.lastFocused.item]); //$NON-NLS-1$
			}
		},
		
		_isNodeForItemVisible: function(item) {
			var nodes = this._scopeTree.getNodesByItem(item);
			var isInTree = dojo.hitch(this, function(node) {
				node = node.parentElement;
				while (node !== undefined && node !== null) {
					if (node === this._scopeTree.domNode) {
						return true;
					}
					node = node.parentElement; 
				}
				return false;
			});
			return nodes.length === 1 && nodes[0] && nodes[0].domNode && isInTree(nodes[0].domNode);
		},
		
		/**
		 * Given a mapping of scope items that are to be filtered to reveal,
		 * adds mappings for all parent scope items.
		 * 
		 * As a side effect, arrays are added to the output parameter
		 * pathsArray that can be used to expand nodes in the tree to reveal
		 * the original items in the configuredScopes map.
		 */
		_addParentItems: function(configuredScopes, pathsArray) {
			var expandedConfiguredScopes = {};
			for (var contextId in configuredScopes) {
				expandedConfiguredScopes[contextId] = true;
				
				var path = [];
				var enabledContext = this._scopeTree.model.store.getValue(configuredScopes[contextId], "enabledContext"); //$NON-NLS-1$
				//Don't add the PROJECT or TEAM container items to the path,
				//since they are not represented by nodes
				if (ProcessConstants.PermissionContext.PROJECT !== enabledContext && ProcessConstants.PermissionContext.TEAM !== enabledContext) {
					path.unshift(configuredScopes[contextId]);
				}
				pathsArray.push(path);
				
				var parentItem = this._scopeTree.model.store.getParent(configuredScopes[contextId]);
				while (parentItem !== null) {
					//Only add development line nodes to the path if we're
					//editing a project area; otherwise the nodes are not
					//in the tree
					if (this._fromProjectAreaEditor() || ProcessConstants.PermissionContext.DEV_LINE !== this._scopeTree.model.store.getValue(parentItem, "enabledContext")) { //$NON-NLS-1$
						path.unshift(parentItem);
					}
					expandedConfiguredScopes[this._scopeTree.model.store.getSavedContextId(parentItem)] = parentItem;
					parentItem = this._scopeTree.model.store.getParent(parentItem);
				}
				
				path.unshift(this._scopeTree.model.root);
			}
			return expandedConfiguredScopes;
		},
		
		_getScopeLookupContent: function() {
			var listItem = "<li>${0}</li>"; //$NON-NLS-1$
			var content = this.messages.PermissionsScopeSection_RuntimeLookupHeader;
			content += "<ul>"; //$NON-NLS-1$
			content += dojo.string.substitute(listItem, [this.messages.PermissionsScopeSection_RuntimeLookupListItem1]);
			content += dojo.string.substitute(listItem, [this.messages.PermissionsScopeSection_RuntimeLookupListItem2]);
			content += dojo.string.substitute(listItem, [this.messages.PermissionsScopeSection_RuntimeLookupListItem3]);
			content += dojo.string.substitute(listItem, [this.messages.PermissionsScopeSection_RuntimeLookupListItem4]);
			content += "</ul>"; //$NON-NLS-1$
			return content;
		},
		
		_getContentForIterationTypesSelected: function() {
			var content = this.messages.PermissionsScopeSection_IterationTypesNodeMessage;
			content += "<br><br>"; //$NON-NLS-1$
			content += this._getScopeLookupContent();
			return content;
		},
		
		_getContentForTimelinesSelected: function() {
			var content = this.messages.PermissionsScopeSection_TimelinesNodeMessage;
			content += "<br><br>"; //$NON-NLS-1$
			content += this._getScopeLookupContent();
			return content;
		},
		
		_getContentForIterationsSelected: function() {
			var content = this.messages.PermissionsScopeSection_IterationsNodeMessage;
			content += "<br><br>"; //$NON-NLS-1$
			content += this._getScopeLookupContent();
			return content;
		},
		
		_createScopeLookupHelpNode: function() {
			return dojo.create("div", {innerHTML: this._getScopeLookupContent()}); //$NON-NLS-1$
		},
		
		_createScopeTree: function() {
			this._model = this._getModel();
			
			if (this.editorState && this.editorState[MEMENTO_FILTER_ARCHIVED] === true) {
				this._model.showArchived = true;
			} 
			
			var self = this;
			var editorContext = self._fromProjectAreaEditor() ? ProcessConstants.EditorContext.PROJECT : ProcessConstants.EditorContext.TEAM;
			this._scopeTree = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree({
				model: this._model,
				editorContext: editorContext,
				getLabelClass: dojo.hitch(this, function(item) {
					if (item !== this._model.root) {
						//Skip the root item, and do not highlight Always unless
						//we're editing a child area
						if (this._configuredScopes !== undefined && (this.workingCopy.isChildArea() || item !== this._model.store.alwaysRootItem)) {
							var contextId = this._model.store.getSavedContextId(item);
							if (this._configuredScopes[contextId] === item) {
								return "configuredScopeLabel"; //$NON-NLS-1$
							}
						}
					}
				}),
				focusChanged: function(node) {
					delete self._nothingSelected;
					var id = self._model.getIdentity(node.item);
					if (id === ProcessConstants.PermissionsScopeContext.ITERATION_TYPES) {
						self.timePeriodSelected(null, self._getContentForIterationTypesSelected());
					} else if (id === ProcessConstants.PermissionsScopeContext.DEV_LINES) {
						if (editorContext === ProcessConstants.EditorContext.PROJECT) {
							self.timePeriodSelected(null, self._getContentForTimelinesSelected());
						} else {
							self.timePeriodSelected(null, self._getContentForIterationsSelected());
						}
					} else {
						self.timePeriodSelected(node.item);
					}
				}
			});
			
			//Select the scope that was selected before a refresh/save, if such
			//a scope is available, after first expanding its parent nodes in
			//the tree
			if (this.editorState && this.editorState[MEMENTO_SELECTED_SCOPE] !== undefined) {
				this.processAreaStore.fetch({
					query: {
						"contextId": this.editorState[MEMENTO_SELECTED_SCOPE] //$NON-NLS-1$
					},
					queryOptions: {
						deep: true
					},
					onItem: dojo.hitch(this, function (item) {
						var pathsArray = [];
						var items = {};
						items[this.editorState[MEMENTO_SELECTED_SCOPE]] = item;
						this._addParentItems(items, pathsArray);
						this._scopeTree.set("paths", pathsArray).then(dojo.hitch(this, function() { //$NON-NLS-1$
							this._scopeTree.set("paths", []); //$NON-NLS-1$
							
							this._scopeTree.focusItem(item);
						}));
					})
				})
			}
		},
		
		_getModel: function(){
			return new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel({
				store: this.processAreaStore,
				childrenAttrs: ["items"], //$NON-NLS-1$
				showArchived: this._filterArchivedButton.pressed
			});
		},
		
		_fromProjectAreaEditor: function(){
			return this._processAreaEditor.declaredClass == "com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"; //$NON-NLS-1$
		},
		
		uninitialize: function() {
			if (this._sectionHeading) {
				this._sectionHeading.destroyRecursive();
				delete this._sectionHeading;
			}
			
			if (this._scopeTree) {
				this._scopeTree.destroyRecursive();
				delete this._scopeTree;
			}
			
			if (this._scopePageHoverHelp) {
				this._scopePageHoverHelp.destroyRecursive();
				delete this._scopePageHoverHelp;
			}
			
			delete this._processAreaEditor;
		}
	});
})();
