/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.AdminHome");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.schema.AdminHome");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.DataTableContainer");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminHomeMessages");

(function() {

var MessageBox = jazz.ui.MessageBox;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var AdminHomeSchema = com.ibm.team.repository.web.ui.internal.admin.server.schema.AdminHome;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var UserRegistryClient = com.ibm.team.repository.web.client.internal.userRegistry;
var HttpConstants = com.ibm.team.repository.web.client.internal.HttpConstants;
var DataTableContainer = com.ibm.team.repository.web.ui.internal.admin.DataTableContainer;
var DiagnosticsUtil = com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil;
var XhrUtil = jazz.app.internal.util.XhrUtil;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var JFS_NS = "http://jazz.net/xmlns/foundation/1.0/";
var LPA_IDENTIFIER = "http://jazz.net/application/lpa";
var SETUP_HAS_RUN_PROPERTY = "completed.setup";
var SETUP_CONFIGURATION_SERVICE = "com.ibm.team.repository.service.internal.setup.SetupConfigurationService";

var CREATED_USERS_THRESHOLD = 3; //# of non-functional users that need to be in the repo before we check it off in the task guide
var ASSIGNED_CALS_THRESHOLD = 2; //# of unique users with non-functional licenses assigned before we check it off in the task guide

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminHomeMessages");

var BASE_INFOCENTER_URL = "http://pic.dhe.ibm.com/infocenter/clmhelp/v4r0m5/topic/";
var HELP_LINKS = {
	1: { label: messages.aboutAdministeringTheJTS, topic: "com.ibm.jazz.repository.web.admin.doc/topics/tworkwithadminwebui.html"},
	2: { label: messages.aboutCreatingUsers, topic: "com.ibm.jazz.repository.web.admin.doc/topics/taddnewuser.html"},
	3: { label: messages.aboutImportingUsers, topic: "com.ibm.jazz.repository.web.admin.doc/topics/t_users_import.html"},
	4: { label: messages.aboutProjects, topic: "com.ibm.jazz.platform.doc/topics/c_creating_managing_cross_application_projects.html"},
	5: { label: messages.aboutAddingMembers, topic: "com.ibm.jazz.platform.doc/topics/t_adding_members_to_projects.html"},
	6: { label: messages.aboutAddingUsers, topic: "com.ibm.team.platform.doc/topics/c_add_mod_users_web.html"},
	7: { label: messages.aboutManagingUsers, topic: "com.ibm.jazz.repository.web.admin.doc/topics/tmanageusers.html"}	
};

var ACTION_LINKS = {
	setupServer: { className: "", label: messages.setupServer, url: "/setup" },
	manageServer: { className: "manageServer", label: messages.manageServer, url: "#action=jazz.viewPage&id=com.ibm.team.repository.server" },
	manageUsers: { className: "manageUsers", label: messages.manageUsers, url: "#action=com.ibm.team.repository.manageUsers" },
	importUsers: { className: "importUsers", label: messages.importUsersAction, url: "#action=com.ibm.team.repository.manageUsers&importUsers=true" },
	createUsers: { className: "createUsers", label: messages.createUsersAction, url: "#action=com.ibm.team.repository.editUser&itemId=new" },
	personalDashboards: { className: "personalDashboards", label: messages.personalDashboards, url: "/web" },
	editProfile: { className: "editProfile", label: messages.editProfile, url: "#action=com.ibm.team.repository.viewUserProfile&itemId=" },
	syncJts: { className: "syncJts", label: messages.syncJts },
	assignCALS: { className: "assignCALS", label: messages.assignCALS, url: "#action=com.ibm.team.repository.manageUserCALs" },
	createProject: { className: "createProject", label: messages.createProject, url: "/web/projects/new" },
	manageProjects: { className: "manageProjects", label: messages.manageProjects, url: "/web/projects" },
	trySample: { className: "infoLink", label: messages.trySample, url: "/web/sample" }
};

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AdminHome", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AdminHome.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function() {
		this.messages = messages;
		this.JAZZ_ROOT = net.jazz.ajax.contextRoot();
	},
	
	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);

		var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		actionRegistry.registerAction("com.ibm.team.repository.admin.adminHome", this, "checkDBStatus");
		
		this.diagUtil = new DiagnosticsUtil();
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
		//Add images next to the section headers
		dojo.create("span", { className: "spritedIcon connectedIcon" }, this._headerManageServer);
		dojo.create("span", { className: "spritedIcon usersIcon" }, this._headerManageUsers);
		dojo.create("span", { className: "spritedIcon lpaIcon" }, this._headerManageLPA);
			
		//Action links for Manage Server
		dojo.create("li", {}, this.serverLinks).appendChild(this.createLink(ACTION_LINKS["manageServer"]));
		dojo.create("li", {}, this.serverLinks).appendChild(this.createHelpLink(HELP_LINKS[1]));
		
		//Action links for Manage Profile
		var personalDashboardsLink = ACTION_LINKS["personalDashboards"];
		personalDashboardsLink.contextRoot = this.JAZZ_ROOT;
		dojo.create("li", {}, this.profileLinks).appendChild(this.createLink(personalDashboardsLink));
		
		//Get & Append the itemId for the current contributor
		var ac = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR");
		if(ac && ac.itemId) {
			var editProfileLink = ACTION_LINKS["editProfile"];
			editProfileLink.url += ac.itemId;
			dojo.create("li", {}, this.profileLinks).appendChild(this.createLink(editProfileLink));
		}

		//Add User/Project Counts to Section headers
		this.userCountDIV = dojo.create("span", { className: "status" }, this._headerManageUsers);
		this.projectsCountDIV = dojo.create("span", { className: "status" }, this._headerManageLPA);
	},
	
	checkDBStatus: function() {
		var srh = new ServiceResponseHandler(this, "handleDBStatus", "handleDatabaseErrorReponse");
		ServerStatusClient.getDatabaseStatus(srh);	
	},
	
	handleDBStatus: function(status){
		this.dbError = (status.code && status.code === "ERROR");
		if (this.dbError) {
			var statusSummaryLink = "<a href=\"#action=com.ibm.team.repository.admin.serverStatus\">";
			this.header.showMessage({
				type: status.code,
				summary: dojo.string.substitute(this.messages.dbErrorOccurred, [statusSummaryLink, "</a>"])
			});
		} 
		else {
			this.setUserRoles();	
		}
    },
	
	handleDatabaseErrorReponse: function(errorObj) {
		console.error("An error occurred while trying to determine the database status.");
		this.setUserRoles();
	},
	
	reload: function () {
		if(this.dbError) return;
		this.header.hideMessage();

		if(this.isAdmin) {
			dojo.removeClass(this.serverAdminHeader, "noDisplay");
			dojo.removeClass(this.showServer, "noDisplay");
			this.fetchServerHealth();
			this.getSetupStatus();
			this.getFunctionalUsers();
			this.getAssignedCALs();
		} else {
			//Destroy the DOM that we don't need
			this._ManageUsers.destroyRecursive();
			this._ManageServer.destroyRecursive();
		}
		
		var data = dojo.fromJson(localStorage["com-ibm-team-repository-web-SetupHasFinished"]);
		if(data && data.fromSetup) {
			this.header.showMessage({
				type: "OK",
				summary: this.messages.setupJustCompleted,
				closable: true
			});
			localStorage["com-ibm-team-repository-web-SetupHasFinished"] = null;
		}
		
		this._reloadApplicationLinks();
	},
	
	buildPage: function() {
		dojo.empty(this.userLinks);
		dojo.empty(this.taskGuideTasks);
		this.taskEntries = [];
		
		if(this.isAdmin) {
			var setupAction = ACTION_LINKS["setupServer"];
			setupAction.contextRoot = this.JAZZ_ROOT;
			this.taskEntries["setupServer"] = dojo.create("li", { className: "incomplete" }, this.taskGuideTasks);
			this.taskEntries["setupServer"].appendChild(this.createLink(setupAction));

			this.taskEntries["createUsers"] = dojo.create("li", { className: "incomplete noDisplay" }, this.taskGuideTasks);
			this.taskEntries["createUsers"].appendChild(this.createLink(ACTION_LINKS["createUsers"]));
			
			this.taskEntries["importUsers"] = dojo.create("li", { className: "incomplete noDisplay" }, this.taskGuideTasks);
			this.taskEntries["importUsers"].appendChild(this.createLink(ACTION_LINKS["importUsers"]));

			//Create/Import Action link
			if(this.isTomcat || this.isJetty || this.isExternalRegistry) {
				dojo.create("li", {}, this.userLinks).appendChild(this.createLink(ACTION_LINKS["createUsers"]));
				dojo.removeClass(this.taskEntries["createUsers"], "noDisplay");
				this.usersDescription.innerHTML = this.messages.manageUsersDescCreate;
			} else if(this.isLDAP) {
				this.syncLink = dojo.create("li", {}, this.userLinks);
				dojo.create("li", {}, this.userLinks).appendChild(this.createLink(ACTION_LINKS["importUsers"]));
				dojo.removeClass(this.taskEntries["importUsers"], "noDisplay");
				this.usersDescription.innerHTML = this.messages.manageUsersDescImport;
			}
			
			//Check to see if the User Registry supports Import/Create actions. If not, hide them.
			this.checkRegistrySupportedActions();
			
			dojo.create("li", {}, this.userLinks).appendChild(this.createLink(ACTION_LINKS["manageUsers"]));
			
			//Assign Licenses Action link
			this.assignLink = dojo.create("li", {}, this.userLinks);
			this.assignLink.appendChild(this.createLink(ACTION_LINKS["assignCALS"]));
			this.taskEntries["assignCALS"] = dojo.create("li", { className: "incomplete" }, this.taskGuideTasks);
			this.taskEntries["assignCALS"].appendChild(this.createLink(ACTION_LINKS["assignCALS"]));
			dojo.removeClass(this.showUsers, "noDisplay");
			
			//Create/Import users help link
			if(this.isTomcat || this.isExternalRegistry) {
				dojo.create("li", {}, this.userLinks).appendChild(this.createHelpLink(HELP_LINKS[2]));
			} else if(this.isLDAP) {
				dojo.create("li", {}, this.userLinks).appendChild(this.createHelpLink(HELP_LINKS[3]));
			}
			dojo.create("li", {}, this.userLinks).appendChild(this.createHelpLink(HELP_LINKS[7]));
		}
		
		//Task Guide is JazzAdmins only - for now
		if(this.isAdmin) {
			this.taskEntries["createLPA"] = dojo.create("li", { className: "incomplete noDisplay" }, this.taskGuideTasks);
			//Add Task Guide Help Links
			dojo.empty(this.taskGuideHelpLinks);
			this.trySampleTaskEntry = dojo.create("li", { className: "noDisplay" }, this.taskGuideHelpLinks);
			this.lpaTaskEntry = dojo.create("li", { className: "noDisplay" }, this.taskGuideHelpLinks);
			this.lpaTaskEntry.appendChild(this.createHelpLink(HELP_LINKS[5]));
			dojo.create("li", {}, this.taskGuideHelpLinks).appendChild(this.createHelpLink(HELP_LINKS[6]));
		}
		if(this.isAdmin) {
			dojo.removeClass(this.sidebar, "noDisplay");
		} else {
			//Stretch the mainContent the full width since we don't have a sidebar
			dojo.addClass(this.mainContentNode, "fullwidth");
		}
		this.reload();
	},
	
	_testLPA: function(abouts) {
		//See if we already know about LPA before checking again...
		if(this.lpaURL) {
			this._showLpaLinks();
			return;
		}
		
		//Let's see if the LPA application is installed
		for(i in abouts) {
			if(abouts[i].identifier == LPA_IDENTIFIER) {
				var parts = i.split("/application-about");
				this.lpaURL = parts[0];
				
				//Action links for Manage LPA Projects
				var createProjectLink = ACTION_LINKS["createProject"];
				createProjectLink.contextRoot = this.lpaURL;
				dojo.create("li", {}, this.lpaLinks).appendChild(this.createLink(createProjectLink));
				var manageProjectsLink = ACTION_LINKS["manageProjects"];
				manageProjectsLink.contextRoot = this.lpaURL;
				dojo.create("li", {}, this.lpaLinks).appendChild(this.createLink(manageProjectsLink));
				var trySampleLink = ACTION_LINKS["trySample"];
				trySampleLink.contextRoot = this.lpaURL;
				dojo.create("li", {}, this.lpaLinks).appendChild(this.createLink(trySampleLink));
				dojo.create("li", {}, this.lpaLinks).appendChild(this.createHelpLink(HELP_LINKS[4]));
				
				//These are for Project Admins who only see the Task Guide and JTS Admin section if LPA is installed
				dojo.removeClass(this.serverAdminHeader, "noDisplay");
				dojo.removeClass(this.sidebar, "noDisplay");
				
				this._showLpaLinks();
				break;
			}
		}
	},
	
	_showLpaLinks: function() {
		//Create Task Guide entries
		var createProjectLink = ACTION_LINKS["createProject"];
		createProjectLink.contextRoot = this.lpaURL;
		createProjectLink.label = this.messages.createLPA;
		dojo.place(this.createLink(createProjectLink), this.taskEntries["createLPA"], "only");
		
		var trySampleLink = ACTION_LINKS["trySample"];
		//Custom link to prevent long label from wrapping under the icon
		var aLink = dojo.create("a", { href: this.lpaURL + trySampleLink.url });
		dojo.create("span", { className: "spritedIcon infoLink", style: { "float": "left" } }, aLink);
		dojo.create("div", { className: "helpLink", innerHTML: this.messages.trySampleInLPA }, aLink);
		dojo.place(aLink, this.trySampleTaskEntry, "only");
		
		//Show relevant LPA links and section
		dojo.removeClass(this.showLPA, "noDisplay");
		dojo.removeClass(this.taskEntries["createLPA"], "noDisplay");
		dojo.removeClass(this.lpaTaskEntry, "noDisplay");
		dojo.removeClass(this.trySampleTaskEntry, "noDisplay");
		
		this.getLpaProjectCount();
	},
	
	_reloadApplicationLinks: function () {
        dojo.empty(this.applicationLinks);		
		this.services = [];
	    var deferred = DiscoveredService.loadDiscoveryServices(JFS_NS + "AdminHome", AdminHomeSchema.discovery);
		deferred.addCallback(dojo.hitch(this, function(services) {
			if (!services.AdminHome)	{
				console.warn("No AdminHome entries found.");
				return;
			}
			var abouts = jazz.app.internal.applicationAbout.get();
			
			if(this.isAdmin) {
				//Use this opportunity to test for the LPA presence
				this._testLPA(abouts);	
			}

			dojo.forEach(services.AdminHome, function(service) {
				service.about = abouts[service.applicationAbout];
				service.title = service.about.title + " (" + service.publisher + ")";
				this.services.push(service);
			}, this);
			if (this.services.length) {
				this.services.sort(function(a, b){if (a.title==b.title) return 0;return (a.title<b.title)?-1:1;});
				var oldAbout = null;				
				dojo.forEach(this.services, function(service){
					this._callService(service, true, oldAbout);
					oldAbout = service.about;
				}, this);
			}
		}));
		deferred.addErrback(function(error){
			console.warn("[AdminHome] Service unreachable: ", error);
		}); 		
	},

	_callService: function(service, passive, oldAbout) {
		var def = XhrUtil.getXml(service.service, {schema:AdminHomeSchema.adminHomeService, passive:passive});
		def.addCallback(dojo.hitch(this, function (doc) {
			service.results = [];
			service.entryCount = 0;
			if (doc) {
				if (doc.entries) {
					dojo.forEach(doc.entries, function(item){
						service.entryCount++;
						service.results.push(item);
					}, this);
					if (!service.entryCount) {
						service.results.splice(0, 0, "noEntries");
					}
				}
			}
			else {
				service.results.push("authRequired");
			}
			this._updateUI(service, oldAbout);
		}));
		def.addErrback(function(error) {
			// fail silently if user closed the auth popup
			if (!error || error.dojoType !== "cancel") {
				console.error(error);
			}
		});
	},

	_updateUI: function (service, oldAbout) {
		if (service.about && service.about != oldAbout) {
			// insert an application row
			var appnode = dojo.create("div", { className: "app-type" }, this.applicationLinks);
			var inode = dojo.create("img", {}, appnode);
			inode.src = service.about.icon;
			dojo.create("span", { innerHTML: service.about.title }, appnode);
			if(service.about.instanceName) {
				dojo.create("span", { className: "instanceName", innerHTML: "(" + service.about.instanceName +")" }, appnode);
			}
			
			//Jazz Guests cannot see any action/admin links
			if(!this.isGuest) {
				dojo.removeClass(this.showApplications, "noDisplay");
			} else {
				return;
			}
		}
		
		//TODO: Delete the "return" here but make sure applications pass user-appropriate links!
		if(this.isUser){
			this._headerManageApps.innerHTML = this.messages.viewApplications;
		}
		
		//Add all of the Action & Admin Links
		var actionLinks = [];
		var normalLinks =  []; //aka "admin links"
		var ACTION_USAGE = "http://jazz.net/ns/ui/usage#action";	
		var usersRoles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		if(service.results && service.results.length > 0) {
			for (var i=0; i < service.results.length; i++) {
				var entry = service.results[i];
				var enode = dojo.create("div", { className: "app-link" });
				if(entry.usage && entry.usage === ACTION_USAGE) {
					//Check to see if this link has restricted visibility. (Only Action links support this for 4.0)
					if(entry.visibility && entry.visibility.roles) {
						var allowedRoles = entry.visibility.roles;
						var showLink = false;
						if(usersRoles) {
							inner: for (var r=0; r < allowedRoles.length; r++) {
								var inArray = dojo.some(usersRoles, function(role){ return role == allowedRoles[r] });
								if(inArray) {
									showLink = true;
									break inner;
								}
							}	
						}
						if(!showLink) continue; //user does not have permission to see this link
					}
					var aLink = dojo.create("a", { href: entry.href }, enode);
					var promotedCls = entry.promoted === "true" ? "promoted" : "";
					dojo.create("img", { src: entry.icon, alt: entry.title }, aLink);
					dojo.create("span", { className: "actionLink " + promotedCls, innerHTML: entry.title }, aLink);
					actionLinks.push(enode);
				} else {
					dojo.create("span", { className: "spritedIcon customBullet" }, enode);
					var link = dojo.create("a", { href: entry.href, innerHTML: entry.title  }, enode);
					normalLinks.push(enode);	
				}
			}
			dojo.forEach(actionLinks, function(link) {
				this.applicationLinks.appendChild(link);
			}, this);
			
			if(this.isUser) return; //Jazz Users cannot see normal action links
			
			dojo.create("div", { className: "app-text", innerHTML: messages.linksToAppPages }, this.applicationLinks);	
			dojo.forEach(normalLinks, function(link) {
				this.applicationLinks.appendChild(link);
			}, this);
		}
	},
	
	fetchProjectInfo: function() {
		var deferred = new dojo.Deferred();
		jazz.client.xhrGet({
			url: this.lpaURL + "/projects/L0",
			handleAs: "json",
			headers: this.headers,
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		return deferred;
	},
	
	getLpaProjectCount: function() {
		if(this.lpaURL) {
			var dfd = this.fetchProjectInfo();
			//Code from LifecycleProjectAdminClient:getProjectList()
			dfd.addCallback(dojo.hitch(this, function(subjects){
				if(!this.domNode) return;
				var projects = [];
				for (var subjectUri in subjects) {
					var subject = subjects[subjectUri];
					var type = this.getRdfValue(subject, "http://www.w3.org/1999/02/22-rdf-syntax-ns#type")[0];
					var title = this.getRdfValue(subject, "http://purl.org/dc/terms/title")[0];
					if (type === "http://jazz.net/xmlns/foundation/lpa/1.0/Project") {
						projects.push({
							uri: subjectUri,
							label: title,
						});
					}
				}

				var disp = "0";
				if(projects.length > 0) {
					disp = projects.length;
					this.taskEntries["createLPA"].className = "successful";
				} else {
					disp = messages.noProjectsYet;
				}
				this.projectsCountDIV.innerHTML = "("+ disp + ")";
			}));
			
			dfd.addErrback(function(errObject){
				//todo
			});
		}
	},
	
	getUserCount: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				if(!this.domNode) return;
				if(response) {
					var disp = "0";
					this.totalUserCount = response.count;
					
					if(this.totalUserCount === 0 && this.assignLink) {
						dojo.addClass(this.assignLink, "noDisplay");
					}
					
					if(this.totalUserCount > 0) {
						disp = this.totalUserCount;
					} else if(this.isLDAP) {
						disp = this.messages.noUsersImported;
					} else if(this.isTomcat || this.isExternalRegistry) {
						disp = this.messages.noUsersCreated;
					}
					this.userCountDIV.innerHTML = "(" + disp + ")";
					
					//Update the task guide
					var totalNonFunctional = this.functionalUsers ? this.totalUserCount - this.functionalUsers.length : this.totalUserCount;
					if(totalNonFunctional >= CREATED_USERS_THRESHOLD) {
						dojo.removeClass(this.taskEntries["createUsers"], "incomplete");
						dojo.removeClass(this.taskEntries["importUsers"], "incomplete");
						dojo.addClass(this.taskEntries["createUsers"], "successful");
						dojo.addClass(this.taskEntries["importUsers"], "successful");
					}
				} 
			}),
			error: function(){} //do nothing, not critical
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		AdminClient.getContributors({pageSize: 1, hideAdmin: true, hideGuest: true, hideArchivedUsers: true}, srh);
	},
	
	setUserRegistryType: function(value) {
		if(!value) {
			console.error("Unable to determine user registry type.");
		} else {
			value = value.toUpperCase();	
		}
		if(value == "LDAP" || value == "LDAPLocalGroup") {
			this.isLDAP = true;
			this.doesRegistrySupportSync();
		}
		if(value == "DETECT" || value == "TOMCAT") {
			this.isTomcat = true;
		}
		if(value == "JETTY") {
			this.isJetty = true;
		}
		if(value == "UNSUPPORTED") {
			this.isExternalRegistry = true;
			this.doesRegistrySupportSync();
		}
		this.buildPage();
	},
	
	handleUserRegistryError: function(errorObject) {		
		//Since we can't figure out the registry, just set it to the default: Tomcat
		this.isTomcat = true;
		this.buildPage();
	},
	
	doesRegistrySupportSync: function() {
		var srh = new ServiceResponseHandler(this, "handleSyncDetails", "handleSyncDetectError");
		UserRegistryClient.getIsExternalRegistrySynchronizeable(srh);
	},
	
	handleSyncDetails: function(response) {
		if(!this.domNode) return;
		if(response === "true") {
			this.syncSupported = true;
			var clickHandlerMethod = dojo.hitch(this, function(e) {
				dojo.stopEvent(e);
				if(this.syncLink._aLink && dojo.hasClass(this.syncLink._aLink, "disabled") === true) {
					return; //don't run another sync	
				}
				var srh = new ServiceResponseHandler(this, "isSyncInProgress", "handleSyncRequestFailed");
				UserRegistryClient.postSynchronizeUsers(srh);	
			});
			this.syncLink._aLink = this.createLink(ACTION_LINKS["syncJts"], clickHandlerMethod);
			this.syncLink.appendChild(this.syncLink._aLink);
			this.syncLink._progressSpan = dojo.create("span", { className: "syncProgress" }, this.syncLink);
			
			//Check to see if there is a sync already in progress
			this.isSyncInProgress();
		}
	},
	
	handleSyncDetectError: function(err) {
		console.error("Unable to detect if the User Registry supports synchronization with this repository.");
	},
	
	isSyncInProgress: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				if(!this.domNode) return;
				if(response.state == "NOT_STARTED") {
					if(this.syncInterval) {
						//JUST FINISHED
						dojo.removeClass(this.syncLink._aLink, "disabled");
						this.syncLink._progressSpan.innerHTML = "";
						this.header.showMessage({
							type: "OK",
							summary: this.messages.syncFinished,
							closable: true
						});
						clearInterval(this.syncInterval);
						this.syncInterval = null;
					}
				} else if(response.state == "IN_PROGRESS") {
					dojo.addClass(this.syncLink._aLink, "disabled");
					if(!this.syncInterval) {
						//JUST STARTED
						if(response.percentComplete && response.percentComplete > 0) {
							//Just started, but was already running before...(page refresh, or started by another admin)
							this.syncLink._progressSpan.innerHTML = dojo.string.substitute(this.messages.syncRunning, [ response.percentComplete ]);
						} else {
							this.syncLink._progressSpan.innerHTML = this.messages.syncStarted;
						}
						this.syncInterval = setInterval(dojo.hitch(this, "isSyncInProgress"), 5000);
					} else {
						//RUNNING
						this.getUserCount(); //update the user count
						this.syncLink._progressSpan.innerHTML = dojo.string.substitute(this.messages.syncRunning, [ response.percentComplete ]);
					}
				}
			}),
			error: dojo.hitch(this, function(response) {
				console.error("Unable to determine if sync is in progress.");
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		UserRegistryClient.getSynchronizationProgress(srh);	
	},

	handleSyncRequestFailed: function(err) {
		this.header.showMessage({
			errorObject: err,
			summary: this.messages.syncRequestFailed
		});	
	},
	
	checkRegistrySupportedActions: function() {
		var handler = {
			success: dojo.hitch(this, function(userRegistryInfo) {
				if (userRegistryInfo) {
					//Import Users is disabled, update Task Guide
					if (!userRegistryInfo.queryable) {
						dojo.addClass(this.taskEntries["importUsers"], "noDisplay");
					}
					//Create Users is disabled, update Task Guide
					if (!userRegistryInfo.writable && !this.isExternalRegistry) {
						dojo.addClass(this.taskEntries["createUsers"], "noDisplay");	
					}
				}
			}),
			error: function() { 
				console.error("Unable to determine if the User Registry is queryable or writable.");
			}
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		AdminClient.getUserRegistryInfo(srh);
	},

	setUserRoles: function() {
		var roles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		this.isAdmin = false;
		this.isProjectAdmin = false;
		this.isUser = false;
		if (roles) {
			for (var i=0; i < roles.length; i++) {
				if (roles[i] === "JazzAdmins") {
					this.isAdmin = true;
					this.subTitle.innerHTML = this.messages.subTitle_admin;
					break;
				}
				if (roles[i] === "JazzProjectAdmins") {
					this.isProjectAdmin = true;
					this.subTitle.innerHTML = this.messages.subTitle_projectAdmin;
					break;
				}
				if (roles[i] === "JazzUsers") {
					this.isUser = true;
					this.subTitle.innerHTML = this.messages.subTitle_user;
					break;
				}
				if (roles[i] === "JazzGuests") {
					this.isGuest = true;
					this.subTitle.innerHTML = this.messages.subTitle_guest;
					break;
				}
			}
		}
		var srh = new ServiceResponseHandler(this, "setUserRegistryType", "handleUserRegistryError");
		UserRegistryClient.getUserRegistryType(srh);
	},
	
	getSetupStatus: function() {
		var srh = new ServiceResponseHandler(this, "handleSetupStatus", "handleSetupStatusError");
		ServerStatusClient.getServiceConfiguration(srh, SETUP_CONFIGURATION_SERVICE);
	},
	
	handleSetupStatus: function(serviceInfo) {
		if (serviceInfo.properties && serviceInfo.properties.length > 0) {
			dojo.forEach(serviceInfo.properties, function(prop){
				if (prop.name == SETUP_HAS_RUN_PROPERTY) {
					this.hasSetupRun = prop.overrideValue;
					return;
				}
			}, this);
		}
		
		if(this.hasSetupRun) {
			this.taskEntries["setupServer"].className = "completed";
		} else {
			var setupURL = this.JAZZ_ROOT + "/setup";
			if(location.search) {
				setupURL += location.search;
			}
			var msg = dojo.create("div", { innerHTML: messages.noSetupDetected });
			dojo.create("a", { style: { marginLeft: "3px" }, innerHTML: messages.goToSetup, href: setupURL }, msg);
			this.header.showMessage({
				type: "INFO",
				summary: msg
			});
		}
	},
	
	handleSetupStatusError: function(errorObject) {
		console.warn("Could not determine if the Setup Wizard was run.");
	},
	
	/* Help Links have a slightly different structure than action links since they can wrap... */
	createHelpLink: function(link) {
		var aLink = dojo.create("a", { href: BASE_INFOCENTER_URL + link.topic, target: "_blank" });
		dojo.create("span", { className: "spritedIcon aboutLink j-float-leading j-ar-mirror" }, aLink);
		dojo.create("div", { className: "helpLink", innerHTML: link.label }, aLink);
		return aLink;
	},
	
	createLink: function(link, clickHandler) {
		var url = link.contextRoot ? link.contextRoot + link.url : link.url;
		var aLink = dojo.create("a", { href: url || "#", target: link.target || "" });
		dojo.create("span", { className: "spritedIcon j-mirror " + (link.className || "") }, aLink);
		var span = dojo.create("span", { className: "actionLink " + (link.className || ""), innerHTML: link.label || "" }, aLink);
		if(clickHandler) {
			this.connect(span, "onclick", clickHandler);
		}
		return aLink;
	},
	
	fetchServerHealth: function() {	
		var handler = {
			success: dojo.hitch(this, function(data) {
				if(!this.domNode) return;
				dojo.empty(this.serverStatusArea);
	            var summaryDiv = dojo.create("div", {}, this.serverStatusArea);
	            var dataBlock = [{
	                label: this.messages.databaseStatus,
	                value: data.databaseAvailable ? this.messages.connected : this.messages.unavailable,
	                style: data.databaseAvailable ? "" : "error"
	            }];
				if(data.publicURI) {
					var publicNode = dojo.create("a", { href: data.publicURI, innerHTML: data.publicURI });	
				}
				dataBlock.push({
	                label: this.messages.publicURI,
	                value: publicNode || this.messages.notSet
	            });
				this.diagNode = dojo.create("div", { innerHTML: this.messages.loading, className: "loading" });
				dataBlock.push({
	                label: this.messages.diagnosticStatus,
	                value: this.diagNode,
	            });
					
	            this.serverDataTable = new DataTableContainer({
	                heading: { label: this.messages.serverStatus }
	            });
				dojo.forEach(dataBlock, function(item) {
		            var rowData = {
		                0: {
		                    content: item.label,
		                    style: "rank tight double" + (item.labelStyle ? " " + item.labelStyle : "")
		                },
		                1: {
		                    content: item.value,
		                    style: item.style
		                }
		            };
		            this.serverDataTable.addRow(rowData, 2);
				}, this);
				
				dojo.create("div", { className: "stats-block" }, summaryDiv).appendChild(this.serverDataTable.domNode);
				this.fetchDiagnosticStatus();
			}),
			error: dojo.hitch(this, function(errorObj) {
				//TODO do something
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getServerInfo(srh);
	},
	
	fetchDiagnosticStatus: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				if(response && this.domNode) {
					var display = this.diagUtil.displayDiagnosticStatus(response);
					if(this.diagNode && display) {
						dojo.place(display, this.diagNode, "only");
					} else {
						this.unableToDisplayDiagnostics();
					}
				} else {
					this.unableToDisplayDiagnostics();
				}
			}),
			error: dojo.hitch(this, function(response) {
				this.unableToDisplayDiagnostics();
			})
		}
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.getAllDiagnosticResults(srh);
	},
	
	unableToDisplayDiagnostics: function() {
		if(this.diagNode) {
			this.diagNode.innerHTML = this.messages.unavailable;
			var aLink = dojo.create("a", { href:"#", innerHTML: this.messages.tryAgain, className: "aLink" }, this.diagNode);
			this.connect(aLink, "onclick", dojo.hitch(this, function(e) {
				dojo.stopEvent(e);
				this.fetchDiagnosticStatus();
				this.diagNode.innerHTML = this.messages.loading;
			}));
		}
	},

	getAssignedCALs: function() {
		var srh = new ServiceResponseHandler(this, "processCALs", "doNothing");
		LicenseClient.getLicenseInfo(srh);	
	},
	
	processCALs: function (data) {
		this.usersWithCAL = [];
		this.licensePool = [];
		if (data.types) {
			dojo.forEach(data.types, function (license) {
				if(!license.functional) {
					this.licensePool.push(license);
				}
			}, this);
			//only process these one at a time to limit HTTP requests that we might not even need
			if(this.licensePool.length > 0) {
				this.getLicenseUsers(this.licensePool.pop()); //remove it from the licensePool after we process it
			}
		}
	},
	
	getLicenseUsers: function(license) {
		var srh = new ServiceResponseHandler(this, "processLicenseUsers", "handleLicenseUsersError");
		AdminClient.getContributors({searchTerm: license.id, isLicenseSearch: true, pageSize: ASSIGNED_CALS_THRESHOLD, hideGuest: true, hideUnassigned: true }, srh);			
	},
	
	processLicenseUsers: function(cal) {
		if(cal && cal.elements) {
			//Loop through all users that have this CAL and track them
			for(var i = 0; i < cal.elements.length; i++) {
				var userId = cal.elements[i].userId;
				//see if we already have this user in our array
				var inArray = dojo.some(this.usersWithCAL, function(user){ return user == userId });
				if(!inArray) {
					this.usersWithCAL.push(userId);
					if(this.usersWithCAL.length >= ASSIGNED_CALS_THRESHOLD) break; //don't do extra work
				}
			}
		}
		if(this.usersWithCAL.length >= ASSIGNED_CALS_THRESHOLD) {
			this.taskEntries["assignCALS"].className = "successful";
		} else {
			//try another license
			if(this.licensePool.length > 0) {
				this.getLicenseUsers(this.licensePool.pop()); //remove it from the licensePool after we process it
			}
		}
	},
	
	handleLicenseUsersError: function(err) {
		//Failed to get the details about the license, let's try a different license from the pool
		if(this.licensePool.length > 0) {
			this.getLicenseUsers(this.licensePool.pop()); //remove it from the licensePool after we process it
		}
	},
	
	getRdfValue: function (subjectNode, key, isArrayLike) {
		var valueList = [];
		if(subjectNode[key]) {
			dojo.forEach(subjectNode[key], function(tuple) {
				valueList.push(tuple.value);
			});
		} else if(!isArrayLike) {
			valueList.push("");
		}
		return valueList;
	},
	
	getFunctionalUsers: function () {
		this.functionalUsers = [];
		var handler = {
			success: dojo.hitch(this, function(consumers) {
				if (consumers){
					dojo.forEach(consumers, function(consumer){
						if(consumer && consumer.userId) {
							this.functionalUsers.push(consumer.userId);		
						}
		            }, this);
				}
				this.getUserCount();
			}),
			error: dojo.hitch(this, function(errorObject) {
				this.getUserCount();
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		OAuthAdminClient.getConsumers(srh);
	},
	
	uninitialize: function() {
		if(this.serverDataTable) {
			this.serverDataTable.destroyRecursive();
		}
		this.inherited(arguments);
	},
	
	doNothing: function() {} //no-op, for now

});

})();