/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("dojo.string");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamFactory");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamGroup");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParametersPanel");

dojo.requireLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");


(function() {

/*
 * Aliases 
 */
var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var ReportParamFactory = com.ibm.team.reports.web.ui.internal.ReportParamFactory;
var ReportParametersPanel = com.ibm.team.reports.web.ui.internal.ReportParametersPanel;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var REPORTS_REST_SERVICE_URI = "com.ibm.team.reports.common.internal.service.IReportRestService";

/*
 * Constants 
 */
var STYLE_LOADING = "color:#BBBBBB;";

/*
 * Class 
 */
dojo.declare("com.ibm.team.reports.viewlets.web.ui.internal.utils.ReportsPreferenceProvider",
	[dijit._Widget, CustomPreferenceProvider], {

	/*
	 * Properties
	 */	
	context: null,

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ReportsPreferenceProvider
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.viewlets.web.ui.internal", "TrendReportMessages");
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		
		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->
		
		this._reportParams = [];
		this._reportQueryLabelMap = [];
		this._fetchLabelInProgress = false;
		this._customParamsQueryUUID = null;		
	},
	
	/*
	 * @param void
	 * 
	 * @return void
	 * @overrides super
	 */
	destroy: function() {
		if (this._customParamsQueryUUID && this.context._isValidUUID(this._customParamsQueryUUID)) {
			//destroy any temp query descriptor from the custom param panel
			com.ibm.team.reports.viewlets.web.ui.internal.TrendReportViewlet.deleteTransientReportQueryUUID(this._customParamsQueryUUID, this.id);
			delete this._customParamsQueryUUID;
		}
		
		this.inherited(arguments);
	},

	/*
 	 * @param String value
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getLabel: function(value) {
		var queryLabel = this._getReportQueryLabel(value); 
		if (!queryLabel) {
			// Return null to initiate polling of this method by the framework 
			return null;
		}
		return this._buildLabel(queryLabel, null);
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 * @overrides super
 	 */	
	getLoadingLabel: function() {
		return this._buildLabel(this.messages.msgLoading, STYLE_LOADING);
	},
	
	/*
	 * @return a deferred - result is an object with fields parameterValues and parameterNames ([])
	 */
	_getParameterSelections: function() {
		if (!this._customParamsQueryUUID) {
			var deferred = new dojo.Deferred();
			var selections = {
					parameterNames: [],
					parameterValues: []
			};

			for (var i = 0, len = this._reportParams.length; i < len; ++i) {
				var reportParam = this._reportParams[i];
				selections.parameterNames[i] = reportParam.parameterDTO.name;
				var vals = reportParam.getSelectedValues();

				var noSelection;
				if (((vals instanceof Array) === false) ||
						(vals.length === 0) ||
						!vals[0]) {
					noSelection = true;
				} else {
					noSelection = false;
				}

				if (reportParam.parameterDTO.required && noSelection) {
					var errMessage = dojo.string.substitute(this.messages.msgReportParamMissing, [reportParam.parameterDTO.name]);
					this.context._alertErrorMessage( //TODO
							errMessage,
							this.messages.msgReportParamMissing_explanation,
							this.messages.msgReportParamMissing_useraction);
					deferred.errback(errMessage);
					return deferred;
				}
				if (noSelection) {
					selections.parameterValues[i] = "";
				} else {
					selections.parameterValues[i] = vals.join(","); 
				}
			}
			
			deferred.callback(selections);
			return deferred;
		} else {
			//custom parameters; we need to fetch their values out of temp storage on the server instead
			var deferred = new dojo.Deferred();
			var serviceObject = {
				success: function(queryDescriptor) {
					var selections = {
							parameterNames: queryDescriptor.parameterNames,
							parameterValues: queryDescriptor.parameterValues
					};
					deferred.callback(selections);
				},
				
				failure: function(err) {
					//TODO
				}
			};
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			this._callRESTGetQueryDescriptor(srh, this._customParamsQueryUUID);			
			return deferred;
		}
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Object
 	 */		
	getSelectedParameterValues: function() {

		var selections = {};
		
		for (var index = 0; index < this._reportParams.length; index++) {
			var reportParam = this._reportParams[index];
			if (reportParam.parameterDTO.concealed === false) {
				selections[reportParam.parameterDTO.name] = reportParam.getSelectedValues();
			}
		}
		
		return selections;
	},	
	
	_cacheReportQueryLabel: function(queryUUID, queryName) {
		this._reportQueryLabelMap[queryUUID] = queryName;
	},
	
	/*
 	 * @param String queryUUID
 	 * 
 	 * @return String
 	 */	
	getCachedQueryLabel: function(queryUUID) {
		if (queryUUID in this._reportQueryLabelMap) {
			return this._reportQueryLabelMap[queryUUID];
		}
		return null;
	},	
	
	/*
 	 * @param String queryUUID
 	 * 
 	 * @return String
 	 */	
	_getReportQueryLabel: function(queryUUID) {
		
		if (this.context._isValidUUID(queryUUID) === false) {
			// No query has been selected yet
			return this.messages.msgNoQuerySpecified;
        } else if (queryUUID in this._reportQueryLabelMap) {
			// Query label found in cache
			return this._reportQueryLabelMap[queryUUID];
		} else if (this._fetchLabelInProgress === false) {
			// Fetch query label
			this._fetchReportQueryLabel(queryUUID);
		}
		return null;
	},
	
	/*
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_fetchReportQueryLabel: function(queryUUID) {
		
		this._fetchLabelInProgress = true;
		
		var self = this;
		var serviceObject = {
			success: function(queryDescriptorDTO) {
				self._fetchLabelInProgress = false;
				self._cacheReportQueryLabel(queryDescriptorDTO.queryUUID, queryDescriptorDTO.name);
			},
			failure: function(errorObject) {
				self._fetchLabelInProgress = false;
				self._cacheReportQueryLabel(queryUUID, self.messages.msgPreferenceLoadingFailure);
				self.context._displayErrorMessage(
						self.messages.msgReportParameterLoadingFailed, 
						self.messages.msgReportParameterLoadingFailed_explanation, 
						self.messages.msgReportParameterLoadingFailed_useraction, 
						errorObject);
			}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

		this._callRESTGetQueryDescriptor(handler, queryUUID);
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */
	save: function() {
		// Viewlet settings area:  "Save" button pressed
		// NO-OP
	},
		
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_saveQuery: function() {
		
		this._okBtn.disabled = true;
		
		var deferred = this._getParameterSelections();
		deferred.addCallback(dojo.hitch(this, function(selections){
			var self = this;
			var serviceObject = {
				success: function(queryUUID){
					if (self.context._isValidUUID(queryUUID)) {
						self._cacheReportQueryLabel(queryUUID, self._nameField.value);
						self._onPrefOk(queryUUID, self.context._static._PREFERENCE_ID_QUERY_UUID);
						self._reportParametersPanel.closeDialog();
					} else {
						var errorObject = new Error(self.messages.msgInvalidQueryUUID);
						self.context._alertErrorMessage(
								self.messages.msgQuerySaveFailed, 
								self.messages.msgQuerySaveFailed_explanation, 
								self.messages.msgQuerySaveFailed_useraction, 
								errorObject);
						self._okBtn.disabled = false;
					}
				},
				failure: function(errorObject){
					self.context._alertErrorMessage(
							self.messages.msgQuerySaveFailed, 
							self.messages.msgQuerySaveFailed_explanation, 
							self.messages.msgQuerySaveFailed_useraction, 
							errorObject);
					self._okBtn.disabled = false;
				}
			};
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");

			this._callRESTPostSaveQueryDescriptor(
				handler, 
				this._reportID,
				this._projectAreaUUID,
				"", // Create a new query item ID
				selections.parameterNames, 
				selections.parameterValues, 
				this._nameField.value,
				true /* keepInTempStorage */);

		}));
		deferred.addErrback(dojo.hitch(this, function(err){
			var errorObject = new Error(this.messages.msgNoParametersSelected);
			this.context._alertErrorMessage(
					this.messages.msgQuerySaveFailed, 
					this.messages.msgQuerySaveFailed_explanation, 
					this.messages.msgQuerySaveFailed_useraction, 
					errorObject);
			this._okBtn.disabled = false;
		}));
	},		

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_calculateContentMaxHeight: function() {
		
		// Set the maximum height of the content area
		var viewPortHeight = dijit.getViewport().h;
		var maxHeight = Math.floor(viewPortHeight * 0.80);
		if (maxHeight < 360) {
			maxHeight = 360;
		}
		
		this._reportParametersPanel.setContentMaxHeight(maxHeight);
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_createParametersPanel: function() {
		
		this._reportParametersPanel = new ReportParametersPanel({
				nestedStyledBox: true,
				padding: 0
			});
		
		// Preview Button
		this._previewBtn = document.createElement("button");
		this._previewBtn.innerHTML = this.messages.msgPreview;
		this.connect(this._previewBtn, "onclick", this.handleEventPreviewButtonClicked);
		
		// OK Button
		this._okBtn = document.createElement("button");
		this._okBtn.innerHTML = this.messages.msgOK;
		this.connect(this._okBtn, "onclick", this._handleEventOKButtonClicked);
		
		// Cancel Button		
		var cancelBtn = document.createElement("button");
		cancelBtn.innerHTML = this.messages.msgCancel;
		this.connect(cancelBtn, "onclick", this._handleEventCancelButtonClicked);
		
		// Button Site
		var buttonSite = document.createElement("div");
		buttonSite.style.margin = "8px 8px 6px 0";
		buttonSite.appendChild(this._previewBtn);
		buttonSite.appendChild(this._okBtn);
		buttonSite.appendChild(cancelBtn);

		var bottomDiv = document.createElement("div");
		bottomDiv.style.textAlign = "right";
		bottomDiv.style.borderTop = "solid #DDDDDD 1px";
		bottomDiv.appendChild(buttonSite);

		this._reportParametersPanel.setBottomButtonsSection(bottomDiv);
		
		// Create right preview section
		var previewDiv = document.createElement("div");
		previewDiv.style.position = "relative";
		var previewLabelDiv = document.createElement("div");
		this._previewContentDiv = document.createElement("div");
		this._previewContentDiv.className = "sectionContent";
		previewLabelDiv.className = "sectionTitle";
		previewLabelDiv.innerHTML = this.messages.msgPreviewLabel;
		previewDiv.appendChild(previewLabelDiv);
		previewDiv.appendChild(this._previewContentDiv);
		previewDiv.appendChild(this._createClosePreviewButton());
		this._reportParametersPanel.setMainSectionRight(previewDiv);
		
		// Create top name field
		this._nameField = document.createElement("input");
		this._nameField.type = "text";
		this._nameField.size = 70;
		this._nameField.value = this.context._getReportTitle(this._reportID);
		
		this._nameField.style.margin = "10px 0 1px 7px";
		this.connect(this._nameField, "onblur", this.handleEventNameFieldBlurred);
		
		this._reportParametersPanel.setTopInputFieldSection(this._nameField);
		this._reportParametersPanel.focusOnOpen(this._nameField);
		
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventPreviewButtonClicked: function(event) {
		dojo.stopEvent(event);
		
		// Show preview pane
		this.showPreviewPane(true);
		// Stop subsequent clicks
		this._previewBtn.disabled = true;
		// Clear previous 'preview'
		this._previewContentDiv.innerHTML = "";
		// Scroll to the top of the content area
		this._reportParametersPanel.scrollToTopContent();
			
		var loadingDiv = document.createElement("div");
		loadingDiv.className = "progress-icon";
		this._previewContentDiv.appendChild(loadingDiv);
			
		var deferred = this._getParameterSelections();
		deferred.addCallback(dojo.hitch(this, function(selections){
			var self = this;
			var serviceObject = {
				success: function(chartHTML){
					// TODO if cancelled ...
					self._previewBtn.disabled = false;
					self._previewContentDiv.innerHTML = chartHTML;
				},
				failure: function(errorObject){
					self._previewBtn.disabled = false;
					self._previewContentDiv.innerHTML = "";
					self.context._displayErrorMessage(
							self.messages.msgReportParameterLoadingFailed, 
							self.messages.msgReportParameterLoadingFailed_explanation, 
							self.messages.msgReportParameterLoadingFailed_useraction, 
							errorObject);
				}
			};
			
			var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
			this._callRESTPostRenderReport(
				handler, 
				this._reportID, 
				this._projectAreaUUID,
				selections.parameterNames, 
				selections.parameterValues);
			
			return true;

		}));
		deferred.addErrback(dojo.hitch(this, function(err){
			// Missing required parameter
			this._previewBtn.disabled = false;
			this._previewContentDiv.innerHTML = "";
			return true;
		}));	
	},
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventNameFieldBlurred: function(event) {
		dojo.stopEvent(event);
		
		if (!this._nameField.value) {
			this._nameField.value = this.context._getReportTitle(this._reportID);
		}
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventOKButtonClicked: function(event) {
		dojo.stopEvent(event);
		
		this._saveQuery();
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventCancelButtonClicked: function(event) {
		dojo.stopEvent(event);

		this._reportParametersPanel.closeDialog();
		
		if (this._onPrefCancel) {
			this._onPrefCancel();
		}
				
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */	
	_handleEventDialogClosed: function() {
		// Events:
		// OK button clicked
		// Cancel button clicked
		// X button clicked
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return Element
 	 */
	_createClosePreviewButton: function() {
		
		this._closeButton = document.createElement("img");
		this._closeButton.alt = "X";
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/images/close.gif");
		this._closeButton.style.cursor = "pointer";
		
		this.connect(this._closeButton, "onclick", this.handleEventClosePreviewClicked);		
		this.connect(this._closeButton, "onmouseover", this.handleEventClosePreviewMouseOver);
		this.connect(this._closeButton, "onmouseout", this.handleEventClosePreviewMouseOut);
		
		var closeButtonSite = document.createElement("div");
		closeButtonSite.style.position = "absolute";
		closeButtonSite.style.right = "5px";
		closeButtonSite.style.top = 0;

		closeButtonSite.appendChild(this._closeButton);
		
		return closeButtonSite;
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventClosePreviewClicked: function(event) {
		dojo.stopEvent(event);
		
		// Hide preview pane
		this.showPreviewPane(false);
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventClosePreviewMouseOver: function(event) {
		dojo.stopEvent(event);
		
		// Highlight on
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/images/close-hov.gif");		
		
		return true;
	},

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */	
	handleEventClosePreviewMouseOut: function(event) {
		dojo.stopEvent(event);
		
		// Highlight off
		this._closeButton.src = dojo.moduleUrl("com.ibm.team.reports.viewlets.web", "graphics/shared/images/close.gif");		
		
		return true;
	},

	/*
	 * @param Boolean show
	 * 
	 * @return void
	 */			
	showPreviewPane: function(show) {
		this._reportParametersPanel.showPreviewPane(show);
	},

	/*
	 * @param Function onOk(String value, String prefId)
	 * @param Object settings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
	 * @param Object viewlet
	 * 
	 * @return void
 	 * @overrides super
 	 */		
	createChooser: function(onOk, settings, viewlet) {
		
		this._onPrefOk = onOk;
		this._projectAreaUUID = settings.scopeItemId;
		this._reportID = settings.preferences.report;
		this._reportParams = [];
		this._customParamsQueryUUID = null;
		
		// TODO: MP: No access to project area name
		
		if (!this._projectAreaUUID) {
			// No project area selected
			alert(this.messages.msgSelectProject);
			return;
		} else if (!this._reportID) {
			// No report selected
			alert(this.messages.msgSelectReport);
			return;			
		}
		
		// Check to see if the report is deployed to the project area
		var deferred = viewlet._isReportDeployed(this._reportID, this._projectAreaUUID);
		
		deferred.addCallback(dojo.hitch(this, function(deployed) {
			if (deployed) {
				this._createReportParametersDialog(true);				
			} else {
				// Report not deployed
				var message = dojo.string.substitute(this.messages.msgReportNotDeployed2, 
						[this.context._getReportTitle(this._reportID), this._reportID]);
				viewlet._alertErrorMessage(
						message, 
						this.messages.msgReportNotDeployed2_explanation, 
						this.messages.msgReportNotDeployed2_useraction);
			}
			return deployed;
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			viewlet._alertErrorMessage(
					this.messages.msgWhitelistError, 
					this.messages.msgWhitelistError_explanation, 
					this.messages.msgWhitelistError_useraction, 
					errorObject);
			return errorObject;
		}));		

	},	
	
	/*
	 * Creates the chooser widget and returns the content as a DOM node
	 * 
	 * When a selection is made in the chooser it should call the function passed in
	 * the onOk parameter with the new preference value as the first parameter and
	 * the preference id as the second parameter (e.g. onOk(value, prefId)).
	 * 
	 * When the cancel button is clicked, the onCancel function should be called and
	 * all changes will be discarded.
	 * 
	 * @param Function onOk(String value, String prefId)
	 * @param Object settings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
	 * @param Object viewlet
	 * @param Function onCancel
	 * @return void
	 */
	createDOMChooser: function(onOk, settings, viewlet, onCancel) {
		this._onPrefOk = onOk;
		this._onPrefCancel = onCancel;
		this._projectAreaUUID = settings.scopeItemId;
		this._reportID = settings.preferences.report;
		
		this._reportParams = [];
		this._customParamsQueryUUID = null;
		
		// TODO: MP: No access to project area name
		
		if (!this._projectAreaUUID) {
			// No project area selected
			return dojo.create("div", { innerHTML: "<p>" + this.messages.msgSelectProject + "</p>" });
		} else if (!this._reportID) {
			// No report selected
			return dojo.create("div", { innerHTML: "<p>" + this.messages.msgSelectReport + "</p>" });
		}
		
		// Check to see if the report is deployed to the project area
		var deferred = viewlet._isReportDeployed(this._reportID, this._projectAreaUUID);
		
		deferred.addCallback(dojo.hitch(this, function(deployed) {
			if (deployed) {
				this._createReportParametersDialog(false);				
			} else {
				// Report not deployed
				var message = dojo.string.substitute(this.messages.msgReportNotDeployed2, 
						[this.context._getReportTitle(this._reportID), this._reportID]);
				return dojo.create("div", { innerHTML: "<p>" + message + "</p>" });
			}
			return deployed;
		}));
		deferred.addErrback(dojo.hitch(this, function(errorObject) {
			viewlet._alertErrorMessage(this.messages.msgWhitelistError, errorObject);
			return errorObject;
		}));
		
		return this._reportParametersPanel.domNode;
	},

	/*
	 * @param initDialog
	 * 
	 * @return void
 	 */		
	_createReportParametersDialog: function(initDialog) {		
		this._createParametersPanel();
		if (initDialog) 
			this._reportParametersPanel.showAsDialog(dojo.hitch(this, this._handleEventDialogClosed));

		// get the report descriptor to check what type of params this report uses
		var self = this;
		var serviceObject = {
				success : function(reportDescriptor) {
					var customParams = reportDescriptor.engine.hasCustomParams;
					if (customParams) {
						var serviceObject = {
								success: function(customParams) {
									//string from the custom HTML.  Display it in the panel
									if (customParams && customParams.html && customParams.html != "") {
										self._reportParametersPanel.insertParamHTML(customParams.html);
										self._customParamsQueryUUID = customParams.tempQueryUUID;
									}
								},
								
								failure: function(error) {
									self.context._displayErrorMessage(
											self.messages.msgReportLoadingFailed, 
											self.messages.msgReportLoadingFailed_explanation, 
											self.messages.msgReportLoadingFailed_useraction, 
											errorObject);
								}					
						};
						var curQueryUUID = self.context._getReportQueryUUID();
						var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
						if (self.context._isValidUUID(curQueryUUID)) {
							//if there is a currently saved query, pass it to the method so that the saved values will be selected
							self._callRESTGetCustomParam(handler, self._reportID, self._projectAreaUUID, curQueryUUID);
						} else {
							self._callRESTGetCustomParam(handler, self._reportID, self._projectAreaUUID);
						}
					} else {
						// Create the service object
						var paramsServiceObject = {
								// Service success
								success: function(baseParamsArray){
									dojo.forEach(baseParamsArray, dojo.hitch(self, function(baseParam){

										if (baseParam._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
											var reportParamGroup = new com.ibm.team.reports.web.ui.internal.ReportParamGroup({
												parameterGroupDTO: baseParam,
												parameterName: self.parameterName,
												parameterValue: self.parameterValue,
												urlPrefix: "../../",
												nestedStyledBox: true
											});
											self._reportParametersPanel.appendParamGroup(reportParamGroup);
											reportParamGroup.appendReportParameters(self._reportParams);

											// Create the service object
											var serviceObject = {
													// Service success
													success: function(reportParameterGroupDTO){
														reportParamGroup.initGroupValues(reportParameterGroupDTO);
													},
													// Service failure
													failure: function(errorObject){
														self.context._displayErrorMessage(
																self.messages.msgReportParameterLoadingFailed, 
																self.messages.msgReportParameterLoadingFailed_explanation, 
																self.messages.msgReportParameterLoadingFailed_useraction, 
																errorObject);
													}
											};

											var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
											self._callRESTGetParameterGroupValues(handler, self._reportID, self._projectAreaUUID, baseParam.name);
										}
										else 
											if (baseParam._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterDTO") {
												var reportParam = ReportParamFactory.create(baseParam, "../../");
												if (!baseParam.concealed) {
													reportParam.isRoot = true;
													self._reportParametersPanel.appendParam(reportParam);
												}
												self._reportParams.push(reportParam);

												// Create the service object
												var serviceObject2 = {
														// Service success
														success: function(reportParameterDTO){
															reportParam.init(reportParameterDTO);
															if (self.parameterName && self.parameterName.length > 0) {
																for (var i = 0; i < self.parameterName.length; ++i) {
																	if (self.parameterName[i] == reportParam.parameterDTO.name) {
																		reportParam.setValues(self.parameterValue[i]);
																		break;
																	}
																}
															}
															reportParam.postInit();
														},
														// Service failure
														failure: function(errorObject){
															self.context._displayErrorMessage(
																	self.messages.msgReportParameterLoadingFailed, 
																	self.messages.msgReportParameterLoadingFailed_explanation, 
																	self.messages.msgReportParameterLoadingFailed_useraction, 
																	errorObject);
														}
												};

												var handler2 = new ServiceResponseHandler(serviceObject2, "success", "failure");
												self._callRESTGetParameterValues(handler2, self._reportID, self._projectAreaUUID, baseParam.name);
											}
									}));
									// Set content area max height
									self._calculateContentMaxHeight();
									// Reposition the dialog to be vertically centered on the screen 
									// after the parameter boxes have loaded
									if (initDialog)
										self._reportParametersPanel._dialog.updatePosition();
								},
								// Service failure
								failure: function(errorObject){
									/* TODO update loading message */
									self.context._displayErrorMessage(
											self.messages.msgReportLoadingFailed, 
											self.messages.msgReportLoadingFailed_explanation, 
											self.messages.msgReportLoadingFailed_useraction, 
											errorObject);
								}
						};

						var curQueryUUID = self.context._getReportQueryUUID();
						if (self.context._isValidUUID(curQueryUUID)) {
							// We have an existing query uuid so we must fetch its associated parameters to
							// select them in the dialog
							var serviceObject3 = {
									success: function(queryDescriptor) {
										self.parameterName = queryDescriptor.parameterNames;
										self.parameterValue = queryDescriptor.parameterValues;
										// Create the handler
										var handler = new ServiceResponseHandler(paramsServiceObject, "success", "failure");
										// Invoke the service
										self._callRESTGetParameters(handler, self._reportID, self._projectAreaUUID);

									},
									failure: function(errorObject) {
										self.context._displayErrorMessage(
												self.messages.msgReportParameterLoadingFailed, 
												self.messages.msgReportParameterLoadingFailed_explanation, 
												self.messages.msgReportParameterLoadingFailed_useraction, 
												errorObject);
									}
							};
							var handler = new ServiceResponseHandler(serviceObject3, "success", "failure");
							self._callRESTGetQueryDescriptor(handler, curQueryUUID);
						} else {
							// Create the handler
							var handler3 = new ServiceResponseHandler(paramsServiceObject, "success", "failure");
							// Invoke the service
							self._callRESTGetParameters(handler3, self._reportID, self._projectAreaUUID);
						}
					}		
				},

				failure : function(errObject) {
					self.context._displayErrorMessage(
							self.messages.msgReportParameterLoadingFailed, 
							self.messages.msgReportParameterLoadingFailed_explanation, 
							self.messages.msgReportParameterLoadingFailed_useraction, 
							errorObject);
				}
		};
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		this._callRESTGetReportDescriptor(handler, this._reportID, this._projectAreaUUID);
	},		

	/*
 	 * @param String text
 	 * @param String style
 	 * 
 	 * @return String
 	 */	
	_buildLabel: function (text, style) {
		var html = "<input type='text' readonly='readonly' value='" + 
					text.replace("'","&#39;") + 
					"'";
		if (style) {
			html = html + 
					" style='" +
					style +
					"'"; 
		}
		html = html + "/>"; 
		
		return html;
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTPostDeleteQueryDescriptor: function(handler, queryUUID) {
		var arguments = {
			queryUUID: queryUUID,
			onlyIfInTempStorage: true
		};
		this._callRESTReportsService(handler, "postDeleteQueryDescriptor", arguments);
	},

	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String queryUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * @param String queryName
 	 * @param Boolean keepInTempStorage
 	 * 
 	 * @return void
 	 */	
	_callRESTPostSaveQueryDescriptor: function(handler, reportUUID, projectAreaUUID, queryUUID, parameterNames, parameterValues, queryName, keepInTempStorage) {
		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID,
			queryUUID: queryUUID,
			parameterName: parameterNames,
			parameterValue: parameterValues,
			name: queryName,
			keepInTempStorage: keepInTempStorage,
			sharedIfNullFolder: true
		};
		this._callRESTReportsService(handler, "postSaveQueryDescriptor", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTGetParameters: function(handler, reportUUID, projectAreaUUID) {
		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID
		};
		this._callRESTReportsService(handler, "getParameters", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String queryUUID
 	 * 
 	 * @return void
 	 */	
	_callRESTGetQueryDescriptor: function(handler, queryUUID) {
		var arguments = {
			queryUUID: queryUUID
		};
		this._callRESTReportsService(handler, "getQueryDescriptor", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String parameterName
 	 *  
 	 * @return void
 	 */	
	_callRESTGetParameterValues: function(handler, reportUUID, projectAreaUUID, parameterName) {
		var arguments = {
			reportUUID: reportUUID,
			parameter: parameterName,
			showArchived: this.context._showArchived,
			projectAreaUUID: projectAreaUUID
		};
		this._callRESTReportsService(handler, "getParameterValues", arguments);
	},		
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param String groupName
 	 *  
 	 * @return void
 	 */	
	_callRESTGetParameterGroupValues: function(handler, reportUUID, projectAreaUUID, groupName) {
		var arguments = {
			reportUUID: reportUUID,
			groupName: groupName,
			showArchived: this.context._showArchived,
			projectAreaUUID: projectAreaUUID
		};
		this._callRESTReportsService(handler, "getParameterGroupValues", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String reportUUID
 	 * @param String projectAreaUUID
 	 * @param Array parameterNames
 	 * @param Array parameterValues
 	 * 
 	 * @return void
 	 */	
 	_callRESTPostRenderReport: function(handler, reportUUID, projectAreaUUID, parameterNames, parameterValues) {
		var arguments = {
			projectAreaUUID: projectAreaUUID,
			reportUUID: reportUUID,
			parameterName: parameterNames,
			parameterValue: parameterValues,
			showArchived: this.context._showArchived,
			urlPrefix: this.context._getImageServerRoot(),
			serverURL: this.context._getDrillInServerRoot()
		};
		// TODO: MP: Disable drill-in link for preview images 
		// BUG: Drill-in link for remote viewlets can get out of sync.
		// e.g. User selects a new project area, opens the report parameter
		// dialog and previews (old project area is used for the project area
		// name)
		this._callRESTReportsService(handler, "postRenderReport", arguments);
	},
	
	/*
	 * queryUUID: optional
	 */
	_callRESTGetCustomParam: function(handler, reportUUID, projectAreaUUID, queryUUID) {
		var arguments;
		
		if (queryUUID) {
			arguments = {
					projectAreaUUID : projectAreaUUID,
					reportUUID : reportUUID,
					queryUUID : queryUUID
			};
		} else {
			arguments = {
					projectAreaUUID : projectAreaUUID,
					reportUUID : reportUUID
			};
		}
		this._callRESTReportsService(handler, "getCustomParams", arguments);
	},
	
	_callRESTGetReportDescriptor: function(handler, reportUUID, projectAreaUUID) {
		var arguments = {
				projectAreaUUID : projectAreaUUID,
				reportUUID : reportUUID
		};
		this._callRESTReportsService(handler, "getReportDescriptor", arguments);
	},
	
	/*
 	 * @param ServiceResponseHandler handler
 	 * @param String method
 	 * @param Object arguments
 	 * 
 	 * @return void
 	 */	
	_callRESTReportsService: function(handler, method, arguments) {
		// Create the service request
		var request = new ServiceRequest(
					REPORTS_REST_SERVICE_URI,
					method,
					arguments);
		this.context.getServiceTracker().register(request, handler);
		// Invoke the service
		TeamServerClient.invokeService(request, handler);
	}
	
});

})();
