/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.MessageBox");
dojo.require("net.jazz.ajax.contextRoot");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigurePublicURIMessages");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var REMOTE_SERVICE_NAME = "com.ibm.team.repository.service.internal.BaseServerDescriptionService";
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var messages;
var PUBLIC_URI_PROPERTY = "server.webapp.url";
var planningLink = "<a href='http://pic.dhe.ibm.com/infocenter/clmhelp/v4r0m5/topic/com.ibm.jazz.install.doc/topics/c_planning_URLs.html' target='_blank'>";
var STATUS_WARN_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/images/warn_obj.gif");
var DISPLAY_PROPS = [
	PUBLIC_URI_PROPERTY
];

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI",  WizardConfigurationStep, {

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigurePublicURIMessages");
		this.implementationClassName = REMOTE_SERVICE_NAME;
		this.stepTitle = messages.wizardPageHeading;
		
		this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
		this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
		
		this.description = this.buildDescription();	
		this.innerArgs = {headingLabel: messages.step1Label, heading: messages.step1Title};	
		
		//Our CONNECTED_WARN state should allow the user to go next.
		this.STATUS.CONNECTED_WARN = {test: true, revert: true, next: true, edit: true, code: "WARNING", message: 'connectedWarn'};
		
		//Custom status states based off of PENDING_OK but w/ custom messages.
		this.STATUS.CUSTOM_NEEDS_AGREEMENT = {test: true, revert: true, next: false, edit: true, code: "INFO", resolvedMessage: this.messages.okNeedsAgreement };
		this.STATUS.CUSTOM_INTRO_MESSAGE = {test: true, revert: true, next: false, edit: true, code: "INFO", resolvedMessage: this.messages.uriComputedAutomatically, hideBottomStatus: true };
		this.STATUS.CUSTOM_PREFILL_FAILED = {test: true, revert: true, next: false, edit: true, code: "WARNING", resolvedMessage: this.messages.prefillInvalid, hideBottomStatus: true };
		this.STATUS.CUSTOM_DISABLED = {test: false, revert: false, next: true, edit: false, code: "OK", resolvedMessage: this.messages.alreadySet };
	},
	
	buildDescription: function() {
		var description = dojo.create("div");
		dojo.create("p", { innerHTML: this.messages.wizardPageDescription1 }, description);
		dojo.create("p", { innerHTML: dojo.string.substitute(this.messages.wizardPageDescription2, [planningLink, "</a>"])  }, description);
		var p4 = dojo.create("p", {}, description);
		p4.appendChild(document.createTextNode(this.messages.wizardPageDescription4));
		return description;
	},
	
	focusStep: function() {
		this.inherited(arguments);
		if(!this.isJTS && !this.isEmbedded && this.jtsURL === undefined) {
			var self = this;
			//Need this message to display AFTER all of the other pending/loading messages
			setTimeout(function() {
				var srh = new ServiceResponseHandler(self, "displayJTSInfo", "handleErrorFetchingURL");
				ServerStatusClient.getConfiguredJazzTeamServerURI(srh);	
			}, 300);
		}
		
		//Disable the public uri step if it's already set in the db		
		if (this.originalURL) {
			this._setState(this.STATUS.CUSTOM_DISABLED);
			this.publicURLConfigUI.btnAgree.disabled = true;
			this.publicURLConfigUI.hideMessage();
		}
	},
	
	/* Used by Express Setup */
	customOnPrevious: function() {
		if(this.backURL && this.position === 1) {
			location.href = this.backURL;
		}
	},

	createInnerStepUI: function () {
		this.publicURLConfigUI = new ConfigurePublicURIInnerSteps();
		this.serviceContainer = this.publicURLConfigUI._hiddenTableBody;
		this.configArea.appendChild(this.publicURLConfigUI.domNode);
		this.connect(this.publicURLConfigUI.btnAgree, "onclick", "onCheckboxClicked");
	},
	
	startup: function() {
		this.inherited(arguments);
		if(this.publicURLConfigUI) {
			this.publicURLConfigUI.startup();
		}
	},
	
	onCheckboxClicked: function() {
		this.hasAgreed = this.publicURLConfigUI.btnAgree.checked === true;
		if(this.hasAgreed) {
			this.publicURLConfigUI.hideMessage();
		} else {
			this.publicURLConfigUI.showMessage();
		}
		this._testConnectionCallback(this.lastConnectionCallbackState);
	},
	
	displayJTSInfo: function (response) {
		this.jtsURL = response;
		if(!this.jtsURL) {
			//Instruct the user to visit the JTS Setup Wizard to register this application.
			this.header.showMessage({
                summary: this.messages.appNotRegistered,
				type: "ERROR"
            });
		}
	},
	
	handleErrorFetchingURL: function(error) {
		//do nothing
	},
	
	onNext: function() {
		this.invokeSave(this, "processNext", "saveError");
	},
	
	// override WizardConfigurationStep#processNext in order to make sure the
	// browser URL matches the public URI root, and redirect if necessary
	processNext: function() {
		this._initialLoadedState = this.STATUS.CONNECTED_OK;
		if (this.pending){
			var pendingProperties = this.pending.properties;
			for (var i = 0; i < pendingProperties.length; i++) {
				if (pendingProperties[i]._restartRequired) {
					serverRestart = true;
				}
				pendingProperties[i]._handleSuccessfulChange();
			}
		}
		
		//Disconnect our normalizing onblur if it exists
		if(this.normalizeAction) this.disconnect(this.normalizeAction);
		
		if (!this.originalURL) {
			this._setState(this.STATUS.CONNECTED_OK);
		}
		
		this.pending = null;
		if (!this.isEmbedded && this.publicUriPropertyElement) {
			var publicUri = this.publicUriPropertyElement._getChange();
			if (publicUri && document.location.href.indexOf(publicUri.value) !== 0) {
				var redirectUrl = publicUri.value;
				var lastChar = redirectUrl.charAt(redirectUrl.length-1);
  				if(lastChar != '/') redirectUrl += "/";
				//Make sure we remain in the same setup wizard (custom or express) after redirect
				if(document.location.pathname) {
					var s = document.location.pathname.split('/'); // example pathname: /jazz/customSetup
				}
				redirectUrl += (s && s[2]) ? s[2] : "setup";
				if (document.location.search) {
					redirectUrl += document.location.search;
				}
				redirectUrl += "#/steps/" + (this.wizard.selectedStepPosition + 1);
				document.location.href = redirectUrl;
			} else {
				this.triggerButtonStateUpdate();
				this.wizard.processNext();
			}
		} else {
			this.triggerButtonStateUpdate();
			this.wizard.processNext();
		}
	},

	//Custom override in order to remove a lot of the test connection wiring.
	//To include test connection code again, it's probably best to remove these 2 functions.
	_initializePropertiesAfterLoaded: function () {
		this.connect(this._config, "_markInvalid", "_handleInvalidInput");		
		this._initializePropertyDisplay(this.publicURLConfigUI._configTable, DISPLAY_PROPS);
		this.connect(this._config, "_markChanged", function() {
			this._testConnection();
		});		
		this.inherited(arguments);
	},
	
	_initializePropertyDisplay: function (attachPoint, displayProps) {		
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties publicURI"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
		
		var longExists = false;
		var shortExists = false;
		for (var i=0; i < displayProps.length; i++) {
			var propertyElement = this._config.getServerPropertyElement(displayProps[i]);
			if(displayProps[i] == "server.webapp.url") {
				//Override the display name to drop the "Root" part.
				propertyElement._propertyName.innerHTML = this.messages.publicURI;
				propertyElement._propertyExample.innerHTML = dojo.string.substitute(this.messages.exampleURI, [ net.jazz.ajax.contextRoot() ]);
				var inputNode = propertyElement._editValueNode;
				this.originalURL = propertyElement._savedValue;
				if(!this.originalURL) {
					//Pre-fill based on the request URI
					this.suggestionPending = true;
					var url = new dojo._Url(self.location.href);
					if(url && url.scheme) {
						var newUrl = url.scheme.toLowerCase() + "://";
						newUrl += url.host.toLowerCase();
						newUrl += url.port ? ":" + url.port : "";
						inputNode.value = newUrl + net.jazz.ajax.contextRoot();
						propertyElement._handleValueChange();
					}
				} else {
					this.publicURLConfigUI.btnAgree.checked = true;
					this.hasAgreed = true;	
				}
				this.normalizeAction = this.connect(inputNode, "onblur", function(e) {
					dojo.stopEvent(e);
					var url = new dojo._Url(inputNode.value);
					if(url && url.scheme) {
						var newUrl = url.scheme.toLowerCase() + "://";
						newUrl += url.host.toLowerCase();
						newUrl += url.port ? ":" + url.port : "";
						inputNode.value = newUrl + url.path;
					}
				});
				//Get a handle on this property element since we need to make sure it's not empty.
				this.publicUriPropertyElement = propertyElement;
			}
			if (propertyElement.elementInfo.propertyType === "STRING") {
				longExists = true;
				var table = targetPropertyServiceRow._longPropertiesTableBody;
			} else {
				shortExists = true;
				var table = targetPropertyServiceRow._shortPropertiesTableBody;
			}
			table.appendChild(propertyElement.domNode);
			
			//Don't show the Public URI description for default value
			if(propertyElement._propertyDescription) {
				dojo.addClass(propertyElement._propertyDescription, "noDisplay");
				dojo.addClass(propertyElement._propertyDefaultSection, "noDisplay");
			}
		};
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
	
	_testConnectionCallback: function(msg) {
		this.lastConnectionCallbackState = msg;
		var connected = null;
		if(msg) {
			connected = this.STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = this.STATUS.CONNECTED_ERR;
				}
			}
			
			//See if this warning/error is a result of the pre-filling
			if(this.suggestionPending) {
				var suggestionInvalid = true;
			}
			
		} else {
			if(this.hasAgreed) {
				connected = this.STATUS.CONNECTED_OK;
				connected.resolvedMessage = null;
			} else {
				connected = this.STATUS.CUSTOM_NEEDS_AGREEMENT;
			}
			if (this.suggestionPending) {
				this.suggestionPending = false;
			}
		}
		
		if (this._initialLoadedState == null) {
			if(this.originalURL) {
				this._initialLoadedState = connected = this.STATUS.CUSTOM_DISABLED;
				msg = []; //clear initial warnings/errors
				this.publicURLConfigUI.btnAgree.disabled = true;
				this.publicURLConfigUI.hideMessage();
			} else if(suggestionInvalid) {
				this._initialLoadedState = connected = this.STATUS.CUSTOM_PREFILL_FAILED;
				this.publicUriPropertyElement._warnIcon.style.display = "inline";
				this.publicUriPropertyElement._errIcon.style.display = "none";
			} else {
				this._initialLoadedState = connected = this.STATUS.CUSTOM_INTRO_MESSAGE;
				msg = []; //clear initial warnings/errors
			}
		} else {
			if(this._checkIsEmpty() === true) {
				connected = this.STATUS.CONNECTED_ERR;
				var msg = new Array();
				msg[0] = {
					code: "ERROR", 
					message: this.messages.urlCannotBeEmpty,
					name: PUBLIC_URI_PROPERTY,
					implementationClassName: this.implementationClassName,
					id: null
				}; 
			}
		}
		this._setState(connected, msg);
	},
	
	canFlipToNextPage: function () {
		return this.enableNext && this.hasAgreed;
	},
	
	_checkIsEmpty: function() {
		if(!this.publicUriPropertyElement) return true;
		var newValue = this.publicUriPropertyElement._getChange();
		if(newValue && newValue.value) {
			return newValue.value === "";
		} else {
			return true;
		}
	}
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURIInnerSteps", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ConfigurePublicURI.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigurePublicURIMessages");
	},
	
	postCreate: function() {

		this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
		this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";

		this.warningBox = new jazz.ui.MessageBox();
		this.warningInfo.appendChild(this.warningBox.domNode);
		this.showMessage();
	},
	
	hideMessage: function() {
		this.warningBox.hideMessage();
	},
	
	showMessage: function() {
		this.warningBox.showMessage({
			type: "WARNING",
			summary: dojo.string.substitute(this.messages.step2warningText, [planningLink, "</a>"])
		});
	}
	
});

var ConfigurePublicURIInnerSteps = com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURIInnerSteps;


})();